/*********************************************************
 * AcqConstraint class (Acquisition constraints) 
 * 
 * 
 * @author LAZAAR
 * @date 2016-11-29
 ********************************************************/


package acqConstraints;


import java.util.*;

import vocabulary.AcqVariable;
import config.Cst_type;



public class AcqConstraint {

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				ATTRIBUTEs
	///
	////////////////////////////////////////////////////	
	
	
	protected List<AcqVariable> variables;
	protected String name;
	protected int arity;
	private AcqConstraint reified;





	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				CONSTRUCTORs
	///
	////////////////////////////////////////////////////

	public AcqConstraint(String name) {
		this.name=name;
		this.variables= new ArrayList<AcqVariable>(); 
	}


	public AcqConstraint(AcqConstraint c, boolean reified) {

		this.variables=new ArrayList<AcqVariable>(c.getVariables());
		this.constants= new ArrayList<Integer>(c.constants);
		this.arity=c.arity;
		if(reified)
		{
			this.type=Cst_type.Reified;
			this.arity++;
		}
		else
			this.type=c.type;


	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				METHODs
	///
	////////////////////////////////////////////////////




	
	public AcqConstraint getNegation() {
		// TODO Auto-generated method stub
		return null;
	}


	/************************************************
	 * constraint Checker 
	 * is a method that checks if an instantiation
	 * satisfies or not a given constraint
	 *  
	 * @param values  (ground instance)
	 * @return	(sat/unsat)
	 * @date 2016-12-13
	 * @author LAZAAR
	 *************************************************/

	public boolean checker(int[] values) {


		// NL: if the constraint arity is greater than the 
		// given set of values => CST is sat! 

		if(this.arity>values.length)
			return true;

		switch (this.type) {

		case EqualX: 
			return values[0] == this.getConstant(0);
		case DiffX: 
			return values[0] != this.getConstant(0);
		case DifferentXY: 
			return values[0] != values[1];
		case EqualXY: 
			return values[0] == values[1];
		case GreaterOrEqualXY: 
			return values[0] >= values[1];
		case GreaterXY: 
			return values[0] > values[1];
		case LessOrEqualXY: 
			return values[0] <= values[1];
		case LessXY: 
			return values[0] < values[1]; 

		case Next: 
			return values[0] == values[1]+1;
		case NNext: 
			return values[0] != values[1]+1;
		case Prev: 
			return values[0] == values[1]-1;
		case NPrev: 
			return values[0] != values[1]-1;

		case Neighbour: 
			return values[0] == values[1]-1;
		case NotNeighbour: 
			return values[0] != values[1]-1;
		case Neighbour2: 
			return values[0] == values[1]-2;
		case NotNeighbour2: 
			return values[0] != values[1]-2;
		case Neighbour3: 
			return values[0] == values[1]-3;
		case NotNeighbour3: 
			return values[0] != values[1]-3;
		case Neighbour4: 
			return values[0] == values[1]-4;
		case NotNeighbour4: 
			return values[0] != values[1]-4;

		case At1: 
			return values[0] == 1;
		case At2: 
			return values[0] == 2;
		case At3: 
			return values[0] == 3;
		case At4: 
			return values[0] == 4;
		case At5: 
			return values[0] == 5;
		case notAt1: 
			return values[0] != 1;
		case notAt2: 
			return values[0] != 2;
		case notAt3: 
			return values[0] != 3;
		case notAt4: 
			return values[0] != 4;
		case notAt5: 
			return values[0] != 5;


		case OutDiag1: 
			return values[0]+this.getConstant(0) != values[1]+this.getConstant(1);
		case OutDiag2: 
			return values[0]-this.getConstant(0) != values[1]-this.getConstant(1);
		case InDiag1: 
			return values[0]+this.getConstant(0) == values[1]+this.getConstant(1);
		case InDiag2: 
			return values[0]-this.getConstant(0) == values[1]-this.getConstant(1);

		case AbsOutDiag:
			return Math.abs(values[0] - values[1]) != this.getConstant(0);
		case AbsInDiag:
			return Math.abs(values[0] - values[1]) == this.getConstant(0);

		case AllDiff:
			for (int i = 0; i < values.length-1; i++) {
				for (int j = i+1; j < values.length; j++) {
					if (values[i] == values[j]) return false;
				}
			}
			return true;  

			//Allen
		case Before: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]<values[2]);
		case BeforeInv: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]<values[0]);
		case During: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]<values[1]) && (values[2]>values[0]);
		case DuringInv: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]<values[3]) && (values[0]>values[2]);
		case Equals: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]==values[2]) && (values[1]==values[3]);
		case Finish: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]>values[2]) && (values[1]==values[3]);
		case FinishInv: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]>values[2]) && (values[1]==values[3]);
		case Meet: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]==values[2]);
		case MeetInv: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]==values[0]);
		case Overlap: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]>values[2]) && (values[3]>values[1]);
		case OverlapInv: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]>values[0]) && (values[1]>values[3]);
		case Start: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]==values[2]) && (values[3]>values[1]);
		case StartInv: 
			return (values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]==values[2]) && (values[1]>values[3]);


		case NotBefore: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]<values[2]));
		case NotBeforeInv: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]<values[0]));
		case NotDuring: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]<values[1]) && (values[2]>values[0]));
		case NotDuringInv: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]<values[3]) && (values[0]>values[2]));
		case NotEquals: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]==values[2]) && (values[1]==values[3]));
		case NotFinish: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]>values[2]) && (values[1]==values[3]));
		case NotFinishInv: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]>values[2]) && (values[1]==values[3]));
		case NotMeet: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]==values[2]));
		case NotMeetInv: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]==values[0]));
		case NotOverlap: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[1]>values[2]) && (values[3]>values[1]));
		case NotOverlapInv: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[3]>values[0]) && (values[1]>values[3]));
		case NotStart: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]==values[2]) && (values[3]>values[1]));
		case NotStartInv: 
			return !((values[1]<=values[0]) && (values[3] <=values[2]) && (values[0]==values[2]) && (values[1]>values[3]));   

			//SchursLemma
		case AllDiffXYZ: 
			return (values[0]!=values[1]) && (values[0]!=values[2]) && (values[1]!=values[2]);    
		case AllEqualXYZ: 
			return (values[0]==values[1]) && (values[1]==values[2]);  
		case NotAllDiffXYZ: 
			return (values[0]==values[1]) || (values[0]==values[2]) || (values[1]==values[2]);     
		case NotAllEqualXYZ: 
			return (values[0]!=values[1]) || (values[0]!=values[2]) || (values[1]!=values[2]);


			//Graceful graph
		case AbsDistanceXYZ:
			return (Math.abs(values[0]-values[1])==values[2]);
		case NotAbsDistanceXYZ:
			return (Math.abs(values[0]-values[1])!=values[2]);

			//meeting scheduling 
		case ArrivalConstraint :
			return (Math.abs(values[0]-values[1])<=this.getConstant(0));
		case NotArrivalConstraint :
			return (Math.abs(values[0]-values[1])>this.getConstant(0));

			//Golomb Rulers
			//Distance constraints, ternary : we only want the distances values[1]-values[0] != values[2]-values[1] 
			// because values[0] < values[1] < values[2]

		case DistanceXYZ:
			return (Math.abs(values[0]-values[1])==Math.abs(values[1]-values[2]));
		case NotDistanceXYZ:
			return (Math.abs(values[0]-values[1])!=Math.abs(values[1]-values[2]));
		case DistanceXYZT:
			return (Math.abs(values[0]-values[1])==Math.abs(values[2]-values[3]));
		case NotDistanceXYZT:
			return (Math.abs(values[0]-values[1])!=Math.abs(values[2]-values[3]));
		case GreaterXYZT:
			return ((values[1]-values[0])>(values[3]-values[2]));
		case LessOrEqualXYZT:
			return ((values[1]-values[0])<=(values[3]-values[2]));

			//RFLAP
		case DistanceSup2:
			return Math.abs(values[0]-values[1])>1;
		case DistanceSup3:
			return Math.abs(values[0]-values[1])>2;
		case DistanceInfEq2:
			return Math.abs(values[0]-values[1])<=1;
		case DistanceInfEq3:
			return Math.abs(values[0]-values[1])<=2;

			//Langford
		case Distance2:
			return Math.abs(values[0]-values[1])==2;
		case NotDistance2:
			return Math.abs(values[0]-values[1])!=2;
		case Distance3:
			return Math.abs(values[0]-values[1])==3;
		case NotDistance3:
			return Math.abs(values[0]-values[1])!=3;
		case Distance4:
			return Math.abs(values[0]-values[1])==4;
		case NotDistance4:
			return Math.abs(values[0]-values[1])!=4;
		case Distance5:
			return Math.abs(values[0]-values[1])==5;
		case NotDistance5:
			return Math.abs(values[0]-values[1])!=5;
		case Distance6:
			return Math.abs(values[0]-values[1])==6;
		case NotDistance6:
			return Math.abs(values[0]-values[1])!=6;

			//General distance constraint
		case Distance:
			return Math.abs(values[0]-values[1])== this.getConstant(0);
		case NotDistance:
			return Math.abs(values[0]-values[1])!=this.getConstant(0);
		case DistanceSup:
			return Math.abs(values[0]-values[1])>this.getConstant(0);
		case DistanceInfEq:
			return Math.abs(values[0]-values[1])<=this.getConstant(0);

			//RoundOfGolf
		case ScoredPlus10:
			return values[0] == values[1] + 10;
		case NotScoredPlus10:
			return !(values[0] == values[1] + 10);
		case ScoredPlus4or7:
			if(values[0]==values[1]+7) return values[2]==values[1]+4;
			else if(values[0]==values[1]+4) return values[2]==values[1]+7;
		case NotScoredPlus4or7:
			if(values[0]!=values[1]+7) return values[2]!=values[1]+4;
			else if(values[0]!=values[1]+4) return values[2]!=values[1]+7;

		default:
			throw new RuntimeException("unknow constraint type in class AcqConstraint");
		}
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	/*********************************************************************
	 * Constraint negation using reformulation
	 * 
	 * @param constraint c
	 * @return negation of c
	 * 
	 * @author LAZAAR
	 * @date 2014-12-03
	 * @updated 2016-12-13
	 ********************************************************************/
	public  AcqConstraint negation() {

		switch(this.getType()){
		//Arithmetic
		case DifferentXY: 
			return new AcqConstraint(Cst_type.EqualXY,this.getListVariables(), this.getListConstant());
		case EqualXY: 
			return new AcqConstraint(Cst_type.DifferentXY,this.getListVariables(), this.getListConstant());
		case GreaterOrEqualXY: 
			return new AcqConstraint(Cst_type.LessXY,this.getListVariables(), this.getListConstant());
		case GreaterXY: 
			return new AcqConstraint(Cst_type.LessOrEqualXY,this.getListVariables(), this.getListConstant());
		case LessOrEqualXY: 
			return new AcqConstraint(Cst_type.GreaterXY,this.getListVariables(), this.getListConstant());
		case LessXY: 
			return new AcqConstraint(Cst_type.GreaterOrEqualXY,this.getListVariables(), this.getListConstant());

			//N-queens
		case OutDiag1: 
			return new AcqConstraint(Cst_type.InDiag1,this.getListVariables(), this.getListConstant());
		case OutDiag2: 
			return new AcqConstraint(Cst_type.InDiag2,this.getListVariables(), this.getListConstant());
		case InDiag1: 
			return new AcqConstraint(Cst_type.OutDiag1,this.getListVariables(), this.getListConstant());
		case InDiag2: 
			return new AcqConstraint(Cst_type.OutDiag2,this.getListVariables(), this.getListConstant());
		case AbsOutDiag: 
			return new AcqConstraint(Cst_type.AbsInDiag,this.getListVariables(), this.getListConstant());
		case AbsInDiag: 
			return new AcqConstraint(Cst_type.AbsOutDiag,this.getListVariables(), this.getListConstant());

			//Zebra
			//@FIXME
			//return eq(abs(minus(scope[0],scope[1])),1);
			//Global
		case AllDiff:
			return null;

			//SchursLemma
		case AllDiffXYZ: 
			return	new AcqConstraint(type.NotAllDiffXYZ,this.getListVariables(), this.getListConstant());
		case AllEqualXYZ: 
			return	new AcqConstraint(Cst_type.NotAllEqualXYZ,this.getListVariables(), this.getListConstant());
		case NotAllDiffXYZ: 
			return	new AcqConstraint(Cst_type.AllDiffXYZ,this.getListVariables(), this.getListConstant());
		case NotAllEqualXYZ: 
			return	new AcqConstraint(Cst_type.AllEqualXYZ,this.getListVariables(), this.getListConstant());
		case DistanceXYZ:
			return	new AcqConstraint(Cst_type.NotDistanceXYZ,this.getListVariables(), this.getListConstant());
		case NotDistanceXYZ:
			return	new AcqConstraint(Cst_type.DistanceXYZ,this.getListVariables(), this.getListConstant());
		case DistanceXYZT:
			return	new AcqConstraint(Cst_type.NotDistanceXYZT,this.getListVariables(), this.getListConstant());
		case NotDistanceXYZT:
			return	new AcqConstraint(Cst_type.DistanceXYZT,this.getListVariables(), this.getListConstant());
		case GreaterXYZT:
			return	new AcqConstraint(Cst_type.LessOrEqualXYZT,this.getListVariables(), this.getListConstant());
		case LessOrEqualXYZT:
			return	new AcqConstraint(Cst_type.GreaterXYZT,this.getListVariables(), this.getListConstant());


			// ZEBRA
		case Neighbour:
			return	new AcqConstraint(Cst_type.NotNeighbour,this.getListVariables(), this.getListConstant());
		case NotNeighbour:
			return	new AcqConstraint(Cst_type.Neighbour,this.getListVariables(), this.getListConstant());
		case Neighbour2:
			return	new AcqConstraint(Cst_type.NotNeighbour2,this.getListVariables(), this.getListConstant());
		case NotNeighbour2:
			return	new AcqConstraint(Cst_type.Neighbour2,this.getListVariables(), this.getListConstant());
		case Neighbour3:
			return	new AcqConstraint(Cst_type.NotNeighbour3,this.getListVariables(), this.getListConstant());
		case NotNeighbour3:
			return	new AcqConstraint(Cst_type.Neighbour3,this.getListVariables(), this.getListConstant());
		case Neighbour4:
			return	new AcqConstraint(Cst_type.NotNeighbour4,this.getListVariables(), this.getListConstant());
		case NotNeighbour4:
			return	new AcqConstraint(Cst_type.Neighbour4,this.getListVariables(), this.getListConstant());
		case At1:
			return	new AcqConstraint(Cst_type.notAt1,this.getListVariables(), this.getListConstant());
		case notAt1:
			return	new AcqConstraint(Cst_type.At1,this.getListVariables(), this.getListConstant());
		case At2:
			return	new AcqConstraint(Cst_type.notAt2,this.getListVariables(), this.getListConstant());
		case notAt2:
			return	new AcqConstraint(Cst_type.At2,this.getListVariables(), this.getListConstant());
		case At3:
			return	new AcqConstraint(Cst_type.notAt3,this.getListVariables(), this.getListConstant());
		case notAt3:
			return	new AcqConstraint(Cst_type.At3,this.getListVariables(), this.getListConstant());
		case At4:
			return	new AcqConstraint(Cst_type.notAt4,this.getListVariables(), this.getListConstant());
		case notAt4:
			return	new AcqConstraint(Cst_type.At4,this.getListVariables(), this.getListConstant());
		case At5:
			return	new AcqConstraint(Cst_type.notAt5,this.getListVariables(), this.getListConstant());
		case notAt5:
			return	new AcqConstraint(Cst_type.At5,this.getListVariables(), this.getListConstant());

		case Next:
			return	new AcqConstraint(Cst_type.NNext,this.getListVariables(), this.getListConstant());
		case NNext:
			return	new AcqConstraint(Cst_type.Next,this.getListVariables(), this.getListConstant());
		case Prev:
			return	new AcqConstraint(Cst_type.NPrev,this.getListVariables(), this.getListConstant());
		case NPrev:
			return	new AcqConstraint(Cst_type.Prev,this.getListVariables(), this.getListConstant());

		case DiffX:
			return	new AcqConstraint(Cst_type.EqualX,this.getListVariables(), this.getListConstant());
		case EqualX:
			return	new AcqConstraint(Cst_type.DiffX,this.getListVariables(), this.getListConstant());

		case DistanceSup2:
			return new AcqConstraint(Cst_type.DistanceInfEq2,this.getListVariables(), this.getListConstant());
		case DistanceInfEq2:
			return new AcqConstraint(Cst_type.DistanceSup2,this.getListVariables(), this.getListConstant());
		case DistanceSup3:
			return new AcqConstraint(Cst_type.DistanceInfEq3,this.getListVariables(), this.getListConstant());
		case DistanceInfEq3:
			return new AcqConstraint(Cst_type.DistanceSup3,this.getListVariables(), this.getListConstant());

			//Meeting Scheduling
		case ArrivalConstraint:
			return	new AcqConstraint(Cst_type.NotArrivalConstraint,this.getListVariables(), this.getListConstant());
		case NotArrivalConstraint:
			return	new AcqConstraint(Cst_type.ArrivalConstraint,this.getListVariables(), this.getListConstant());

		case AbsDistanceXYZ:
			return	new AcqConstraint(Cst_type.NotAbsDistanceXYZ,this.getListVariables(), this.getListConstant());
		case NotAbsDistanceXYZ:
			return new AcqConstraint(Cst_type.AbsDistanceXYZ,this.getListVariables(), this.getListConstant());

			//Langford
		case Distance2:
			return new AcqConstraint(Cst_type.NotDistance2,this.getListVariables(), this.getListConstant());
		case NotDistance2:
			return new AcqConstraint(Cst_type.Distance2,this.getListVariables(), this.getListConstant());
		case Distance3:
			return new AcqConstraint(Cst_type.NotDistance3,this.getListVariables(), this.getListConstant());
		case NotDistance3:
			return new AcqConstraint(Cst_type.Distance3,this.getListVariables(), this.getListConstant());
		case Distance4:
			return new AcqConstraint(Cst_type.NotDistance4,this.getListVariables(), this.getListConstant());
		case NotDistance4:
			return new AcqConstraint(Cst_type.Distance4,this.getListVariables(), this.getListConstant());
		case Distance5:
			return new AcqConstraint(Cst_type.NotDistance5,this.getListVariables(), this.getListConstant());
		case NotDistance5:
			return new AcqConstraint(Cst_type.Distance5,this.getListVariables(), this.getListConstant());
		case Distance6:
			return new AcqConstraint(Cst_type.NotDistance6,this.getListVariables(), this.getListConstant());
		case NotDistance6:
			return new AcqConstraint(Cst_type.Distance6,this.getListVariables(), this.getListConstant());


		case Distance:
			return new AcqConstraint(Cst_type.NotDistance,this.getListVariables(), this.getListConstant());
		case NotDistance:
			return new AcqConstraint(Cst_type.Distance,this.getListVariables(), this.getListConstant());
		case DistanceSup:
			return new AcqConstraint(Cst_type.DistanceInfEq,this.getListVariables(), this.getListConstant());
		case DistanceInfEq:
			return new AcqConstraint(Cst_type.DistanceSup,this.getListVariables(), this.getListConstant());

		default:
			throw new RuntimeException("unknow constraint type in class AcqConstraint: "+this.toString());    
		}
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------




	/*******************************************************************
	 * GENERALISATION FUNCTION:  isInListOfConstraint
	 * 
	 * checks if a given constraint is in a given list
	 * @author LAZAAR
	 * @date 2015-04-20
	 * @updated 2016-12-13
	 *******************************************************************/

	public boolean isInListOfConstraint(List<AcqConstraint> listofcst) {

		for (AcqConstraint c : listofcst) 
			if (this.equals(c)) 
				return true;


		return false;
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	/************************************
	 * shortString
	 * 
	 * @author LAZAAR
	 * @date 2016-12-13
	 ************************************/

	public String shortString() {
		switch (this.getType()) {

		case Next: 
			return " next ";
		case Prev: 
			return " prev ";
		case Neighbour: 
			return " neighbour ";

		case DifferentXY: 
			return " != ";
		case EqualXY: 
			return " = ";
		case GreaterOrEqualXY: 
			return " >= ";
		case GreaterXY: 
			return " > ";
		case LessOrEqualXY: 
			return " <= ";
		case LessXY: 
			return " < "; 
		default:
			return this.getType().toString();
		}

	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


	/************************************
	 * cst type to Tex
	 * 
	 * @author LAZAAR
	 * @date 2016-12-13
	 ************************************/

	public String typeTex() {
		switch (this.getType()) {
		case DifferentXY: 
			return "\\neq";
		case EqualXY: 
			return "=";
		case GreaterOrEqualXY: 
			return "\\geq";
		case GreaterXY: 
			return ">";
		case LessOrEqualXY: 
			return "\\leq";
		case LessXY: 
			return "<"; 
		default:
			return this.getType().toString();
		}
	}



	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				GETTERs / SETTERs
	///
	////////////////////////////////////////////////////

	public Cst_type getType() {
		return this.type;
	}



	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	public List<AcqVariable> getVariables() {

		return this.variables;
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	public ArrayList<Integer> getListConstant() {
		return this.constants;
	}

	public AcqVariable[] getArrayVariables() {
		return (AcqVariable[]) this.variables.toArray();
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	public int getArity(){
		return this.arity;
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	public int getConstant(int i){
		return this.constants.get(i);
	}



	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				OVERRIDED METHODs
	///
	////////////////////////////////////////////////////

	@Override
	public String toString() {

		// CST(X1,....,Xn,cst1,...,cstN)

		String st = "  " +this.type+ " (";
		// 
		for (AcqVariable v : this.variables)   st += v.name()+", ";        

		for (int c : this.constants)   st += c+", ";        

		return st.substring(0, st.length()-2)+")";
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AcqConstraint other = (AcqConstraint) obj;

		if (this.getType() != other.getType())
			return false;

		if (!this.getListVariables().equals(other.getListVariables()))
			return false;

		if (!this.constants.equals(other.constants))
			return false;

		return true;
	}




	public boolean isEmpty() {
		// TODO Auto-generated method stub
		return false;
	}









	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


}