/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package ag;
/**
 * Class used to represent the bias in the constraint acquisition.
 */
public class ACQ_Bias {
	/**
	 * Network of the bias, including both variables and constraints
	 */
    protected ACQ_Network network;
    /**
     * Constructor of the bias from a network
     * 
     * @param network Network
     */
    public ACQ_Bias(ACQ_Network network) {
        this.network = network;
    }
    /**
     * Returns the size of the smallest scope of this bias
     * 
     * @return size of the smallest scope
     */
    public int computeMinArity() {
        int minArity = Integer.MAX_VALUE;
        for (ACQ_IConstraint constraint : network.constraints) {
            if (constraint.getArity() < minArity) {
                minArity = constraint.getArity();
            }
        }
        return minArity;
    }
    /**
     * Returns the variables of this bias
     * 
     * @return variables of this bias
     */
    public ACQ_Scope getVars() {
        return network.variables;
    }
    /**
     * Returns the set of constraints of this bias
     * 
     * @return set of constraints of this bias
     */
    public ACQ_ConstraintSet getConstraints() {
        return network.constraints;
    }
    /**
     * Returns the set of constraints violated by query_bgd
     * 
     * @param query_bgd Positive or negative example
     * @return set of constraints violated by query_bgd
     */
    ACQ_ConstraintSet getKappa(ACQ_Query query_bgd) {
        ACQ_ConstraintSet set = new ACQ_ConstraintSet();
        for (ACQ_IConstraint cst : network.constraints) {
            if (query_bgd.scope.containsAll(cst.getScope())) {
                if (!cst.checker(cst.getProjection(query_bgd))) {
                    set.add(cst);
                }
            }
        }
        return set;
    }
    /**
     * <p>Remove a set of constraints contained into this bias</p> 
     * <p>Take a set of constraints as parameter</p>
     * @param temp_kappa set of constraints to remove from this bias
     */
    void reduce(ACQ_ConstraintSet temp_kappa) {
        network.constraints.removeAll(temp_kappa);
    }
    /**
     * <p>Remove a set of constraints contained into this bias</p> 
     * <p>Take as parameter an example (positive or negative)</p>
     * <p>Calculate the kappa of e and remove it from this bias</p>
     * @param e Positive or negative example
     */
    public void reduce(ACQ_Query e) {

        reduce(getKappa(e));
    }
    /**
     * <p>Returns a subnet of this bias. This subnet has the specified scope as scope
     * and the set of constraints whose scope is included in varSet</p>
     * 
     * @param varSet ACQ_Scope
     * @return a constraint network, subnet of this bias
     */
    ACQ_Network getProjection(final ACQ_Scope varSet) {
        // calcule l'ensemble des contraintes du biais dont les variables sont contenues dans varSet
        final ACQ_ConstraintSet projectedCSet = new ACQ_ConstraintSet();
        network.constraints.forEach((x) -> {
            if (varSet.containsAll(x.getScope())) {
            	projectedCSet.add(x);
            }
        });
        return new ACQ_Network(varSet, projectedCSet);
    }
}
