/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package ag;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import org.chocosolver.solver.variables.IntVar;

/**
 * Abstract class that defines the functions common to all constraints
 * @author agutierr
 */
public abstract class ACQ_Constraint implements ACQ_IConstraint{
	/**
	 * Name of this constraint
	 */
    private final String name;
    /**
     * Variables of this constraint
     */
    private final int[] variables;
    /**
     * Constructor of this constraint
     * 
     * @param name Name of this constraint
     * @param variables Variables of this constraint
     */
    public ACQ_Constraint(String name,int[] variables)
    {
        this.name=name;
        this.variables=variables;
    }
    /**
     * Returns a sub array of the specified IntVar array,
     * that only contains the variables involved into this constraint
     * 
     * @param fullVarSet All the variables of the solver model
     * @return A sub array containing the variables of this constraint
     * @author teddy
     */
    public IntVar[] getVariables(IntVar[] fullVarSet) {
    	ACQ_Scope scope = getScope();
    	ArrayList<IntVar> intVars = new ArrayList<>();
    	Iterator<Integer> iterator = scope.iterator();
    	while(iterator.hasNext()) {
    		intVars.add(fullVarSet[iterator.next()]);
    	}
    	return intVars.toArray(new IntVar[intVars.size()]);
    }
    
    /**
     * Returns the name of this constraint
     * 
     * @return name of this constraint
     */
    @Override
    public String getName(){
        return name;
    }
    /**
     * Returns the scope of this constraint
     * 
     * @return scope of this constraint
     */
    @Override
    public ACQ_Scope getScope() {
        return new ACQ_Scope(variables);
    }
    /**
     * Returns the number of variables of this constraint
     * 
     * @return number of variable of this constraint
     */
    @Override
    public int getArity() {
        return variables.length;
    }
    
    /**
     * Returns the number of variables of this constraint
     * 
     * @return number of variable of this constraint
     */
    @Override
    public int[] getVariables() {
        return variables;
    }
    
    /**
     * Get the numeric values of the example query
     * 
     * @param query Example positive or negative
     * @return numeric values of the example query
     */
    @Override
    public int[] getProjection(ACQ_Query query){
        int index=0;
        int [] values=new int[variables.length];
        for(int numvar:variables)
            values[index++]=query.getValue(numvar);
        return values;
    }
    /**
     * Checks if the constraint is violated for a given set of values
     * 
     * @param values set of values to check
     * @return false if the set of values violate this constraint
     */
    @Override
    public final boolean checker(int[] values){
        return check(values);
    }
    /**
     * Checks if the constraint is violated for a given set of values
     * 
     * @param value set of values to check
     * @return false if the set of values violate this constraint
     */
    protected abstract boolean check(int... value);
/*    @Override
    public int compareTo(ACQ_IConstraint o){
        return Integer.compare(hashCode(),o.hashCode());
    }
*/
	@Override
	public String toString() {
		return name + Arrays.toString(variables);
	}


}
