/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package ag;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Class that gathers multiples constraints into a set of constraint
 * @author agutierr
 */
public final class ACQ_ConstraintSet implements Iterable<ACQ_IConstraint>{
	// a revoir en fonction des besoins
	/**
	 * Set of constraints
	 */
	protected HashSet<ACQ_IConstraint> constraints=new HashSet<>();
	/**
	 * Constructor of an empty set
	 */
	public ACQ_ConstraintSet() {

	}

	/**
	 * Constructor of constraint set 
	 */
	public ACQ_ConstraintSet(ACQ_IConstraint...cstr_set) {
		for(ACQ_IConstraint c : cstr_set)
			this.constraints.add(c);
	}
	/**
	 * Constructor from a given set of constraints
	 * Does not affect the set of constraints given as parameter
	 * 
	 * @param other set of constraints
	 */
	public ACQ_ConstraintSet(ACQ_ConstraintSet other)
	{
		constraints.addAll(other.constraints);
	}
	/**
	 * Returns an iterator of this set of constraints
	 * 
	 * @return iterator of this set of constraints
	 */
	@Override
	public Iterator<ACQ_IConstraint> iterator() {
		return constraints.iterator();
	}
	/**
	 * Add a constraint to this set of constraints
	 * 
	 * @param cst constraint
	 */
	public void add(ACQ_IConstraint cst) {
		constraints.add(cst);
	}

	/**
	 * Remove a constraint from this set of constraint
	 * 
	 * @param cst constraint
	 * @return true if the constraint has been successfully removed
	 */
	public boolean remove(ACQ_IConstraint cst) {
		return constraints.remove(cst);
	}

	/**
	 * Returns the size of this set of constraints
	 * 
	 * @return size of this set of constraints
	 */
	public int size() {
		return constraints.size();
	}

	/**
	 * <p>Retains only the elements in this set that are contained in the 
	 * specified set (optional operation). In other words, removes 
	 * from this set all of its elements that are not contained in the specified 
	 * set. If the specified collection is also a set, this operation 
	 * effectively modifies this set so that its value is the intersection of the two sets.</p>
	 * 
	 * @param other_set set containing elements to be retained in this set
	 */
	public void retainAll(ACQ_ConstraintSet other_set)
	{
		constraints.retainAll(other_set.constraints);
	}

	/**
	 * Checks if this set of constraints is empty
	 * 
	 * @return true if this set is empty
	 */
	public boolean isEmpty() {
		return constraints.isEmpty();
	}

	/**
	 * Remove from this set of constraints all constraints contained in the specified set
	 * 
	 * @param temp_kappa set of constraints to remove
	 */
	public void removeAll(ACQ_ConstraintSet temp_kappa) {
		constraints.removeAll(temp_kappa.constraints);
	}

	public boolean contains(ACQ_ConjuctionConstraint candidate) {
		if(constraints.contains(candidate))
			return true;
		return false;
	}

	@Override
	public String toString() {
		return "ACQ_ConstraintSet [constraints=" + constraints + "]";
	}

	

}
