package ag;

import config.Heuristic;
/**
 * Abstract class that defines the main functions used by the solver
 * 
 *
 */
abstract public class ACQ_ConstraintSolver {
	/**
	 * Used to evaluate the performance of this solver
	 */
	boolean timeouted = false;
	
	/**
	 * Checks if there is any solution of the specified constraint network
	 * 
	 * @param learned_network Constraint network to check
	 * @return true if there is at least one solution
	 */
	public abstract boolean solve(ACQ_Network learned_network);
	
	/**
     * Generate a query that satisfies networkA but violates at least one constraint of networkB
     * 
     * @param networkA Constraint network to satisfy
     * @param networkB Constraint network to violate
     * @param heuristic
     * @return Query that satisfies networkA but violates at least one constraint of networkB
     */
	public abstract ACQ_Query solve_AnotB(ACQ_Network network1,
			ACQ_Network network2, boolean findc, Heuristic heuristic);
	
	/**
	 * Function used to check if this solver exceed a definite time
	 * 
	 * @return true if the time is exceeded
	 */
	public boolean timeout_reached(){
		return this.timeouted;
	}
	/**
     * Generate a query that satisfies networkA and at least one constraint of networkB
     * 
     * @param networkA Constraint network to satisfy
     * @param networkB Constraint network where at least one constraint is satisfied
     * @param heuristic
     * @return Query that satisfies networkA and at least one constraint of networkB
     */
	public abstract ACQ_Query solve_AnotAllB(ACQ_Network networkA, 
			ACQ_Network networkB, Heuristic heuristic);

}
