package ag;

import java.util.Scanner;

import config.Heuristic;
import config.Query_type;

public class ACQ_QUACQ implements ACQ_Acquisition{

	private ACQ_Network learned_network;
	private ACQ_ConstraintSolver solver;
	private ACQ_Bias bias;
	private ACQ_Learner learner;
	private Heuristic heuristic;
	private boolean auto_learn;
	private boolean normalizedCSP;

	public ACQ_QUACQ(ACQ_ConstraintSolver solver, ACQ_Bias bias, ACQ_Learner learner, Heuristic heuristic) {

		//NL: config part
		this.heuristic = heuristic;
		this.solver = solver;
		this.bias = bias;
		this.learner = learner;
		this.learned_network = new ACQ_Network(bias.getVars());
		this.auto_learn = false;
		this.normalizedCSP=true;
		//////////////////////////////////

	}

	public ACQ_Network getLearnedNetwork() {
		return learned_network;
	}

	@Override
	public ACQ_IConstraint findC(ACQ_Scope scope, ACQ_Query e, boolean normalizedCSP) {

		if(normalizedCSP)
			return findC_ijcai13(scope,e);
		else
			return findC_AIJ(scope,e);

	}
	/**
	 * ****************************************************************
	 * findC (new findC presented in AIJ on non-normalized CSPs
	 *
	 * @date 30-05-19
	 * @author LAZAAR
	 * *****************************************************************
	 */
	private ACQ_IConstraint findC_AIJ(ACQ_Scope scope, ACQ_Query e) {

		ACQ_Network learned_network_y = new ACQ_Network(learned_network, scope);
		ACQ_Bias bias_y = new ACQ_Bias(bias.getProjection(scope));
		ACQ_ConstraintSet temp_kappa;

		assert scope.size() > 1;

		ACQ_ConstraintSet candidates = new ACQ_ConstraintSet(bias_y.getConstraints());  //NL:  candidates = delta in IJCAI13

		assert (candidates.size() == bias_y.getConstraints().size());

		candidates= join(candidates,bias_y.getKappa(e));		//NL: join operator between delta and kappa_delta

		while (true) {

			if (candidates.isEmpty()) {
				return null;
			}
			//TODO check if returning null or empty constraint (NL: 03-10-17)			

			int temp = candidates.size();

			ACQ_Query partial_findC_query = query_gen(learned_network_y, new ACQ_Network(scope, candidates), scope, Query_type.findc, Heuristic.SOL);

			assert (candidates.size() == temp);

			if (partial_findC_query.isEmpty()) {

				bias.reduce(candidates);

				return candidates.iterator().next();

			}

			temp_kappa = bias.getKappa(partial_findC_query);

			if (temp_kappa.isEmpty()) {
				throw new RuntimeException("Collapse state");
			}

			learner.asked_query(partial_findC_query);

			if (!partial_findC_query.isClassified()) {
				boolean b = learner.ask(partial_findC_query);
				//                System.out.println("learner ask="+b+" for "+partial_findC_query);
				partial_findC_query.classify(b);
				learner.memory_up(partial_findC_query);
			}

			if (partial_findC_query.isPositive()) {
				bias.reduce(temp_kappa);			// NL: kappa of partial_findC_query
				candidates.removeAll(temp_kappa);
			} else {
				candidates.retainAll(temp_kappa);
			}
		}

	}
	/************************************************************
	 * Join Operator
	 * 
	 * set_one join set_two = { c1 and c2 | c1 in set_one and c2 in set_two and (c1 and c2 is sat)}
	 * @param set_one
	 * @param set_two
	 * @return set_three 
	 ************************************************************/

	public ACQ_ConstraintSet join(ACQ_ConstraintSet set_one, ACQ_ConstraintSet set_two) {

		ACQ_ConstraintSet set_three= new ACQ_ConstraintSet();


		for(ACQ_IConstraint c1: set_one)
			for(ACQ_IConstraint c2: set_two) {
				System.out.println("how many??");
				if(!c1.equals(c2) && 
						solver.solve(new ACQ_Network(this.bias.getVars(),new ACQ_ConstraintSet(new ACQ_IConstraint[] {c1,c2})))) {

					ACQ_ConjuctionConstraint candidate= new ACQ_ConjuctionConstraint(c1,c2);
					if(!set_three.contains(candidate) && !set_three.contains(new ACQ_ConjuctionConstraint(c2,c1)))
						set_three.add(candidate);
				}
				set_three.constraints.addAll(set_one.constraints);
			//	set_three.constraints.addAll(set_two.constraints);
			}


		return set_three;
	}
	/**
	 * ****************************************************************
	 * findC (classic findC of IJCAI13)
	 *
	 * @date 03-10-17
	 * @update 17-11-18
	 * @author LAZAAR
	 * *****************************************************************
	 */
	private ACQ_IConstraint findC_ijcai13(ACQ_Scope scope, ACQ_Query e) {

		ACQ_Network learned_network_y = new ACQ_Network(learned_network, scope);
		ACQ_Bias bias_y = new ACQ_Bias(bias.getProjection(scope));
		ACQ_ConstraintSet temp_kappa;

		assert scope.size() > 1;

		ACQ_ConstraintSet candidates = new ACQ_ConstraintSet(bias_y.getConstraints());  //NL:  candidates = delta in IJCAI13

		assert (candidates.size() == bias_y.getConstraints().size());

		candidates.retainAll(bias.getKappa(e));

		while (true) {

			if (candidates.isEmpty()) {
				return null;
			}
			//TODO check if returning null or empty constraint (NL: 03-10-17)			

			int temp = candidates.size();

			ACQ_Query partial_findC_query = query_gen(learned_network_y, new ACQ_Network(scope, candidates), scope, Query_type.findc, Heuristic.SOL);

			assert (candidates.size() == temp);

			if (partial_findC_query.isEmpty()) {

				bias.reduce(candidates);

				return candidates.iterator().next();

			}

			temp_kappa = bias.getKappa(partial_findC_query);

			if (temp_kappa.isEmpty()) {
				throw new RuntimeException("Collapse state");
			}

			learner.asked_query(partial_findC_query);

			if (!partial_findC_query.isClassified()) {
				boolean b = learner.ask(partial_findC_query);
				//                System.out.println("learner ask="+b+" for "+partial_findC_query);
				partial_findC_query.classify(b);
				learner.memory_up(partial_findC_query);
			}

			if (partial_findC_query.isPositive()) {
				bias.reduce(temp_kappa);			// NL: kappa of partial_findC_query
				candidates.removeAll(temp_kappa);
			} else {
				candidates.retainAll(temp_kappa);
			}
		}

	}


	/**
	 * **************************************************************
	 * FindScope procedure
	 *
	 * @param negative_example : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @param Bgd : background variables
	 * @param mutex : boolean mutex
	 * @return variable scope
	 *
	 * @author LAZAAR
	 * **************************************************************
	 */
	@Override
	public ACQ_Scope findScope(ACQ_Query negative_example,
			ACQ_Scope X, ACQ_Scope Bgd, boolean mutex) {

		if (mutex && Bgd.size() >= bias.computeMinArity()) {	//TESTME if minArity has the good value !!

			ACQ_Query query_bgd = new ACQ_Query(Bgd, Bgd.getProjection(negative_example));      // projection e|Bgd
			ACQ_ConstraintSet temp_kappa = bias.getKappa(query_bgd);

			learner.asked_query(query_bgd);

			if (!query_bgd.isClassified()) {
				learner.ask(query_bgd);
				learner.memory_up(query_bgd);
			}
			if (!query_bgd.isPositive()) //NL: negative
			{
				return ACQ_Scope.EMPTY;	//NL: return emptyset
			} else {
				bias.reduce(temp_kappa);
			}
		}

		if (X.size() == 1) {
			return X;
		}

		//NL: different splitting manners can be defined here!
		ACQ_Scope[] splitedX = X.split();
		ACQ_Scope X1 = splitedX[0];
		ACQ_Scope X2 = splitedX[1];

		System.out.println("X1="+X1);
		ACQ_Scope S1 = findScope(negative_example, X1, Bgd.union(X2), true);    // NL: First recursive call of findScope
		System.out.println("S1="+S1);
		mutex = (S1 != ACQ_Scope.EMPTY);
		System.out.println("X2="+X2);
		ACQ_Scope S2 = findScope(negative_example, X2, Bgd.union(S1), mutex);    // NL: Second recursive call of findScope
		System.out.println("S2="+S2);

		return S1.union(S2);
	}

	/**
	 * **************************************************************************
	 * query_gen
	 *
	 * @param network1
	 * @param network2
	 * @param scope
	 * @param type
	 * @return Query
	 * @author LAZAAR
	 * @date 03-10-2017
	 *
	 * get a query of type "type" on scope "scope" s.t., network1 and not
	 * network2
	 * ***************************************************************************
	 */
	@Override
	public ACQ_Query query_gen(ACQ_Network network1, ACQ_Network network2, ACQ_Scope scope, Query_type type, Heuristic h) {
		System.out.println("query_gen() network1="+network1);
		System.out.println("query_gen() network2="+network2);
		System.out.println("query_gen() scope="+scope);
		switch (type) {
		case findc:
			return solver.solve_AnotB(network1, network2,true, heuristic);
		case MQ:
			return solver.solve_AnotB(network1, network2,false, heuristic);
		case findscope:
		default:
			return solver.solve_AnotB(network1, network2,false, heuristic);
		}
	}


	@Override
	public boolean process() {
		boolean convergence=false;
		boolean collapse=false;

		assert(learned_network.size()==0);
		while(!convergence && !collapse){
			if(!solver.solve(learned_network) && !solver.timeout_reached()){
				collapse=true;
			}


			ACQ_Query membership_query=query_gen(learned_network, bias.network, bias.getVars(), Query_type.MQ, heuristic );
			System.out.println(membership_query);
			if(membership_query.isEmpty())
				convergence=true;

			else {
				if(learner.ask(membership_query))
					bias.reduce(membership_query);
				else{
					ACQ_IConstraint cst= findC(findScope(membership_query, bias.getVars(), 
							new ACQ_Scope(), false),membership_query, normalizedCSP);	
					if(cst==null) 
						collapse=true;	
					else 
						learned_network.add(cst,false);
				}
			}
		}

		return !collapse;
	}
	//		assert(learned_network.size() == 0);
	//		ACQ_Query membership_query = query_gen(learned_network, bias.network, bias.getVars(), Query_type.MQ, heuristic);
	//		while(!bias.network.constraints.isEmpty()) {
	//			// collapse state
	//			if(!solver.solve(learned_network) && !solver.timeout_reached()){
	//				System.out.println("no solution for current learned network");
	//				return false;
	//			}
	//			membership_query = query_gen(learned_network, bias.network, bias.getVars(), Query_type.MQ, heuristic);
	//			System.out.println("Query found:" + membership_query.toString());
	//			System.out.println(bias.network.toString());
	//			if(learner.ask(membership_query)) {
	//				// enlever du biais le kappa de membership_query
	//				System.out.println("------OLD BIAS-----");
	//				System.out.println(bias.network.toString());
	//				ACQ_Network kappa = bias.getProjection(membership_query.getScope());
	//				System.out.println("---------Kappa------");
	//				System.out.println(kappa.toString());
	//				bias.reduce(kappa.getConstraints());
	//				System.out.println("------NEW BIAS-------");
	//				System.out.println(bias.network.toString());
	//			}else {
	//				ACQ_IConstraint cst = findC(findScope(membership_query, bias.getVars(), 
	//						new ACQ_Scope(), false),membership_query);
	////				System.out.println("constraint found: "+cst.toString());
	//				if(cst == null) {
	//					System.out.println("no constraint found");
	//					return false;
	//				}else {
	//					learned_network.add(cst, false);
	//					bias.network.constraints.remove(cst);
	//				}
	//			}
	//		}
	//		return true;

	public boolean isNormalizedCSP() {
		return normalizedCSP;
	}

	public void setNormalizedCSP(boolean normalizedCSP) {
		this.normalizedCSP = normalizedCSP;
	}


}
