package ag;
import java.util.Scanner;

import ag.expe.*;
import config.Heuristic;

/**
 * 
 * 
 * @author teddy
 *
 */
public class ConstraintAcquisition {
	
	private ACQ_Bias bias;
	
	private ACQ_ConstraintSolver solver;
	
	private ACQ_Learner learner;
	
	private Experience exp;
	
	public ConstraintAcquisition(int expChoice, boolean auto_learn) {
		switch(expChoice) {
		case(1):
			System.out.println("Launching Sudoku experience");
			exp = new ExpeSUDOKU(auto_learn);
			break;
		case(2):
			System.out.println("Launching Toy 1 experience");
			exp = new ExpeTOY(auto_learn);
			break;
		case(3):
			System.out.println("Launching Toy 2 experience");
			exp = new ExpeTOY2(auto_learn);
			break;
		}
		bias = exp.createBias();
		solver = exp.createSolver();
		learner = exp.createLearner();
	}
	
	public void startExp() {
		ACQ_QUACQ process = new ACQ_QUACQ(solver, bias, learner, Heuristic.SOL);
		if (process.process()) {
			System.out.println("YES");
			System.out.println(process.getLearnedNetwork());
			
		} else {
			System.out.println("NO");
		}
	}
	
	public static boolean getAnswer() {
		Scanner in = new Scanner(System.in);
		String userAnswer;
		do {
			userAnswer = in.next();
			System.out.println("You entered:" + userAnswer);
			if(!userAnswer.equals("y") && !userAnswer.equals("n")) {
				System.out.println("Incorrect answer.\n Please enter y or n.");
			}
		} while(!userAnswer.equals("y") && !userAnswer.equals("n"));
		if(userAnswer.equals("y")) return true;
		else return false;
    }
	
	public StatManager getStatManager() {
		return this.exp.getStatManager();
	}
	public static void main (String[] args) {
		// Choose experience
		System.out.println("Choose an experience : ");
		System.out.println("1. Sudoku");
		System.out.println("2. Toy 1");
		System.out.println("3. Toy 2");
		
		Scanner in = new Scanner(System.in);
		int expChoice = in.nextInt();      
		System.out.println("You entered : " + expChoice);
		
		// Choose learner method (auto or manual)
		System.out.println("Do you want to use the defined learner ? (y/n)");
		
		// ConstaintAcquisition constructor builds the bias, learner and solver
		ConstraintAcquisition exp = new ConstraintAcquisition(expChoice, getAnswer());
		
		// Start the process
		exp.startExp();
		
		System.out.println("------EXPERIENCE STATS------");
		System.out.println(exp.getStatManager().toString());
	}
}
