/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package ag.expe;

import acqConstraints.Operator;
import ag.ACQ_Bias;
import ag.ACQ_ChocoSolver;
import ag.ACQ_ConstraintSet;
import ag.ACQ_ConstraintSolver;
import ag.ACQ_Domain;
import ag.ACQ_Learner;
import ag.ACQ_Network;
import ag.ACQ_QUACQ;
import ag.ACQ_Query;
import ag.ACQ_Scope;
import ag.BinaryArithmetic;
import combinatorial.AllPermutationIterator;
import combinatorial.CombinationIterator;
import config.Heuristic;
import java.util.BitSet;
import java.util.HashMap;

/**
 *
 * @author agutierr
 */
public class ExpeSUDOKU extends Experience{

//	Heuristic heuristic = Heuristic.SOL;
//	ACQ_ConstraintSolver solver;
//	ACQ_Bias bias;
//	private final ACQ_Learner learner;

    public ExpeSUDOKU(boolean auto_learn) {
    	super(auto_learn);
    }

    public ACQ_ConstraintSolver createSolver() {
        return new ACQ_ChocoSolver(new ACQ_Domain() {
            @Override
            public int getMin(int numvar) {
                return 1;
            }

            @Override
            public int getMax(int numvar) {
                return 9;
            }
        });
    }

    public ACQ_Learner createLearner() {
        return new ACQ_Learner() {
            @Override
            public boolean ask(ACQ_Query e) {
            	if(auto_learn) {
            		int i=0;
            		int[] tuple = e.getTuple();  
            		HashMap<Integer,BitSet> cols=new HashMap<>();
            		HashMap<Integer,BitSet> lines=new HashMap<>();
            		for(int numvar:e.getScope())
            		{
            			int col=numvar%9;
            			BitSet values=cols.get(col);
            			if(values==null)
            			{
            				values=new BitSet();
            				cols.put(col, values);
            			}
            			if(values.get(tuple[i])) {
            				// stats update
            				stats.update(e);
            				
            				return false;
            			}
            			else values.set(tuple[i]);
            			int line=numvar/9;
            			values=lines.get(line);
            			if(values==null)
            			{
            				values=new BitSet();
            				lines.put(col, values);
            			}
            			if(values.get(tuple[i])) {
            				// stats update
            				stats.update(e);
            				
            				return false;
            			}
            			else values.set(tuple[i]);
            			i++;
            		}
            		// stats update
            		stats.update(e);
            		
            		return true;
            	}else 
            		return getAnswer(e);
            	
            }

        };
    }

    public ACQ_Bias createBias() {
        int NB_VARIABLE = 81;
        // build All variables set
        BitSet bs = new BitSet();
        bs.set(0, NB_VARIABLE);
        ACQ_Scope allVarSet = new ACQ_Scope(bs);
        // build Constraints
        ACQ_ConstraintSet constraints = new ACQ_ConstraintSet();
        CombinationIterator iterator = new CombinationIterator(NB_VARIABLE, 2);
        while (iterator.hasNext()) {
            int[] vars = iterator.next();
            AllPermutationIterator pIterator = new AllPermutationIterator(2);
            while (pIterator.hasNext()) {
                int[] pos = pIterator.next();
                // X != Y
                constraints.add(new BinaryArithmetic("DifferentXY", vars[pos[0]], Operator.NQ, vars[pos[1]]));
                // X == Y
                constraints.add(new BinaryArithmetic("EqualXY", vars[pos[0]], Operator.EQ, vars[pos[1]]));
                // X >= Y
                constraints.add(new BinaryArithmetic("GreaterEqualXY", vars[pos[0]], Operator.GE, vars[pos[1]]));
            }
        }
        ACQ_Network network = new ACQ_Network(allVarSet, constraints);
        return new ACQ_Bias(network);
    }

//    public void learn() {
//        ACQ_QUACQ acquisition = new ACQ_QUACQ(solver, bias, learner, heuristic);
//        if (acquisition.process()) {
//            System.out.println("YES");
//        } else {
//            System.out.println("NO");
//        }
//    }
//
//    public static void main(String args[]) {
//        ExpeSUDOKU expe = new ExpeSUDOKU();
//        expe.learn();
//
//    }
    
}
