package ag.expe;

import acqConstraints.Operator;
import ag.ACQ_Bias;
import ag.ACQ_ChocoSolver;
import ag.ACQ_ConstraintSet;
import ag.ACQ_ConstraintSolver;
import ag.ACQ_Domain;
import ag.ACQ_Learner;
import ag.ACQ_Network;
import ag.ACQ_QUACQ;
import ag.ACQ_Query;
import ag.ACQ_Scope;
import ag.BinaryArithmetic;
import ag.StatManager;
import ag.UnaryArithmetic;
import combinatorial.AllPermutationIterator;
import combinatorial.CombinationIterator;
import config.Heuristic;

import java.util.Arrays;
import java.util.BitSet;
import java.util.HashMap;

/**
 * AllEqual to 5 experience
 * @author teddy
 *
 */
public class ExpeTOY2 extends Experience{
//	Heuristic heuristic = Heuristic.SOL;
//	ACQ_ConstraintSolver solver;
//	ACQ_Bias bias;
//	private final ACQ_Learner learner;
	
	public ExpeTOY2(boolean auto_learn) {
		super(auto_learn);
	}
	
	public ACQ_ConstraintSolver createSolver() {
		return new ACQ_ChocoSolver(new ACQ_Domain() {
			@Override
			public int getMin(int numvar) {
				return 1;
			}
			
			@Override
			public int getMax(int numvar) {
				return 10;
			}
		});
	}
	
	public ACQ_Learner createLearner() {
		return new ACQ_Learner() {
			@Override
			public boolean ask(ACQ_Query e) {
				if(auto_learn) {
					int[] tuple = e.getTuple();
					
					for(int i = 0; i < tuple.length - 1; i++)
						for(int j = i + 1; j < tuple.length; j++)
							if(tuple[i] != tuple[j]) {
								// stats update
								stats.update(e);
								return false;
							}
	//				for(int i = 0; i < tuple.length; i++)
	//					if(tuple[i] != 5)
	//						return false;
					
					// stats update
					stats.update(e);
					return true;
				}else
					return getAnswer(e);
			}
		};
	}
	
	public ACQ_Bias createBias() {
		int NB_VARIABLE = 3;
		// build All variables set
		BitSet bs = new BitSet();
		bs.set(0, NB_VARIABLE);
		ACQ_Scope allVarSet = new ACQ_Scope(bs);
		// build Constraints
		ACQ_ConstraintSet constraints = new ACQ_ConstraintSet();
		CombinationIterator iterator = new CombinationIterator(NB_VARIABLE, 2);
		while (iterator.hasNext()) {
			int[] vars = iterator.next();
			AllPermutationIterator pIterator = new AllPermutationIterator(2);
			while (pIterator.hasNext()) {
				int[] pos = pIterator.next();
				// X != Y
				constraints.add(new BinaryArithmetic("DifferentXY", vars[pos[0]], Operator.NQ, vars[pos[1]]));
				// X == Y
				constraints.add(new BinaryArithmetic("EqualXY", vars[pos[0]], Operator.EQ, vars[pos[1]]));
				// X >= Y
				constraints.add(new BinaryArithmetic("GreaterEqualXY", vars[pos[0]], Operator.GE, vars[pos[1]]));
			}
		}
		for(int i = 0; i < 3; i++)
			constraints.add(new UnaryArithmetic("EqualX", i, Operator.EQ, 5));
		
		ACQ_Network network = new ACQ_Network(allVarSet, constraints);
		return new ACQ_Bias(network);
	}
	
//	public void learn() {
//		ACQ_QUACQ acquisition = new ACQ_QUACQ(solver, bias, learner, heuristic);
//		if (acquisition.process()) {
//			System.out.println("YES");
//			System.out.println(acquisition.getLearnedNetwork());
//			
//		} else {
//			System.out.println("NO");
//		}
//	}
//	
//	public static void main (String[]args) {
//		ExpeTOY2 ex = new ExpeTOY2();
//		ex.learn();
//	}
}
