/***********************************************************
 * Choco2solver class (use of choco 2 solver)
 * @author LAZAAR
 * @date 2016-12-13
 **********************************************************/




package cp;


import static choco.Choco.allDifferent;
import static choco.Choco.and;
import static choco.Choco.eq;
import static choco.Choco.geq;
import static choco.Choco.gt;
import static choco.Choco.ifThenElse;
import static choco.Choco.leq;
import static choco.Choco.lt;
import static choco.Choco.makeIntVar;
import static choco.Choco.minus;
import static choco.Choco.neq;
import static choco.Choco.not;
import static choco.Choco.or;
import static choco.Choco.plus;

import java.util.ArrayList;
import java.util.List;

import acqConstraints.AcqConstraint;
import vocabulary.*;
import choco.kernel.model.constraints.Constraint;
import choco.kernel.model.variables.integer.IntegerVariable;

public class Choco2Solver implements ConstraintSolver {

	
////////////////////////////////////////////////////
///
///				ATTRIBUTEs
///
////////////////////////////////////////////////////
	

////////////////////////////////////////////////////
///
///				CONSTRUCTORs
///
////////////////////////////////////////////////////

	

////////////////////////////////////////////////////
///
///				METHODs
///
////////////////////////////////////////////////////

	
/*****************************************************
 * choco2vars returns the choco2 vars from a given 
 * AcqConstraint
 * 
 * @param AcqConstraint
 * @return Choco-2 vars
 *****************************************************/
	
	
	
	public IntegerVariable[] choco2vars(AcqConstraint constraint) {
		
		IntegerVariable[] vars = new IntegerVariable[constraint.getArity()];
		
		int i=0;
		for (AcqVariable v : constraint.getArrayVariables()) {
			IntegerVariable vi = makeIntVar(v.name(), v.getMin(), v.getMax());
			vars[i]=vi;
			i++;

		}
		
		return vars;
	}


/*****************************************************
* acqConstraint2choco2 a AcqConstraint as a Choco2 constraint	
* 
* @param AcqConstraint
* @return Choco-2 Constraint
*****************************************************/

public Constraint acqConstraint2choco2(AcqConstraint constraint, IntegerVariable[] vars) {
	
	
	
	assert(constraint.getArity()==vars.length);
	
	switch (constraint.getType()) {
	//Arithmetic

	case EqualX: 
		return eq(vars[0],constraint.getConstant(0));
	case DiffX: 
		return neq(vars[0],constraint.getConstant(0));
	case DifferentXY: 
		return neq(vars[0],vars[1]);
	case EqualXY: 
		return eq(vars[0],vars[1]);
	case GreaterOrEqualXY: 
		return geq(vars[0],vars[1]);
	case GreaterXY: 
		return gt(vars[0],vars[1]);
	case LessOrEqualXY: 
		return leq(vars[0],vars[1]);
	case LessXY: 
		return lt(vars[0],vars[1]);

		// N-queens
	case OutDiag1: 
		return neq(plus(vars[0],constraint.getConstant(0)),plus(vars[1],constraint.getConstant(1)));
	case OutDiag2: 
		return neq(minus(vars[0],constraint.getConstant(0)),minus(vars[1],constraint.getConstant(1)));
	case InDiag1: 
		return eq(plus(vars[0],constraint.getConstant(0)),plus(vars[1],constraint.getConstant(1)));
	case InDiag2: 
		return eq(minus(vars[0],constraint.getConstant(0)),minus(vars[1],constraint.getConstant(1)));
	case AbsOutDiag: 
		return neq(abs(minus(vars[0],vars[1])), constraint.getConstant(0));
	case AbsInDiag: 
		return eq(abs(minus(vars[0],vars[1])), constraint.getConstant(0));
		
		//Zebra
	case Next: 
		return eq(vars[0],plus(vars[1],1));
	case NNext: 
		return neq(vars[0],plus(vars[1],1));
	case Prev: 
		return eq(vars[0],plus(vars[1],-1));
	case NPrev: 
		return neq(vars[0],plus(vars[1],-1));
	case Neighbour:
		return eq(vars[0],minus(vars[1],1));
	case NotNeighbour:
		return neq(vars[0],minus(vars[1],1));
	case Neighbour2:
		return eq(vars[0],minus(vars[1],2));
	case NotNeighbour2:
		return neq(vars[0],minus(vars[1],2));
	case Neighbour3:
		return eq(vars[0],minus(vars[1],3));
	case NotNeighbour3:
		return neq(vars[0],minus(vars[1],3));
	case Neighbour4:
		return eq(vars[0],minus(vars[1],4));
	case NotNeighbour4:
		return neq(vars[0],minus(vars[1],4));
	case  At1:
		return eq(vars[0],1);
	case  At2:
		return eq(vars[0],2);
	case At3:
		return eq(vars[0],3);
	case  At4:
		return eq(vars[0],4);
	case  At5:
		return eq(vars[0],5);
	case  notAt1:
		return neq(vars[0],1);
	case  notAt2:
		return neq(vars[0],2);
	case notAt3:
		return neq(vars[0],3);
	case  notAt4:
		return neq(vars[0],4);
	case  notAt5:
		return neq(vars[0],5);
		//return eq(abs(minus(scope[0],scope[1])),1);
		//Global
	case AllDiff:
		return allDifferent(vars);
		//Allen
	case Before: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[1],vars[2]));
	case BeforeInv: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[3],vars[0]));
	case During: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[3],vars[1]),gt(vars[2],vars[0]));
	case DuringInv: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[1],vars[3]),gt(vars[0],vars[2]));
	case Equals: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[0],vars[2]),eq(vars[1],vars[3]));
	case Finish: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[0],vars[2]),eq(vars[1],vars[3]));
	case FinishInv: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[0],vars[2]),eq(vars[1],vars[3]));
	case Meet: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[1],vars[2]));
	case MeetInv: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[3],vars[0]));
	case Overlap: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[1],vars[2]),gt(vars[3],vars[1]));
	case OverlapInv: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[3],vars[0]),gt(vars[1],vars[3]));
	case Start: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[0],vars[2]),gt(vars[3],vars[1]));
	case StartInv: 
		return and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[0],vars[2]),gt(vars[1],vars[3]));

		//Allen complement
	case NotBefore: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[1],vars[2])));
	case NotBeforeInv: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[3],vars[0])));
	case NotDuring: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[3],vars[1]),gt(vars[2],vars[0])));
	case NotDuringInv: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),lt(vars[1],vars[3]),gt(vars[0],vars[2])));
	case NotEquals: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[0],vars[2]),eq(vars[1],vars[3])));
	case NotFinish: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[0],vars[2]),eq(vars[1],vars[3])));
	case NotFinishInv: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[0],vars[2]),eq(vars[1],vars[3])));
	case NotMeet: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[1],vars[2])));
	case NotMeetInv: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[3],vars[0])));
	case NotOverlap: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[1],vars[2]),gt(vars[3],vars[1])));
	case NotOverlapInv: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),gt(vars[3],vars[0]),gt(vars[1],vars[3])));
	case NotStart: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[0],vars[2]),gt(vars[3],vars[1])));
	case NotStartInv: 
		return not(and(leq(vars[1],vars[0]),leq(vars[3],vars[2]),eq(vars[0],vars[2]),gt(vars[1],vars[3])));


		//SchursLemma
	case AllDiffXYZ: 
		return and(neq(vars[0],vars[1]), neq(vars[0],vars[2]),neq(vars[1],vars[2]));    
	case AllEqualXYZ: 
		return and(eq(vars[0],vars[1]),eq(vars[1],vars[2]));  
	case NotAllDiffXYZ: 
		return or(eq(vars[0],vars[1]), eq(vars[0],vars[2]), eq(vars[1],vars[2]));   
	case NotAllEqualXYZ: 
		return or(neq(vars[0],vars[1]), neq(vars[0],vars[2]),neq(vars[1],vars[2])); 
	case DistanceXYZ:
		return eq(abs(minus(vars[0],vars[1])), abs(minus(vars[1],vars[2])));
	case NotDistanceXYZ:
		return neq(abs(minus(vars[0],vars[1])), abs(minus(vars[1],vars[2])));
	case DistanceXYZT:
		return eq(abs(minus(vars[0],vars[1])), abs(minus(vars[2],vars[3])));
	case NotDistanceXYZT:
		return neq(abs(minus(vars[0],vars[1])), abs(minus(vars[2],vars[3])));
	case GreaterXYZT:
		return gt(minus(vars[1],vars[0]), minus(vars[3],vars[2]));
	case LessOrEqualXYZT:
		return leq(minus(vars[1],vars[0]), minus(vars[3],vars[2]));

		//Graceful graph
	case AbsDistanceXYZ:
		return eq(abs(minus(vars[0], vars[1])),vars[2]);
	case NotAbsDistanceXYZ:
		return neq(abs(minus(vars[0], vars[1])),vars[2]);

		//meeting scheduling 
	case ArrivalConstraint :
		return leq(abs(minus(vars[0], vars[1])), constraint.getConstant(0));
	case NotArrivalConstraint :
		return gt(abs(minus(vars[0], vars[1])), constraint.getConstant(0));

		// RFLAP
	case DistanceSup2:
		return gt(abs(minus(vars[0], vars[1])), 1);
	case DistanceSup3:
		return gt(abs(minus(vars[0], vars[1])), 2);
	case DistanceInfEq2:
		return leq(abs(minus(vars[0], vars[1])), 1);
	case DistanceInfEq3:
		return leq(abs(minus(vars[0], vars[1])), 2);
        
    //Langford
    case Distance2:
    	return eq(abs(minus(vars[0], vars[1])), 2);
    case NotDistance2:
    	return neq(abs(minus(vars[0], vars[1])), 2);
    case Distance3:
    	return eq(abs(minus(vars[0], vars[1])), 3);
    case NotDistance3:
    	return neq(abs(minus(vars[0], vars[1])), 3);
    case Distance4:
    	return eq(abs(minus(vars[0], vars[1])), 4);
    case NotDistance4:
    	return neq(abs(minus(vars[0], vars[1])), 4);
    case Distance5:
    	return eq(abs(minus(vars[0], vars[1])), 5);
    case NotDistance5:
    	return neq(abs(minus(vars[0], vars[1])), 5);
    case Distance6:
    	return eq(abs(minus(vars[0], vars[1])), 6);
    case NotDistance6:
    	return neq(abs(minus(vars[0], vars[1])), 6);
    	
    //General distance constraint
    case Distance:
    	return eq(abs(minus(vars[0], vars[1])), constraint.getConstant(0));
    case NotDistance:
    	return neq(abs(minus(vars[0], vars[1])), constraint.getConstant(0));
	case DistanceSup:
		return gt(abs(minus(vars[0], vars[1])), constraint.getConstant(0));
	case DistanceInfEq:
		return leq(abs(minus(vars[0], vars[1])), constraint.getConstant(0));
		
	//RoundOfGolf
	case ScoredPlus10:
		return eq(vars[0], plus(vars[1], 10));
	case NotScoredPlus10:
		return not(eq(vars[0], plus(vars[1], 10)));

//	case ScoredPlus4or7:
//		//System.out.println("The scope : "+vars[0] +"  " + vars[1] +"   "+ vars[2]);
//		return or(and(eq(vars[0], plus(vars[1],7)), eq(vars[2], plus(vars[1], 4))),
//				and(eq(vars[0], plus(vars[1], 4)), eq(vars[2], plus(vars[1], 7))));
	case ScoredPlus4or7:
		//System.out.println("The scope : "+vars[0] +"  " + vars[1] +"   "+ vars[2]);
		return ifThenElse(eq(vars[0],plus(vars[1],7)), eq(vars[2], plus(vars[1], 4)), eq(vars[2], plus(vars[1], 7)));

	case NotScoredPlus4or7:
		return ifThenElse(neq(vars[0],plus(vars[1],7)), neq(vars[2], plus(vars[1], 4)), neq(vars[2], plus(vars[1], 7)));
//		return not(or(and(eq(vars[0], plus(vars[1],7)), eq(vars[2], plus(vars[1], 4))),
//				and(eq(vars[0], plus(vars[1], 4)), eq(vars[2], plus(vars[1], 7)))));
//		return and(or(neq(vars[0], plus(vars[1], 4)), neq(vars[2], plus(vars[1], 7))),
//				or(neq(vars[0], plus(vars[1], 7)), neq(vars[2], plus(vars[1], 4))));
        
	default:
		throw new RuntimeException("unknow constraint type in class AcqConstraint");    
	}
}
}