/***************************************************************
 * Bias class
 * 
 * Version Space modelisation
 * 
 * @author LAZAAR
 * @date 29-11-16
 ***************************************************************/

package machineLearning;

import java.util.*;

import acqConstraints.AcqConstraint;
import vocabulary.*;
import exceptions.*;

public class Bias extends AcqNetwork{


	////////////////////////////////////////////////////
	///
	///				ATTRIBUTEs
	///
	////////////////////////////////////////////////////	


	private Language language;



	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				CONSTRUCTORs
	///
	////////////////////////////////////////////////////


	public Bias(Language lang, SortedSet<AcqVariable> vars) {
		super(vars);
		this.language=lang;
		this.min_arity=lang.min_arity;
		this.constraints=build_constraints();

	}




	public Bias(SortedSet<AcqConstraint> constraints_on_scope, SortedSet<AcqVariable> scope) {
		super(scope, constraints_on_scope);
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------
	////////////////////////////////////////////////////
	///
	///				METHODs
	///
	////////////////////////////////////////////////////


	private SortedSet<AcqConstraint> build_constraints() {
		// TODO Auto-generated method stub
		return null;
	}




	/*************************************************
	 * 	get Kappa of a given example
	 * kappa= constraints violated by the given example
	 * 
	 * @date 14-02-17
	 * @author LAZAAR
	 *************************************************/

	public Collection<AcqConstraint> getKappa(Query ex) {
		Collection<AcqConstraint> kappa = new TreeSet<AcqConstraint>();

		for(AcqConstraint c : this.constraints) {    
			try {
				int[] tuple = ex.getTuple((ArrayList<AcqVariable>)c.getVariables());
				if (!c.checker(tuple)) {
					kappa.add(c);     
				}
			} catch (NotInstanciatedException e) {
				System.out.print(e);
			}
		}
		return kappa;
	}

	/*************************************************
	 * 	add variables
	 * @date 14-02-17
	 * @author LAZAAR
	 *************************************************/

	public void addVar(AcqVariable v) {
		variables.add(v);
	}    



	public SortedSet<AcqConstraint> getProjection(List<AcqVariable> scope) {

		SortedSet<AcqConstraint> bias_y=new TreeSet<AcqConstraint>();

		Iterator<AcqConstraint> iterator = constraints.iterator();

		while( iterator.hasNext()) {
			AcqConstraint c= (AcqConstraint) iterator.next();

			if (scope.containsAll(c.getVariables())) {					

				bias_y.add(c);
			}

		}
		return bias_y;
	}


	public  void reduce(Query e) {

		reduce( (SortedSet<AcqConstraint>)getKappa(e));
	}

	public void reduce(Collection<AcqConstraint> csts) {

		this.constraints.removeAll(csts);

	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


	////////////////////////////////////////////////////
	///
	///				OVERRIDED METHODs
	///
	////////////////////////////////////////////////////


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


	////////////////////////////////////////////////////
	///
	///				GETTERs / SETTERs
	///
	////////////////////////////////////////////////////


	public SortedSet<AcqConstraint> getConstraints(){
		return this.constraints;
	}


	public SortedSet<AcqVariable> getVars() {

		return this.variables;
	}


	public List<AcqConstraint> getBias() {
		// TODO Auto-generated method stub
		return (List<AcqConstraint>) constraints;
	}


	public int getNbVariables() {
		return variables.size();
	}

	public int getNbConstraints() {
		return constraints.size();
	}
	=======================================================================

			=======================================================================

			=======================================================================



			public SortedSet<AcqVariable> getVariablesList() {
		return this.variables;
	}

	public AcqVariable[] getVariables() {
		AcqVariable[] cvars = new AcqVariable[variables.size()];
		return variables.toArray(cvars);
	}


	@Override
	public String toString() {
		String st = "";
		for (AcqVariable v : variables) {
			st += v + "\n";
		}        
		for (ConacqConstraint constraint : constraints) {
			st += constraint + "\n";
		}      
		for (Litteral lit : getLitterals()) {
			st += lit.pretty() + "\n";
		}
		return st;
	}

	public Litteral getLitteral(int dimacsLitteral) {
		assert dimacsLitteral > 0;
		return litterals.get(dimacsLitteral);
	}



	public List<Litteral> getLitterals() {
		//The first litteral is null in order to enforce equality between index and dimacs litteral
		return litterals.subList(1, litterals.size());
	}

	public boolean belongs(Litteral litteral) {
		return litteral.getConstraint().equals(getLitteral(litteral.getDimacs()).getConstraint());
	}

	public Example builRandomExample() {
		Example ex = new Example();
		for (AcqVariable v : variables) {
			ex.setValue(v, v.getRandomValue());
		}
		return ex;
	}

	public Interpretation getInterpretation(Learner learner) {
		Interpretation interpretation = new Interpretation();
		for (ConacqConstraint c : learner.getConstraints()) {
			interpretation.add(constraint2Lit.get(c));
		}
		return interpretation;
	}



	public Bias() {
		variables = new ArrayList<AcqVariable>();
		latticeVar= new ArrayList<TypeVar>();
		listOfConstaintType = new ArrayList<Type>();

		constraints = new ArrayList<ConacqConstraint>();
		litterals = new ArrayList<Litteral>();
		litterals.add(null);
		constraint2Lit = new HashMap<ConacqConstraint, Litteral>();
		complement = new HashMap<Litteral, Litteral>();
	}

	public ArrayList<Type> getListOfConstaintType() {
		return listOfConstaintType;
	}

	public static Bias buildAllenBias(int nbEvents) {
		return new Bias("Allen",nbEvents);
	}

	//dedicated to temporal bias
	private Bias(String st, int nbEvents) { 
		this();
		for (int i = 0; i < nbEvents; i++) {
			add(new AcqVariable("start_"+i,0,nbEvents)); 
			add(new AcqVariable("end_"+i,0,nbEvents));
		}
		//Build the litterals
		Library lib = Library.factory(Name.ALLEN);
		libraries = new ArrayList<Library>();
		libraries.add(lib);
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-2 ; i+=2){
			for (int j=i+2 ; j<nbVars ; j+=2){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(i+1)); vars.add(variables.get(j)); vars.add(variables.get(j+1));
				buildAllConstraint(vars, lib);
			}
		}
	}



	public void setBigLibrary(){

		Library lib = Library.factory(Name.BIG_BIAS);

		libraries = new ArrayList<Library>();
		libraries.add(lib);

		int nbVars = variables.size();

		for (int i=0 ; i<nbVars ; i++){
			List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
			vars.add(variables.get(i));
			buildAllBigConstraint(vars, lib,1);
		}

		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildAllBigConstraint(vars, lib, 2);
			}
		}

		for (int i=0 ; i<nbVars-2 ; i++){
			for (int j=i+1 ; j<nbVars-1 ; j++){
				for (int k=j+1 ; k<nbVars ; k++){
					List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
					vars.add(variables.get(i)); vars.add(variables.get(j)); vars.add(variables.get(k));
					buildAllBigConstraint(vars, lib,3);
					vars = new ArrayList<AcqVariable>();
					vars.add(variables.get(i)); vars.add(variables.get(j)); vars.add(variables.get(i)); vars.add(variables.get(k));
					buildGolombConstraint(vars, lib, 4);
				}

			}
		}

		for (int i=0 ; i<nbVars-3 ; i++){
			for (int j=i+1 ; j<nbVars-2 ; j++)
				for (int k=j+1 ; k<nbVars-1 ; k++)
					for (int t=k+1 ; t<nbVars ; t++){
						List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
						vars.add(variables.get(i)); vars.add(variables.get(j)); vars.add(variables.get(k)); vars.add(variables.get(t));
						buildAllBigConstraint(vars, lib,4);
					}
		}
	}

	public void setLibrary(Library lib) {
		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;
		if(lib.getName().equals("Golomb"))
		{
			libraries = new ArrayList<Library>();
			libraries.add(lib);
			int nbVars = variables.size();
			for (int i=0 ; i<nbVars-1 ; i++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(i+1));
				buildGolombConstraint(vars, lib,2);

			}

			for (int i = 0; i < nbVars-2; i++){
				for (int j = i+1; j < nbVars-1; j++){
					for (int k = j+1; k < nbVars; k++){
						List<AcqVariable> vars = new ArrayList<AcqVariable>();
						vars.add(variables.get(i));
						vars.add(variables.get(j));
						vars.add(variables.get(k));
						buildGolombConstraint(vars, lib,3);
					}
				}
			}

			for (int i = 0; i < nbVars-3; i++){
				for (int j = i+1; j < nbVars-2; j++){
					for (int k = j+1; k < nbVars-1; k++){
						for (int l = k+1; l < nbVars; l++){
							List<AcqVariable> vars = new ArrayList<AcqVariable>();
							vars.add(variables.get(i));vars.add(variables.get(j));
							vars.add(variables.get(k));vars.add(variables.get(l));
							buildGolombConstraint(vars, lib,4);
						}
					}
				}
			}
		}
		else if (lib.arity == 2) {
			libraries = new ArrayList<Library>();
			libraries.add(lib);
			assert lib.getArity() == 2;
			int nbVars = variables.size();

			for (int i=0 ; i<nbVars-1 ; i++){
				for (int j=i+1 ; j<nbVars ; j++){
					List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
					vars.add(variables.get(i)); vars.add(variables.get(j));
					buildAllConstraint(vars, lib);
				}
			}
		}
		else {
			List<Library> libs = new ArrayList<Library>();
			libs.add(lib);
			setLibraries(libs);
		}

		// Golomb rulers

	}

	public void setMeetingNoDLibrary(){ 

		Library lib = Library.factory(Name.MEETING_NOD);
		libraries = new ArrayList<Library>();
		libraries.add(lib);

		int nbVars = variables.size();

		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildAllMeetingConstraint(vars, lib, 0, 0);
			}
		}
	}

	public void setMeetingLibrary( int minDisTimeBetweenMeetings, int maxDisTimeBetweenMeetings) { 

		Library lib = Library.factory(Name.MEETING);
		libraries = new ArrayList<Library>();
		libraries.add(lib);

		int nbVars = variables.size();

		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildAllMeetingConstraint(vars, lib, minDisTimeBetweenMeetings, maxDisTimeBetweenMeetings);
			}
		}
	}

	public void setNqueensLibrary(Library lib){

		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;

		libraries = new ArrayList<Library>();
		libraries.add(lib);
		assert lib.getArity() == 2;
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildNQueensConstraints(vars, lib, j - i);
			}
		}
	}

	public void setZebraLibrary(Library lib){

		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;

		libraries = new ArrayList<Library>();
		libraries.add(lib);
		assert lib.getArity() == 2;
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildZebraConstraint(vars, lib,2);
			}


		}

		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++)
				for (int k=j+1 ; k<nbVars ; k++){
					List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
					vars.add(variables.get(i)); vars.add(variables.get(j)); vars.add(variables.get(k));
					buildZebraConstraint(vars, lib,3);
				}


		}

		for (int i=0 ; i<nbVars ; i++){
			List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
			vars.add(variables.get(i));
			buildZebraConstraint(vars, lib,1);
		}

	}

	public void setRandom_RALibrary(Library lib){

		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;

		libraries = new ArrayList<Library>();
		libraries.add(lib);
		assert lib.getArity() == 2;
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildRandom_RAConstraint(vars, lib,2);
			}


		}

		for (int i=0 ; i<nbVars-2 ; i++){
			for (int j=i+1 ; j<nbVars-1 ; j++)
				for (int k=j+1 ; k<nbVars ; k++){
					List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
					vars.add(variables.get(i)); vars.add(variables.get(j)); vars.add(variables.get(k));
					buildRandom_RAConstraint(vars, lib,3);
				}


		}

		for (int i=0 ; i<nbVars-3 ; i++){
			for (int j=i+1 ; j<nbVars-2 ; j++)
				for (int k=j+1 ; k<nbVars-1 ; k++)
					for (int t=k+1 ; t<nbVars ; t++){
						List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
						List<AcqVariable> vars2 = new ArrayList<AcqVariable>(); 
						vars.add(variables.get(i)); vars.add(variables.get(j)); vars.add(variables.get(k)); vars.add(variables.get(t));
						vars2.add(variables.get(i)); vars2.add(variables.get(k)); vars2.add(variables.get(j)); vars2.add(variables.get(t));
						buildRandom_RAConstraint(vars, lib, 4);
						buildRandom_RAConstraint(vars2, lib, 4);
					}


		}

		for (int i=0 ; i<nbVars ; i++){
			List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
			vars.add(variables.get(i));
			buildRandom_RAConstraint(vars, lib,1);
		}

	}

	public void setToyLibrary(Library lib){

		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;

		libraries = new ArrayList<Library>();
		libraries.add(lib);
		assert lib.getArity() == 2;
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildAllConstraint(vars, lib);
			}


		}

	}

	public void setGracefulGraphsLibrary(Library lib){

		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;

		libraries = new ArrayList<Library>();
		libraries.add(lib);
		assert lib.getArity() == 2;
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildGGConstraint(vars, lib );
			} 
		}

		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++)
				for (int k=j+1 ; k<nbVars ; k++){
					List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
					vars.add(variables.get(i)); vars.add(variables.get(j)); vars.add(variables.get(k));
					buildGGConstraint(vars, lib );
				} 
		}
	}

	public void setLibraries(List<Library> libs) {
		assert this.libraries == null;
		this.libraries = libs;
		System.out.println(variables);
		for (Library lib : libs) {
			Iterator<int[]> combinations;
			if (lib.isDirected()) combinations = new PermutationIterator(variables.size(),lib.getArity()); 
			else combinations = new CombinationIterator(variables.size(),lib.getArity());
			int[] indices;
			while (combinations.hasNext()) {
				indices = combinations.next();
				List<AcqVariable> vars = new ArrayList<AcqVariable>();
				for (int i = 0; i < indices.length; i++) 
					vars.add(variables.get(indices[i]));
				buildAllConstraint(vars, lib);
			}
		}
	}


	public void setListOfConstaintType(ArrayList<Type> listOfConstaintType) {
		this.listOfConstaintType = listOfConstaintType;
	}


	public ConacqConstraint buildConstraint(Type constraintType, AcqVariable[] variables) {
		ConacqConstraint c=null;
		if(constraintType.equals(Type.InDiag1)||constraintType.equals(Type.InDiag2)||constraintType.equals(Type.OutDiag1)||constraintType.equals(Type.OutDiag2)){
			int n= variables[0].getMax();

			for(int i=0; i<n-1; i++)
				for(int j=i+1; j<n; j++)
				{
					c=new ConacqConstraint(constraintType,variables[0],variables[1],i,j);
					constraints.add(c);
					Litteral l = new Litteral(c,litteralCounter++);
					litterals.add(l);
					constraint2Lit.put(c, l);

				}

		}
		else if(constraintType.equals(Type.AbsInDiag)||constraintType.equals(Type.AbsOutDiag)
				||constraintType.equals(Type.Distance)||constraintType.equals(Type.NotDistance)
				||constraintType.equals(Type.DistanceSup)||constraintType.equals(Type.DistanceInfEq)){
			int n= Math.max(variables[0].getMax() - variables[1].getMin(), variables[1].getMax() - variables[0].getMin());

			for(int i=1; i<=n; i++){
				c=new ConacqConstraint(constraintType,variables[0],variables[1],i);
				constraints.add(c);
				Litteral l = new Litteral(c,litteralCounter++);
				litterals.add(l);
				constraint2Lit.put(c, l);

			}

		}
		else if(constraintType.equals(Type.EqualX)||constraintType.equals(Type.DiffX)){
			int[] dom= variables[0].getDomain();

			for(int val : dom){
				c=new ConacqConstraint(constraintType,variables[0],val);
				constraints.add(c);
				Litteral l = new Litteral(c,litteralCounter++);
				litterals.add(l);
				constraint2Lit.put(c, l);

			}

		}
		else{		
			c = new ConacqConstraint(constraintType,variables);
			constraints.add(c);
			Litteral l = new Litteral(c,litteralCounter++);
			litterals.add(l);
			constraint2Lit.put(c, l);

		}
		return c;
	}



	public ConacqConstraint buildNQueensConstraint(Type constraintType, AcqVariable[] variables, int dist) {
		ConacqConstraint c=null;
		if(constraintType.equals(Type.InDiag1)||constraintType.equals(Type.InDiag2)||constraintType.equals(Type.OutDiag1)||constraintType.equals(Type.OutDiag2)){
			int n= variables[0].getMax();

			for(int i=0; i<n-1; i++)
				for(int j=i+1; j<n; j++)
				{
					c=new ConacqConstraint(constraintType,variables[0],variables[1],i,j);
					constraints.add(c);
					Litteral l = new Litteral(c,litteralCounter++);
					litterals.add(l);
					constraint2Lit.put(c, l);

				}

		}
		else if(constraintType.equals(Type.AbsInDiag)||constraintType.equals(Type.AbsOutDiag)){

			c=new ConacqConstraint(constraintType,variables[0],variables[1],dist);
			constraints.add(c);
			Litteral l = new Litteral(c,litteralCounter++);
			litterals.add(l);
			constraint2Lit.put(c, l);

		}
		else if(constraintType.equals(Type.EqualX)||constraintType.equals(Type.DiffX)){
			int[] dom= variables[0].getDomain();

			for(int val : dom){
				c=new ConacqConstraint(constraintType,variables[0],val);
				constraints.add(c);
				Litteral l = new Litteral(c,litteralCounter++);
				litterals.add(l);
				constraint2Lit.put(c, l);

			}

		}
		else{		
			c = new ConacqConstraint(constraintType,variables);
			constraints.add(c);
			Litteral l = new Litteral(c,litteralCounter++);
			litterals.add(l);
			constraint2Lit.put(c, l);

		}
		return c;
	}

	private void buildAllMeetingConstraint(List<AcqVariable> vars, Library lib, int minDisTimeBetweenMeetings,
			int maxDisTimeBetweenMeetings) {
		for (Type t : lib.getConstraintTypes()) {
			if(!t.equals(Type.ArrivalConstraint) && !t.equals(Type.NotArrivalConstraint))
				buildConstraint(t, vars.toArray(new AcqVariable[vars.size()]));
			else{
				for(int d=minDisTimeBetweenMeetings; d<=maxDisTimeBetweenMeetings;d++){
					ConacqConstraint c = new ConacqConstraint(t, vars.get(0), vars.get(1), d);
					constraints.add(c);
					Litteral l = new Litteral(c, litteralCounter++);
					litterals.add(l);
					constraint2Lit.put(c, l);
				}
			}
		}
		for (Type c : lib.getConstraintTypes()) {
			Litteral l1= getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);

			Type oc = lib.getComplement(c);
			if (oc != null) { 
				if(!oc.equals(Type.ArrivalConstraint) && !oc.equals(Type.NotArrivalConstraint)){
					l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
					l1.setComplement(l2);
					l2.setComplement(l1);
					//	litterals.add(l1);
					//	litterals.add(l2);
					complement.put(l1, l2);
				}else{
					for(int d=minDisTimeBetweenMeetings; d<=maxDisTimeBetweenMeetings;d++){
						l1 = getLitteral(vars.get(0), vars.get(1), d , c);
						Litteral l2 = getLitteral(vars.get(0), vars.get(1),  d ,oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						complement.put(l1, l2); 
					}
				}
			}
			else {
				complement.put(l1, null);
			}
		}
	}

	private void buildAllConstraint(List<AcqVariable> vars, Library lib) { 
		for (Type c : lib.getConstraintTypes()) {
			buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
		}
		for (Type c : lib.getConstraintTypes()) {
			Litteral l1= getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
			;
			Type oc = lib.getComplement(c);
			if (oc != null) {
				if(oc.equals(Type.InDiag1)||oc.equals(Type.InDiag2)||oc.equals(Type.OutDiag1)||oc.equals(Type.OutDiag2)){
					int n= variables.get(0).getMax();
					for(int i=0; i<n-1; i++)
						for(int j=i+1; j<n; j++)
						{
							l1 = getLitteral(vars.get(0),vars.get(1),c,i,j);
							Litteral l2 = getLitteral(vars.get(0),vars.get(1),oc,i,j);
							l1.setComplement(l2);
							l2.setComplement(l1);
							//	litterals.add(l1);
							//	litterals.add(l2);
							complement.put(l1, l2);
						}

				}
				else if(oc.equals(Type.AbsOutDiag)||oc.equals(Type.AbsInDiag)){
					int n= variables.get(0).getMax();
					for(int i=1; i<=n; i++){
						l1 = getLitteral(vars.get(0),vars.get(1),i,c);
						Litteral l2 = getLitteral(vars.get(0),vars.get(1),i,oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}

				}
				else{
					l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
					l1.setComplement(l2);
					l2.setComplement(l1);
					//	litterals.add(l1);
					//	litterals.add(l2);
					complement.put(l1, l2);
				}
			}
			else {
				complement.put(l1, null);
			}
		}
	}



	private void buildNQueensConstraints(List<AcqVariable> vars, Library lib, int dist) { 
		for (Type c : lib.getConstraintTypes()) {
			buildNQueensConstraint(c,vars.toArray(new AcqVariable[vars.size()]), dist);
		}
		for (Type c : lib.getConstraintTypes()) {
			Litteral l1= getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
			;
			Type oc = lib.getComplement(c);
			if (oc != null) {
				if(oc.equals(Type.InDiag1)||oc.equals(Type.InDiag2)||oc.equals(Type.OutDiag1)||oc.equals(Type.OutDiag2)){
					int n= variables.get(0).getMax();
					for(int i=0; i<n-1; i++)
						for(int j=i+1; j<n; j++)
						{
							l1 = getLitteral(vars.get(0),vars.get(1),c,i,j);
							Litteral l2 = getLitteral(vars.get(0),vars.get(1),oc,i,j);
							l1.setComplement(l2);
							l2.setComplement(l1);
							//	litterals.add(l1);
							//	litterals.add(l2);
							complement.put(l1, l2);
						}

				}
				else if(oc.equals(Type.AbsOutDiag)||oc.equals(Type.AbsInDiag)){
					l1 = getLitteral(vars.get(0),vars.get(1),dist,c);
					Litteral l2 = getLitteral(vars.get(0),vars.get(1),dist,oc);
					l1.setComplement(l2);
					l2.setComplement(l1);
					//	litterals.add(l1);
					//	litterals.add(l2);
					complement.put(l1, l2);

				}
				else{
					l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
					l1.setComplement(l2);
					l2.setComplement(l1);
					//	litterals.add(l1);
					//	litterals.add(l2);
					complement.put(l1, l2);
				}
			}
			else {
				complement.put(l1, null);
			}
		}
	}



	/**************************************************************************************
	 * Graceful graphs Bias
	 * @param vars
	 * @param lib
	 * @author arcangiolirobin
	 **************************************************************************************/
	private void buildGGConstraint(List<AcqVariable> vars, Library lib) { 
		for (Type c : lib.getConstraintTypes()) {

			if(((!c.equals(Type.AbsDistanceXYZ) && !c.equals(Type.NotAbsDistanceXYZ)) || vars.size() == 3) &&
					((!c.equals(Type.EqualXY) && !c.equals(Type.DifferentXY)) || vars.size() == 2))
				buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
		}
		for (Type c : lib.getConstraintTypes()) {
			if(((!c.equals(Type.AbsDistanceXYZ) && !c.equals(Type.NotAbsDistanceXYZ)) || vars.size() == 3) &&
					((!c.equals(Type.EqualXY) && !c.equals(Type.DifferentXY)) || vars.size() == 2)){
				Litteral l1= getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
				;
				Type oc = lib.getComplement(c);
				if (oc != null) {
					if(oc.equals(Type.InDiag1)||oc.equals(Type.InDiag2)||oc.equals(Type.OutDiag1)||oc.equals(Type.OutDiag2)){
						int n= variables.get(0).getMax();
						for(int i=0; i<n-1; i++)
							for(int j=i+1; j<n; j++)
							{
								l1 = getLitteral(vars.get(0),vars.get(1),c,i,j);
								Litteral l2 = getLitteral(vars.get(0),vars.get(1),oc,i,j);
								l1.setComplement(l2);
								l2.setComplement(l1);
								//	litterals.add(l1);
								//	litterals.add(l2);
								complement.put(l1, l2);
							}

					}
					else{
						l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
				}
				else {
					complement.put(l1, null);
				}
			}
		}
	}

	/**************************************************************************************
	 * Golomb rulers Bias
	 * @param vars
	 * @param lib
	 * @author lazaarnadjib
	 **************************************************************************************/

	private void buildGolombConstraint(List<AcqVariable> vars, Library lib, int arity) { 

		if(arity==2)
		{
			for (Type c : lib.getConstraintTypes()) 

				if( !c.equals(Type.DistanceXYZT) && !c.equals(Type.NotDistanceXYZT) && !c.equals(Type.GreaterXYZT) 
						&& !c.equals(Type.DistanceXYZ) && !c.equals(Type.NotDistanceXYZ))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if( !c.equals(Type.DistanceXYZT) && !c.equals(Type.NotDistanceXYZT) && !c.equals(Type.GreaterXYZT)
						&& !c.equals(Type.DistanceXYZ) && !c.equals(Type.NotDistanceXYZ))
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);			//FIXED: double add
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}
		}
		else if(arity == 3){
			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.DistanceXYZ) || c.equals(Type.NotDistanceXYZ))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.DistanceXYZ) || c.equals(Type.NotDistanceXYZ))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}	
		}
		else{
			for (Type c : lib.getConstraintTypes()) 
				if( c.equals(Type.DistanceXYZT) || c.equals(Type.NotDistanceXYZT) || c.equals(Type.GreaterXYZT))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if( c.equals(Type.DistanceXYZT) || c.equals(Type.NotDistanceXYZT) || c.equals(Type.GreaterXYZT))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}	
		}



	}


	/**************************************************************************************
	 * Zebra Bias
	 * @param vars
	 * @param lib
	 * @author lazaarnadjib
	 **************************************************************************************/

	private void buildZebraConstraint(List<AcqVariable> vars, Library lib, int arity) { 

		if(arity==2)
		{
			for (Type c : lib.getConstraintTypes()) 

			{		
				if( !c.equals(Type.At1) && !c.equals(Type.At2)&& !c.equals(Type.At3)&& !c.equals(Type.At4)&& !c.equals(Type.At5) && 
						!c.equals(Type.notAt1) && !c.equals(Type.notAt2)&& !c.equals(Type.notAt3)&& !c.equals(Type.notAt4)&& !c.equals(Type.notAt5)
						&& !c.equals(Type.AllDiffXYZ)&& !c.equals(Type.AllEqualXYZ)&& !c.equals(Type.NotAllDiffXYZ)&& !c.equals(Type.NotAllEqualXYZ))
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
			}	

			for (Type c : lib.getConstraintTypes()) 
				if( !c.equals(Type.At1) && !c.equals(Type.At2)&& !c.equals(Type.At3)&& !c.equals(Type.At4)&& !c.equals(Type.At5) && 
						!c.equals(Type.notAt1) && !c.equals(Type.notAt2)&& !c.equals(Type.notAt3)&& !c.equals(Type.notAt4)&& !c.equals(Type.notAt5)
						&& !c.equals(Type.AllDiffXYZ)&& !c.equals(Type.AllEqualXYZ)&& !c.equals(Type.NotAllDiffXYZ)&& !c.equals(Type.NotAllEqualXYZ))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//		litterals.add(l1);
						//		litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}
		}
		else if(arity==1){
			for (Type c : lib.getConstraintTypes()) 
				if( c.equals(Type.At1) || c.equals(Type.At2)|| c.equals(Type.At3)|| c.equals(Type.At4)|| c.equals(Type.At5) ||
						c.equals(Type.notAt1) || c.equals(Type.notAt2)|| c.equals(Type.notAt3)|| c.equals(Type.notAt4)|| c.equals(Type.notAt5))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if( c.equals(Type.At1) || c.equals(Type.At2)|| c.equals(Type.At3)|| c.equals(Type.At4)|| c.equals(Type.At5) ||
						c.equals(Type.notAt1) || c.equals(Type.notAt2)|| c.equals(Type.notAt3)|| c.equals(Type.notAt4)|| c.equals(Type.notAt5))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}	
		}
		else{

			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.AllDiffXYZ) || c.equals(Type.AllEqualXYZ) || c.equals(Type.NotAllDiffXYZ) || c.equals(Type.NotAllEqualXYZ))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.AllDiffXYZ) || c.equals(Type.AllEqualXYZ) || c.equals(Type.NotAllDiffXYZ) || c.equals(Type.NotAllEqualXYZ))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}	

		}



	}



	/**************************************************************************************
	 * Random_RA Bias
	 * @param vars
	 * @param lib
	 * @author lazaarnadjib
	 **************************************************************************************/

	private void buildRandom_RAConstraint(List<AcqVariable> vars, Library lib, int arity) { 

		if(arity==2)
		{
			for (Type c : lib.getConstraintTypes()) 

			{		
				if( !c.equals(Type.At1) && !c.equals(Type.At2)&& !c.equals(Type.At3)&& !c.equals(Type.At4)&& !c.equals(Type.At5) && 
						!c.equals(Type.notAt1) && !c.equals(Type.notAt2)&& !c.equals(Type.notAt3)&& !c.equals(Type.notAt4)&& !c.equals(Type.notAt5)
						&& !c.equals(Type.DistanceXYZ) && !c.equals(Type.NotDistanceXYZ)
						&& !c.equals(Type.DistanceXYZT)&& !c.equals(Type.NotDistanceXYZT))
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
			}	

			for (Type c : lib.getConstraintTypes()) 
				if( !c.equals(Type.At1) && !c.equals(Type.At2)&& !c.equals(Type.At3)&& !c.equals(Type.At4)&& !c.equals(Type.At5) && 
						!c.equals(Type.notAt1) && !c.equals(Type.notAt2)&& !c.equals(Type.notAt3)&& !c.equals(Type.notAt4)&& !c.equals(Type.notAt5)
						&& !c.equals(Type.DistanceXYZ) && !c.equals(Type.NotDistanceXYZ)
						&& !c.equals(Type.DistanceXYZT) && !c.equals(Type.NotDistanceXYZT))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//		litterals.add(l1);
						//		litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}
		}
		else if(arity==1){
			for (Type c : lib.getConstraintTypes()) 
				if( c.equals(Type.At1) || c.equals(Type.At2)|| c.equals(Type.At3)|| c.equals(Type.At4)|| c.equals(Type.At5) ||
						c.equals(Type.notAt1) || c.equals(Type.notAt2)|| c.equals(Type.notAt3)|| c.equals(Type.notAt4)|| c.equals(Type.notAt5))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if( c.equals(Type.At1) || c.equals(Type.At2)|| c.equals(Type.At3)|| c.equals(Type.At4)|| c.equals(Type.At5) ||
						c.equals(Type.notAt1) || c.equals(Type.notAt2)|| c.equals(Type.notAt3)|| c.equals(Type.notAt4)|| c.equals(Type.notAt5))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}	
		}else if(arity==3){

			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.DistanceXYZ) || c.equals(Type.NotDistanceXYZ))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.DistanceXYZ) || c.equals(Type.NotDistanceXYZ))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}	
		}
		else{

			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.DistanceXYZT) || c.equals(Type.NotDistanceXYZT))	
					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));

			for (Type c : lib.getConstraintTypes()) 
				if(c.equals(Type.DistanceXYZT) || c.equals(Type.NotDistanceXYZT))	
				{
					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						//	litterals.add(l1);
						//	litterals.add(l2);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}	

		}



	}



	/**************************************************************************************
	 * A Big Bias
	 * @param vars
	 * @param lib
	 * @author lazaarnadjib
	 **************************************************************************************/

	private void buildAllBigConstraint(List<AcqVariable> vars, Library lib, int arity) { 

		for(Type c : lib.getConstraintTypes()){

			if(arity==1){
				if(c.equals(Type.EqualX) || c.equals(Type.DiffX)){

					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
				}

			}else if(arity==2){
				if(c.equals(Type.EqualXY) || c.equals(Type.DifferentXY) || c.equals(Type.GreaterOrEqualXY)
						|| c.equals(Type.LessXY) || c.equals(Type.LessOrEqualXY) || c.equals(Type.GreaterXY)
						|| c.equals(Type.Prev) || c.equals(Type.NPrev) || c.equals(Type.Next)
						|| c.equals(Type.NNext) || c.equals(Type.Distance) || c.equals(Type.NotDistance)
						|| c.equals(Type.DistanceSup) || c.equals(Type.DistanceInfEq)){

					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
				}

			}else if(arity==3){
				if(c.equals(Type.AllEqualXYZ) || c.equals(Type.NotAllEqualXYZ) || c.equals(Type.AllDiffXYZ)
						|| c.equals(Type.NotAllDiffXYZ) || c.equals(Type.DistanceXYZ)
						|| c.equals(Type.NotDistanceXYZ)){

					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
				}

			}else if(arity==4){
				if(c.equals(Type.Before) || c.equals(Type.NotBefore) || c.equals(Type.BeforeInv)
						|| c.equals(Type.NotBeforeInv) || c.equals(Type.During) || c.equals(Type.NotDuring)
						|| c.equals(Type.DuringInv) || c.equals(Type.NotDuringInv) || c.equals(Type.Equals)
						|| c.equals(Type.NotEquals) || c.equals(Type.Finish) || c.equals(Type.NotFinish)
						|| c.equals(Type.FinishInv) || c.equals(Type.NotFinishInv) || c.equals(Type.Meet)
						|| c.equals(Type.NotMeet) || c.equals(Type.MeetInv) || c.equals(Type.NotMeetInv)
						|| c.equals(Type.Overlap) || c.equals(Type.NotOverlap) || c.equals(Type.OverlapInv)
						|| c.equals(Type.NotOverlapInv) || c.equals(Type.Start) || c.equals(Type.NotStart)
						|| c.equals(Type.StartInv) || c.equals(Type.NotStartInv) || c.equals(Type.DistanceXYZT)
						|| c.equals(Type.NotDistanceXYZT)){

					buildConstraint(c,vars.toArray(new AcqVariable[vars.size()]));
				}

			}
		}

		for(Type c : lib.getConstraintTypes()){

			if(arity==1){
				if(c.equals(Type.EqualX) || c.equals(Type.DiffX)){
					int[] dom= vars.get(0).getDomain();

					for(int val : dom){

						Litteral l1 = getLitteral(vars.get(0),val,c);
						Type oc = lib.getComplement(c);
						if (oc != null) {
							Litteral l2 = getLitteral(vars.get(0),val,oc);
							l1.setComplement(l2);
							l2.setComplement(l1);
							complement.put(l1, l2);
						}
						else {
							complement.put(l1, null);
						}
					}
				}

			}else if(arity==2){
				if(c.equals(Type.EqualXY) || c.equals(Type.DifferentXY) || c.equals(Type.GreaterOrEqualXY)
						|| c.equals(Type.LessXY) || c.equals(Type.LessOrEqualXY) || c.equals(Type.GreaterXY)
						|| c.equals(Type.Prev) || c.equals(Type.NPrev) || c.equals(Type.Next)
						|| c.equals(Type.NNext)){

					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}

				}else if(c.equals(Type.Distance) || c.equals(Type.NotDistance)
						|| c.equals(Type.DistanceSup) || c.equals(Type.DistanceInfEq)){

					int n= Math.max(vars.get(0).getMax() - vars.get(1).getMin(), vars.get(1).getMax() - vars.get(0).getMin());

					//We dont take the 0 distance cuz it's covered by the equal and diff constraints
					for(int i=1; i<=n; i++){
						//Distance n covers distanceSup n-1 and distanceSup n is irrelevant
						//So we take only distanceSup 1 to distanceSup n-2
						if((!c.equals(Type.DistanceSup)) || i < n -1){
							Litteral l1 = getLitteral(vars.get(0),vars.get(1),i,c);
							Type oc = lib.getComplement(c);
							if (oc != null) {
								Litteral l2 = getLitteral(vars.get(0),vars.get(1),i,oc);
								l1.setComplement(l2);
								l2.setComplement(l1);
								complement.put(l1, l2);
							}
							else {
								complement.put(l1, null);
							}
						}
					}
				}

			}else if(arity==3){
				if(c.equals(Type.AllEqualXYZ) || c.equals(Type.NotAllEqualXYZ) || c.equals(Type.AllDiffXYZ)
						|| c.equals(Type.NotAllDiffXYZ) || c.equals(Type.DistanceXYZ)
						|| c.equals(Type.NotDistanceXYZ)){

					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}

			}else if(arity==4){
				if(c.equals(Type.Before) || c.equals(Type.NotBefore) || c.equals(Type.BeforeInv)
						|| c.equals(Type.NotBeforeInv) || c.equals(Type.During) || c.equals(Type.NotDuring)
						|| c.equals(Type.DuringInv) || c.equals(Type.NotDuringInv) || c.equals(Type.Equals)
						|| c.equals(Type.NotEquals) || c.equals(Type.Finish) || c.equals(Type.NotFinish)
						|| c.equals(Type.FinishInv) || c.equals(Type.NotFinishInv) || c.equals(Type.Meet)
						|| c.equals(Type.NotMeet) || c.equals(Type.MeetInv) || c.equals(Type.NotMeetInv)
						|| c.equals(Type.Overlap) || c.equals(Type.NotOverlap) || c.equals(Type.OverlapInv)
						|| c.equals(Type.NotOverlapInv) || c.equals(Type.Start) || c.equals(Type.NotStart)
						|| c.equals(Type.StartInv) || c.equals(Type.NotStartInv) || c.equals(Type.DistanceXYZT)
						|| c.equals(Type.NotDistanceXYZT)){

					Litteral l1 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),c);
					Type oc = lib.getComplement(c);
					if (oc != null) {
						Litteral l2 = getLitteral(vars.toArray(new AcqVariable[vars.size()]),oc);
						l1.setComplement(l2);
						l2.setComplement(l1);
						complement.put(l1, l2);
					}
					else {
						complement.put(l1, null);
					}
				}

			}
		}
	}

	public  int getMinArity() {
		return minArity;
	}

	static AcqVariable[]  reverse(AcqVariable[] vars)
	{
		AcqVariable[] result = new AcqVariable[vars.length];
		int j=0;
		for(int i=vars.length-1; i>=0; i--)
		{
			result[j]=vars[i];
			j++;
		}	
		return result;
	}


	/***********************************************************
	 * GENERALIZATION PART
	 * 
	 * @author LAZAAR
	 * @date 21-04-15
	 **********************************************************/


	public void setTypeVar(ArrayList<TypeVar> latticeVar2) {

		latticeVar=latticeVar2;

	}

	public ArrayList<TypeVar> getTypeVar() {

		return latticeVar;

	}
	public TypeVar getTop( ) {
		return Top;
	}

	public void setTop(TypeVar top) {
		Top=new TypeVar(top.getName(), top.getVars(), top.getGeneral(), top.getSpecific());
	}


	public void setVesselLoadingLibrary(Library lib){

		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;

		libraries = new ArrayList<Library>();
		libraries.add(lib);
		assert lib.getArity() == 2;
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildAllConstraint(vars, lib);
			}
		}

	}
	public void setFrequencyAllocation(Library lib) {
		// TODO Auto-generated method stub
		assert this.libraries == null;
		this.minArity = lib.arity; //FIXME multiple arity;

		libraries = new ArrayList<Library>();
		libraries.add(lib);
		assert lib.getArity() == 2;
		int nbVars = variables.size();
		for (int i=0 ; i<nbVars-1 ; i++){
			for (int j=i+1 ; j<nbVars ; j++){
				List<AcqVariable> vars = new ArrayList<AcqVariable>(); 
				vars.add(variables.get(i)); vars.add(variables.get(j));
				buildAllConstraint(vars, lib);
			}


		}
	}






}
