package vocabulary;

import java.util.Collection;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import acqConstraints.AcqConstraint;
import machineLearning.Query;



public class AcqNetwork {


	////////////////////////////////////////////////////
	///
	///				ATTRIBUTEs
	///
	////////////////////////////////////////////////////	

	protected SortedSet<AcqConstraint> constraints;
	protected SortedSet<AcqVariable> variables;
	protected SortedSet<Query> solutions;
	protected int min_arity;
	protected String name="unkonwn";



	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				CONSTRUCTORs
	///
	////////////////////////////////////////////////////

	public AcqNetwork(SortedSet<AcqVariable> vars){
		this.variables=new TreeSet<AcqVariable>(vars);
	}

	public AcqNetwork(SortedSet<AcqVariable> vars, Collection<AcqConstraint> csts){

		this.variables=new TreeSet<AcqVariable>(vars);
		this.constraints=new TreeSet<AcqConstraint>(csts);
		this.min_arity= up_min_arity();

	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------
	////////////////////////////////////////////////////
	///
	///				METHODs
	///
	////////////////////////////////////////////////////



	public AcqNetwork(AcqNetwork learned_network, List<AcqVariable> scope) {

		this.variables=new TreeSet<AcqVariable>(scope);

		for(AcqConstraint c : learned_network.getConstraints()){								// projection of the scope Y on constraint of learned_network (i.e., CL[Y])
			List<AcqVariable> temp_vars=(List<AcqVariable>) c.getVariables();
			boolean mutex=true;
			for(AcqVariable v: temp_vars)
				if(!scope.contains(v))
					mutex=false;
			if(mutex)
				this.constraints.add(c);

		}
	}


	public AcqNetwork(AcqNetwork network) {
		this.variables= new TreeSet<AcqVariable>(network.variables);
		this.constraints= new TreeSet<AcqConstraint>(network.constraints);		
		this.min_arity=network.get_min_arity();
	}


	private int up_min_arity() {

		int min=this.variables.size();
		for(AcqConstraint c: this.constraints)
			min=(min > c.getArity()) ? c.getArity() : min;

			return min;
	}



	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


	////////////////////////////////////////////////////
	///
	///				GETTERs / SETTERs
	///
	////////////////////////////////////////////////////


	private int get_min_arity() {
		// TODO Auto-generated method stub
		return this.min_arity;
	}


	public String getName() {
		// TODO Auto-generated method stub
		return this.name;
	}

	private void setName(String name) {
		// TODO Auto-generated method stub
		 this.name=name;
	}
	public int size() {
		return constraints.size();
	}


	public SortedSet<AcqConstraint> getConstraints() {
		// TODO Auto-generated method stub
		return this.constraints;
	}

	public void add(AcqConstraint cst) {
		this.constraints.add(cst);
		this.variables.addAll(cst.getVariables());


	}

	public void add_reified_network(AcqNetwork network) {

		for(AcqConstraint c: network.getConstraints())
			this.add_reified_constraint(c);


	}

	private void add_reified_constraint(AcqConstraint c) {

		for(AcqVariable var: c.getVariables())
			if(!this.variables.contains(var))
				this.variables.add(var);

		this.constraints.add(new AcqConstraint(c,true));


	}

	public Query getSolution() {
		// TODO Auto-generated method stub
		return this.solutions.first();
	}


}
