/*********************************************************
 * AcqVariable class (Acquisition variables) 
 * 
 * 
 * @author LAZAAR
 * @date 2016-07-29
 ********************************************************/


package vocabulary;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

public class AcqVariable implements Comparable<AcqVariable>, Serializable {

	////////////////////////////////////////////////////
	///
	///				ATTRIBUTEs
	///
	////////////////////////////////////////////////////	

	private static final long serialVersionUID = 1L;
	public static Random r = new Random();
	public static final int UNSET = -1000;
	private String name;
	private int min;
	private int max;

	//========= Generalization approach (ECAI14)  attributes ========//
	public String type;
	public ArrayList<String> assendant;
	//================================================================//


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	////////////////////////////////////////////////////
	///
	///				CONSTRUCTORs
	///
	////////////////////////////////////////////////////

	public AcqVariable(String name,int min, int max) {
		this.min = min;
		this.max = max;
		this.name = name;   
	}



	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------
	////////////////////////////////////////////////////
	///
	///				METHODs
	///
	////////////////////////////////////////////////////

	/****************************************************
	 * getRandomValue V included in [min,max]
	 * @return V
	 ***************************************************/

	public int getRandomValue() {
		int indice = r.nextInt(domain.length);
		return domain[indice];
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


	/****************************************************
	 *  V is in  [min,max]?
	 * @return T/F
	 ***************************************************/

	public boolean containsValue(int value) {
		return (value == this.UNSET ||(value >= this.min && value <= this.max));
	}




	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------


	////////////////////////////////////////////////////
	///
	///				OVERRIDED METHODs
	///
	////////////////////////////////////////////////////

	@Override
	public String toString() {
		return name+" ["+domain[0]+","+domain[domain.length-1]+"]";
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	@Override
	public int compareTo(AcqVariable var) {
		return name.compareTo(var.name);
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof AcqVariable))
			return false;
		AcqVariable other = (AcqVariable) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	 

	public int min() {
		return min;
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	public int max() {
		return max;
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	public String name() {
		return name;
	}


	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------



	public void setMin(int newMin) {
		this.min=newMin;
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------

	public void setMax(int newMax) {
		this.max= newMax;
	}

	//---------------------------------------------------------------------------------------------------   
	//---------------------------------------------------------------------------------------------------



}
