package ag;

import static org.junit.Assert.*;

import java.util.BitSet;

import org.junit.Before;
import org.junit.Test;

import acqConstraints.Operator;
import combinatorial.AllPermutationIterator;
import combinatorial.CombinationIterator;
import config.Heuristic;

public class ACQ_ChocoSolverTest {
	
	private static ACQ_Network networkA;
	private static ACQ_Network networkB;
	private static ACQ_ConstraintSolver solver;
	private static Heuristic heuristic;
	
	@Before
	public void setUp() throws Exception {
		ACQ_Scope fullScope = new ACQ_Scope(0,1,2);
		ACQ_ConstraintSet constraintsA = new ACQ_ConstraintSet();
		ACQ_ConstraintSet constraintsB = new ACQ_ConstraintSet();
		CombinationIterator iterator = new CombinationIterator(3, 2);
		while (iterator.hasNext()) {
			int[] vars = iterator.next();
			AllPermutationIterator pIterator = new AllPermutationIterator(2);
			while (pIterator.hasNext()) {
				int[] pos = pIterator.next();
				// X != Y
				constraintsA.add(new BinaryArithmetic("DifferentXY", vars[pos[0]], Operator.NQ, vars[pos[1]]));
			}
			constraintsB.add(new BinaryArithmetic("GreaterOrEqual", vars[0], Operator.GE, vars[1]));
		}
		networkA = new ACQ_Network(fullScope, constraintsA);
		networkB = new ACQ_Network(fullScope, constraintsB);
		
		solver =  new ACQ_ChocoSolver(new ACQ_Domain() {
				@Override
				public int getMin(int numvar) {
					return 1;
				}

				@Override
				public int getMax(int numvar) {
					return 3;
				}
			});
		heuristic = Heuristic.SOL;
	}
	
	@Test
	public void testSolveANotB() {
		ACQ_Query q = solver.solve_AnotB(networkA, networkB, heuristic);
		assertTrue(networkA.check(q));
		assertFalse(networkB.check(q));
	}

	@Test
	public void testSolveANotAllB() {
		ACQ_Query q = solver.solve_AnotB(networkA, networkB, heuristic);
		assertTrue(networkA.check(q));
		assertFalse(networkB.check(q));
	}
}
