package ag;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import acqConstraints.Operator;

public class ACQ_ConstraintSetTest {

	private ACQ_ConstraintSet set;
	
	@Before
	public void setUp() throws Exception {
		set = new ACQ_ConstraintSet();
	}

	@Test
	public void testAddRemove() {
		
		assertEquals(set.size(), 0);
		
		ACQ_IConstraint cst1 = new BinaryArithmetic("CS1",0,Operator.EQ,1);
		
		set.add(cst1);
		
		assertEquals(set.size(), 1);
		
		ACQ_IConstraint cst2 = new BinaryArithmetic("CS2",0,Operator.NQ,1);
		
		set.add(cst2);
		
		assertEquals(set.size(), 2);
		
		assertTrue(set.constraints.contains(cst1));
		
		assertTrue(set.constraints.contains(cst2));
		
		set.remove(cst1);
		
		assertEquals(set.size(), 1);
		
		set.remove(cst2);
		
		assertEquals(set.size(), 0);
		
		assertFalse(set.constraints.contains(cst1));
		
		assertFalse(set.constraints.contains(cst2));
	}
	
	@Test
	public void testRemoveAll() {
		
		assertEquals(set.size(), 0);
		
		ACQ_IConstraint cst1 = new BinaryArithmetic("CS1",0,Operator.EQ,1);
		
		set.add(cst1);
		
		assertEquals(set.size(), 1);
		
		ACQ_IConstraint cst2 = new BinaryArithmetic("CS2",0,Operator.NQ,1);
		
		set.add(cst2);
		
		assertEquals(set.size(), 2);
		
		assertTrue(set.constraints.contains(cst1));
		
		assertTrue(set.constraints.contains(cst2));
		
		ACQ_ConstraintSet set2 = new ACQ_ConstraintSet();
		
		set2.add(cst1);
		
		set2.add(cst2);
		
		set.removeAll(set2);
		
		assertEquals(set.size(), 0);
		
		assertFalse(set.constraints.contains(cst1));
		
		assertFalse(set.constraints.contains(cst2));
	}
	
	@Test
	public void testIsEmpty() {
		
		assertTrue(set.isEmpty());
		
		ACQ_IConstraint cst1 = new BinaryArithmetic("CS1",0,Operator.EQ,1);
		
		set.add(cst1);
		
		assertFalse(set.isEmpty());
	}
	
	@Test
	public void testRetainAll() {
		
		ACQ_IConstraint cst1 = new BinaryArithmetic("CS1",0,Operator.EQ,1);
		
		ACQ_IConstraint cst2 = new BinaryArithmetic("CS2",0,Operator.NQ,1);
		
		ACQ_ConstraintSet set2 = new ACQ_ConstraintSet();
		
		set2.add(cst1);
		
		set2.add(cst2);
		
		set.retainAll(set2);
		
		assertTrue(set.isEmpty());
		
		set.add(cst1);
		
		set.retainAll(set2);
		
		assertEquals(set.size(), 1);
		
		assertTrue(set.constraints.contains(cst1));
	}

}
