package ag;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import acqConstraints.Operator;
import combinatorial.AllPermutationIterator;
import combinatorial.CombinationIterator;

public class ACQ_NetworkTest {
	
	private static ACQ_Network network;

	
	@Before
	public void setUp() throws Exception {
		// build constraint network
		ACQ_Scope fullScope = new ACQ_Scope(0,1,2);
		ACQ_ConstraintSet constraints = new ACQ_ConstraintSet();
		CombinationIterator iterator = new CombinationIterator(3, 2);
		while (iterator.hasNext()) {
			int[] vars = iterator.next();
			AllPermutationIterator pIterator = new AllPermutationIterator(2);
			while (pIterator.hasNext()) {
				int[] pos = pIterator.next();
				// X != Y
				constraints.add(new BinaryArithmetic("DifferentXY", vars[pos[0]], Operator.NQ, vars[pos[1]]));
			}
			constraints.add(new BinaryArithmetic("GreaterOrEqualXY", vars[0], Operator.GE, vars[1]));
		}
		network = new ACQ_Network(fullScope, constraints);

	}

	@Test
	public void testCheck() {
		// build query
		System.out.println(network.toString());
		ACQ_Scope partialScope = new ACQ_Scope(0,1);
		int[] values = {8,4};
		ACQ_Query query = new ACQ_Query(partialScope, values);
		assertTrue(network.check(query));
		
		ACQ_Scope partialScope2 = new ACQ_Scope(0,1);
		int[] values2 = {3,4};
		ACQ_Query query2 = new ACQ_Query(partialScope2, values2);
		assertFalse(network.check(query2));
	}
	
	@Test
	public void testCheckNotAllNeg() {
		ACQ_Scope partialScope = new ACQ_Scope(0,1);
		int[] values = {3,4};
		ACQ_Query query = new ACQ_Query(partialScope, values);
		assertTrue(network.checkNotAllNeg(query));
		
		ACQ_Scope partialScope2 = new ACQ_Scope(0,1);
		int[] values2 = {6,4};
		ACQ_Query query2 = new ACQ_Query(partialScope2, values2);
		assertFalse(network.checkNotAllNeg(query2));
	}
	
	@Test
	public void testGetConstraints() {
		
		ACQ_ConstraintSet set1 = network.getConstraints();
		
		assertTrue(set1 instanceof ACQ_ConstraintSet);
		
		assertEquals(set1.size(), 9);
		
		// Add a constraint to the network and re-evaluate the size
		// Rely on the fact that the add() function is operational
		ACQ_IConstraint cs = new BinaryArithmetic("EqualXY", 0, Operator.EQ, 2);
		
		network.add(cs, false);
		
		set1 = network.getConstraints();
		
		assertEquals(set1.size(), 10);
	}
	
	@Test
	public void testAddNoForce() {
				
		// build query
		ACQ_Scope partialScope = new ACQ_Scope(0,1);
		int[] values = {8,4};
		ACQ_Query query = new ACQ_Query(partialScope, values);
		assertTrue(network.check(query));
		
		int size = network.size();
		
		ACQ_IConstraint cs = new BinaryArithmetic("EqualXY", 0, Operator.EQ, 1);
		
		network.add(cs, false);
		
		assertEquals(network.size(), size + 1);
		
		assertFalse(network.check(query));
	}
	
	@Test
	public void testAddWithForce() {
		
		int Scope_Size = network.variables.size();
		assertEquals(Scope_Size, network.variables.size());
		
		int CS_Size = network.size();
		assertEquals(network.size(), CS_Size);
		
		// constraint to add to the network
		ACQ_IConstraint cs = new BinaryArithmetic("EqualXY",3,Operator.EQ,4);
		
		// cs is not added because the network doesnt contain all the variables
		network.add(cs, false);
		
		assertEquals(network.size(), CS_Size);
		assertEquals(Scope_Size, network.variables.size());
		
		// add cs with forcing
		network.add(cs, true);

		assertEquals(network.size(), CS_Size+1);
		assertEquals(network.variables.size(), Scope_Size+cs.getArity());
		
		assertTrue(network.variables.contains(3));
		assertTrue(network.variables.contains(4));
	}
}
