package ag;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.BitSet;
import java.util.Iterator;

import org.junit.Before;
import org.junit.Test;

public class ACQ_ScopeTest {
	
	private static ACQ_Scope scope1;
	private static ACQ_Scope scope2;
	private static ACQ_Scope scope3;

	@Before
	public void setUp() throws Exception {
		
		// Scope 1
		
		BitSet b1 = new BitSet();
		b1.set(0);
		b1.set(1);
		b1.set(2);
		b1.set(4);
		b1.set(5);
		
		scope1 = new ACQ_Scope(b1);
		
		// Scope 2
		
		scope2 = new ACQ_Scope();
		
		// Scope 3
		
		BitSet b3 = new BitSet();
		b3.set(8);
		b3.set(6);
		
		scope3 = new ACQ_Scope(b3);
	}

	
	@Test
	public void testGetFirst() {
		
		assertEquals(scope1.getFirst(),0);
		
		assertEquals(scope2.getFirst(), -1);
		
		assertEquals(scope3.getFirst(), 6);
	}
	
	@Test
	public void testGetSecond() {
		
		assertEquals(scope1.getSecond(), 1);
		
		assertEquals(scope2.getSecond(), -1);
		
		assertEquals(scope3.getSecond(), 8);
	}
	
	@Test
	public void testGetThird() {
		
		assertEquals(scope1.getThird(), 2);
		
		assertEquals(scope2.getThird(), -1);
		
		assertEquals(scope3.getThird(), -1);
	}
	
	@Test
	public void testContains() {
		
		assertTrue(scope1.contains(0));
		assertTrue(scope1.contains(1));
		assertTrue(scope1.contains(2));
		assertTrue(scope1.contains(4));
		assertTrue(scope1.contains(5));
		
		assertFalse(scope1.contains(3));
		assertFalse(scope1.contains(8));
		assertFalse(scope1.contains(9));
		assertFalse(scope1.contains(11));
		
		assertFalse(scope2.contains(5));
		assertFalse(scope2.contains(0));
		assertFalse(scope2.contains(15));
		assertFalse(scope2.contains(8));
		
		assertTrue(scope3.contains(8));
		assertTrue(scope3.contains(6));
		
		assertFalse(scope3.contains(10));
		assertFalse(scope3.contains(0));
		assertFalse(scope3.contains(1));
		assertFalse(scope3.contains(2));
	}
	
	@Test
	public void testContainsAll() {
		
		assertTrue(scope1.containsAll(scope2));
		assertTrue(scope3.containsAll(scope2));
		
		ACQ_Scope temp_scope = new ACQ_Scope(0,1,3);
		ACQ_Scope temp_scope2 = new ACQ_Scope(0,1);
		
		assertFalse(scope1.containsAll(temp_scope));
		
		assertTrue(scope1.containsAll(temp_scope2));
		
		assertFalse(scope3.containsAll(temp_scope2));
		assertFalse(scope3.containsAll(temp_scope));
		
		assertFalse(scope2.containsAll(temp_scope2));
		assertFalse(scope2.containsAll(temp_scope));
	}
	
	@Test
	public void testSize() {
		
		assertEquals(scope1.size(), 5);
		
		assertNotEquals(scope1.size(), 6);
		
		assertEquals(scope2.size(), 0);
		
		assertNotEquals(scope2.size(), 1);
		
		assertNotEquals(scope2.size(), -1);
		
		assertEquals(scope3.size(), 2);
		
		assertNotEquals(scope3.size(), 1);
	}
	
	@Test
	public void testSplit() {
		
		ACQ_Scope[] split1 = scope1.split();
		
		assertEquals(split1[0].size()+split1[1].size(), scope1.size());
		
		assertTrue(split1[0].size() <= (scope1.size()/2)+1);
		
		assertTrue(split1[1].size() <= (scope1.size()/2)+1);
		
		Iterator<Integer> iterator = scope1.iterator();
		
		while(iterator.hasNext()) {
			int a = iterator.next();
			if(!split1[0].contains(a) && !split1[1].contains(a)) {
				fail();
			}
		}
		
		ACQ_Scope[] split3 = scope3.split();
		
		assertEquals(split3[0].size()+split3[1].size(), scope3.size());
		
		assertEquals(split3[0].size(), 1);
		
		assertEquals(split3[1].size(), 1);
	}
	
	@Test(expected=AssertionError.class)
	public void testSplitEmpty() {
		
		ACQ_Scope[] split2 = scope2.split();
		
		assertEquals(split2[0].size()+split2[1].size(), scope2.size());
		
		assertEquals(split2[0].size(), 0);
		
		assertEquals(split2[1].size(), 0);
		
	}
	
	@Test
	public void testUnion() {
		
		ACQ_Scope s11 = new ACQ_Scope(0);
		ACQ_Scope s12 = new ACQ_Scope(1, 2, 4, 5);
		ACQ_Scope s1 = s11.union(s12);
		
		ACQ_Scope s21 = new ACQ_Scope(0, 1);
		ACQ_Scope s22 = new ACQ_Scope(2, 4, 5);
		ACQ_Scope s2 = s21.union(s22);
		
		ACQ_Scope s31 = new ACQ_Scope(0, 1, 2);
		ACQ_Scope s32 = new ACQ_Scope(4, 5);
		ACQ_Scope s3 = s31.union(s32);
		
		ACQ_Scope s41 = new ACQ_Scope(0, 1, 2, 4);
		ACQ_Scope s42 = new ACQ_Scope(5);
		ACQ_Scope s4 = s41.union(s42);
		
		ACQ_Scope s51 = new ACQ_Scope(0, 1, 2, 4, 5);
		ACQ_Scope s52 = new ACQ_Scope();
		ACQ_Scope s5 = s51.union(s52);
		
		ACQ_Scope s61 = new ACQ_Scope();
		ACQ_Scope s62 = new ACQ_Scope(0, 1, 2, 4, 5);
		ACQ_Scope s6 = s61.union(s62);
		
		assertTrue(s1.size() == scope1.size());
		
		assertTrue(s2.size() == scope1.size());
		
		assertTrue(s3.size() == scope1.size());
		
		assertTrue(s4.size() == scope1.size());
		
		assertTrue(s5.size() == scope1.size());
		
		assertTrue(s6.size() == scope1.size());
		
		Iterator<Integer> iterator = scope1.iterator();
		
		while(iterator.hasNext()) {
			int a = iterator.next();
			if(!s1.contains(a) || !s2.contains(a) ||
			   !s3.contains(a) || !s4.contains(a) ||
			   !s5.contains(a) || !s6.contains(a)) {
				fail();
			}
		}
		
		ACQ_Scope empty = new ACQ_Scope();
		
		ACQ_Scope empty_union = empty.union(scope2);
		
		assertEquals(empty_union.size(), 0);
	}
	
	@Test
	public void testGetProjection() {
		
		ACQ_Scope s1 = new ACQ_Scope(0, 1);
		
		int[] v1 = {3,4};
		
		ACQ_Query q1 = new ACQ_Query(s1,v1);
		
		int[] p1 = scope1.getProjection(q1);
		
		assertEquals(p1[0], 3);
		
		assertEquals(p1[1], 4);

		assertEquals(p1.length, 2);
		
		int[] p2 = scope2.getProjection(q1);
		
		assertEquals(p2.length, 0);
	}
}
