/*********************************************************
 * QUACQ (Quick Acquisition) Algorithm
 * @ref IJCAI13
 * 
 * @author LAZAAR
 * @date 2016-07-29
 ********************************************************/

package algo;

import java.util.ArrayList;
import java.util.List;

import acqConstraints.AcqConstraint;
import vocabulary.*;
import machineLearning.*;


public class old_Quacq_file implements QUACQ{

	private Bias bias;	

	public old_Quacq_file(Language language, ArrayList<AcqVariable> variables, ArrayList<AcqDomain> domains){
		
		
		this.bias= new Bias(language, variables, domains);
		
		
	}
	

	public old_Quacq_file(Bias bias,Learner learner) {
		this.minArity = bias.getMinArity();
		this.bias = bias;
		this.learner = learner;
		this.B = bias.getConstraints();
		this.NL = new ArrayList<AcqConstraint>();
		this.bY = new ArrayList<AcqConstraint>();
		this.NLy = new ArrayList<AcqConstraint>();
		this.Y = new ArrayList<AcqVariable>();
		this.solver = new ConstraintSolver(bias,true);
		this.completePQuery=0;
		this.completeNQuery=0;
		this.askedQ=0;
		this.BiasSize= new int[2];
		this.nbIt=0;
		this.partialEx=false;
		this.scopeTabu=true;
		this.NT = new ArrayList<AcqConstraint>();
		this.NT.addAll(learner.constraints);
		this.queryPerLevel= new int[bias.getNbVariables()];

		// GENERALIZATION
		this.generalization=false;

		this.lattice = this.bias.getTypeVar();
		this.Top = this.bias.getTop();
		this.negativeQuery = new ArrayList<CoupleT>();
		this.negativeQuery3 = new ArrayList<CoupleT3>();
		this.negativeQuery4 = new ArrayList<CoupleT4>();


		MNSes = new ArrayList<List<AcqVariable>>();
		MYSes = new ArrayList<List<AcqVariable>>();
		HS = new ArrayList<List<AcqVariable>>();
		recordedQueries = new ArrayList<Integer>();
		recordedTime = new ArrayList<Double>();
		recordedQSizes = new ArrayList<Integer>();
		recordedQTimes = new ArrayList<Double>();
		stDevS = 0;
		stDevQ = 0;
		stDevT = 0;
		stDevQT = 0;
		currQueryTimeOutRange = queryTimeOutRange;
		currMNSTimeOutRange = MNSTimeOutRange;
		nbVisitedNodes = 0;
		nbTimeOut = 0;
		lubyCpt = 0;
		timeRemoved = 0;
		qRemoved= 0;
		reverseVars = false;
		indexOfLastVarFound = 0;
		firstEx = true;
		lastFASTime = 0;
		consPosQ = 0;
		startTime = 0;
		maxHSSize = 0;
		maxUDTime = 0;
		cumulUDTime = 0;

		//deltaInit = new ArrayList<AcqConstraint>();
		
		
		if(QperSize_flag)
		{
			this.queryPerSize= new Integer[2][bias.getNbVariables()];
			for(int i=0; i<2; i++)
				for(int j=0; j<bias.getNbVariables(); j++)
					this.queryPerSize[i][j]=0;
		}

		if(modelSeeker_init)
		{
			this.MS = new ArrayList<AcqConstraint>();
			this.MS.addAll(learner.modelSeeker);
			System.out.println("cstr learned using ModelSeeker: "+ MS.size());
		}
		this.scopeSave=  new ArrayList<AcqVariable>();
		this.tabu=0;
		this.stuck=0;
		this.nbpeelit=0;
		if(sol)
			this.H= heuristic.SOL_H;
		else
			if(max_it)
				this.H= heuristic.MAX_H;
			else 
				this.H= heuristic.MIN_H;


	}

	/************************************************
	 * ASK&SOLVE
	 * @author lazaarnadjib
	 * @version 4
	 * 
	 * @see paper submitted to AAAI14(failed) and ICTAI14 (accepted)
	 * 04/2013
	 * @return
	 ************************************************/
	public boolean askSolve(){

		// definitions
		HashSet<AcqVariable> scope= new HashSet<AcqVariable>();
		List<AcqVariable> X = new ArrayList<AcqVariable>();
		List<AcqVariable> remainSet= new ArrayList<AcqVariable>();
		List<AcqVariable> variables= new ArrayList<AcqVariable>(bias.getVariablesList());
		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();
		List<AcqConstraint> bYguilty= new ArrayList<AcqConstraint>();
		List<AcqConstraint> NL_modelSeeker= new ArrayList<AcqConstraint>();
		AcqVariable guilty;
		boolean guiltyflag=false;
		boolean luby_flag=true;
		int nb_restart=0;

		int pas;
		int nbVar=bias.getVariablesList().size();

		//STAT
		int nb_queries_It=0;
		int nb_queries_lv=0;
		int cumul_queries=0;
		boolean valueOfEx;
		int luby_seq=0;


		if(modelSeeker_init)
		{
			jumpInLearning();
			NL_modelSeeker.addAll(NL);
		}

		// factor initialized to |X|
		factor= 1;//(bias.getVariables().length);
		threshold=1;//factor;

		int min_arity=bias.getMinArity();

		//STAT
		BiasSize[0]=B.size();

		Example e= new Example(); 

		// Initialization : (line 1 to 4)

		remainSet.addAll(bias.getVariablesList());

		if(initShuffle)
			Collections.shuffle(remainSet);

		for(int i=0; i<min_arity-1; i++)  // from 0 to min-arity-1
		{
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			scope.add(v);
		}


		if(luby){
			luby_restart=luby(2,luby_seq);
			luby_seq++;
			threshold=luby_restart*(factor);
		}


		while(!remainSet.isEmpty()){

			if(shuffle)
				Collections.shuffle(remainSet);

			guilty= remainSet.get(0);
			remainSet.remove(0);  // line 5
			scope.add(guilty);				//line 6						// s <-- scope(e) 

			///////////////// BEGIN  GENERATE EXAMPLE

			// NL projection on "scope"
			NLy.clear();

			if(modelSeeker_init)
				NLy.addAll(NL_modelSeeker);

			for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
				if (scope.containsAll(c.getVariablesList())) 					
					NLy.add(c);
			}
			//--------------------------------------------


			// B projection on "scope/guilty"
			scope.remove(guilty);

			Iterator<AcqConstraint> iterator1 = B.iterator();
			bYguilty.clear();
			while( iterator1.hasNext()) {
				AcqConstraint c= (AcqConstraint) iterator1.next();

				if (scope.containsAll(c.getVariablesList())) 				
					bYguilty.add(c);

			}
			//-----------------------------------------------------


			// B projection on "scope"
			scope.add(guilty);

			Iterator<AcqConstraint> iterator2 = B.iterator();
			bY.clear();
			while( iterator2.hasNext()) {
				AcqConstraint c= (AcqConstraint) iterator2.next();

				if (scope.containsAll(c.getVariablesList())) 				
					bY.add(c);

			}
			//-----------------------------------------------------

			//select e in sol(CL[scp] U B[scp/guilty])[scp] maximazing B[scp]	

			NLy.addAll(bYguilty);
			/* TEST*/
			System.err.println("solve A");
			e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 
			//	System.err.println("solved A :"+e);

			pas=10;
			while(e==null && pas<100)
			{
				solver.setTimeLimit(1000*pas);
				System.err.println("solve B");
				e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
				//		System.err.println("solved B "+e);
				pas*=10;
			}
			solver.setTimeLimit(1000);


			if(e==null){		
				guiltyflag=true;
				NLy.removeAll(bYguilty);
				System.err.println("solve D");
				e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 
				System.err.println("solved D "+e);
				pas=10;
				while(e==null&& pas<100)
				{
					solver.setTimeLimit(1000*pas);
					System.err.println("solve E");
					e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
					System.err.println("solved E "+e);
					pas*=10;
				}
				solver.setTimeLimit(1000);


			}

			if(e==null) 
			{
				System.err.println("peel it!");
				e=peelIt(NL);
				System.err.println("that gives "+e);
			}
			e=e.getProjection(scope);	

			//////////////// END 	GENERATE EXAMPLE


			/*			if(luby)
			{
			//	nbIt++;
				luby_flag=false;
				luby_restart=luby(1,nbIt);
			}
			 */

			// memory check
			if(with_m){
				Example ex=checkMemory(e);
				if(ex!=null)
				{
					valueOfEx= ex.isPositif();
					askedQ++;
					//	if(valueOfEx)
					//		asked=true;
					if(printMemory)  
						printMemory(e,ex,true);
				}
				else
				{
					valueOfEx=learner.classify(e,false);
					memory.add(e);
					if(printSize)
						printQsize(e.size());

				}
			}
			else
			{
				valueOfEx=learner.classify(e,false);

			}


			//STAT
			if(mylog)
			{
				//myLog(nb_queries_lv,"QperLevel");
				queryPerLevel[e.size()-1]=nb_queries_lv;
				nb_queries_lv=0;
			}

			while(!valueOfEx){


				nbIt++;
				if(restart)
					nb_negatives++;
				System.err.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>NB NEGATIVES : "+nb_negatives+ "THRESHOLD"+threshold);
				System.out.println("<<<<<<<<<<<<<<<<<<<<<<<<<    ITERATION: "+nbIt);

				//STAT  (print scope size at each iteration)
				if(mylog)
					myLog(scope.size(), "scope");







				if(remainSet.isEmpty())
					completeNQuery++;

				X.clear();
				X.addAll(scope);
				List<AcqVariable> scopeOn;
				AcqConstraint c_delta;

				if(guiltyflag)
				{
					emptySet.clear();
					scopeOn=findScope(e,X,false, emptySet);
					c_delta = findCstr(scopeOn,e);			// learnt constraint
					guiltyflag=false;

				}
				else{
					emptySet.clear();
					emptySet.add(guilty);
					X.remove(guilty);
					scopeOn=findScope(e,X,false, emptySet);
					scopeOn.add(guilty);
					c_delta = findCstr(scopeOn,e);			// learnt constraint

				}


				if(c_delta!=null)
					if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
					{

						ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
						ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
						checkNT1.addAll(NT);
						checkNT2.addAll(NT);
						checkNT1.add(c_delta);
						checkNT2.add(c_delta.negation());

						if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2))
							if(modelSeeker && NL.contains(c_delta))
								cst_MS--;
							else
							{
								NL.add(c_delta);
								myLog("cst: "+c_delta, "constraints");
								if(generalization){
									List<AcqConstraint> lc_delta = new ArrayList<AcqConstraint>();

									if (c_delta.getVariables().length == 2) {
										List<TypeVar> list_type = new ArrayList<TypeVar>();
										boolean methodeComplete = true;
										if (methodeComplete)
											list_type.addAll(generalisetable2(c_delta));
										else
											list_type.addAll(generalisetable2Incomplete(c_delta));

										//						 list_type.addAll(generalisetable2Rand(c_delta));

										for (int i = 0; i < list_type.size(); i += 2) {

											TypeVar t_i = list_type.get(i);
											TypeVar t_j = list_type.get(i + 1);

											if (t_i.getName().equals(t_j.getName())) {
												for (int j = 0; j < t_i.getVars().length - 1; j++)
													for (int k = j + 1; k < t_i.getVars().length; k++)
														lc_delta.add(new AcqConstraint(c_delta
																.getType(), t_i.getVars()[j], t_i
																.getVars()[k]));
											} else
												for (AcqVariable var1 : t_i.getVars())
													for (AcqVariable var2 : t_j.getVars()) {
														if (!var1.equals(var2)) {
															AcqConstraint cst = new AcqConstraint(
																	c_delta.getType(), var1, var2);
															if (!cst.isInListOfConstraint(lc_delta)
																	&& !cst.isInListOfConstraint(NL))
																lc_delta.add(cst);
														}
													}
										}
									}
									
									NL.addAll(lc_delta);
									myLog("list: "+lc_delta, "constraints");
									myLog("=========================", "constraints");
									
								}

							}
						else{
							System.out.println("NL:"+NL);
							System.out.println("delta"+c_delta);
							System.out.println("scope"+scope);
						}
						System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");

						logger.debug("REMOVED FROM B==>  "+c_delta);
					}
					else{

						if(modelSeeker && NL.contains(c_delta))
							cst_MS--;
						else
							{
							NL.add(c_delta);
							
							if(generalization){
								List<AcqConstraint> lc_delta = new ArrayList<AcqConstraint>();

								if (c_delta.getVariables().length == 2) {
									List<TypeVar> list_type = new ArrayList<TypeVar>();
									boolean methodeComplete = true;
									if (methodeComplete)
										list_type.addAll(generalisetable2(c_delta));
									else
										list_type.addAll(generalisetable2Incomplete(c_delta));

									//						 list_type.addAll(generalisetable2Rand(c_delta));

									for (int i = 0; i < list_type.size(); i += 2) {

										TypeVar t_i = list_type.get(i);
										TypeVar t_j = list_type.get(i + 1);

										if (t_i.getName().equals(t_j.getName())) {
											for (int j = 0; j < t_i.getVars().length - 1; j++)
												for (int k = j + 1; k < t_i.getVars().length; k++)
													lc_delta.add(new AcqConstraint(c_delta
															.getType(), t_i.getVars()[j], t_i
															.getVars()[k]));
										} else
											for (AcqVariable var1 : t_i.getVars())
												for (AcqVariable var2 : t_j.getVars()) {
													if (!var1.equals(var2)) {
														AcqConstraint cst = new AcqConstraint(
																c_delta.getType(), var1, var2);
														if (!cst.isInListOfConstraint(lc_delta)
																&& !cst.isInListOfConstraint(NL))
															lc_delta.add(cst);
													}
												}
									}
								}
							}
							}
					}
				else
				{
					scopeOn=findScope(e,X,false, emptySet);
					System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>"+ scopeOn);
					Toolkit.getDefaultToolkit().beep();
				}







				System.out.println("AskSolve => "+nbIt+" nbQ: " +nb_queries_It+"B size: "+ B.size()+ " NL size:" + NL.size()+ "asked Q: "+ askedQ);
				System.err.println(new Date());


				///////////////// BEGIN  GENERATE EXAMPLE

				// NL projection on "scope"
				NLy.clear();

				if(modelSeeker_init)
					NLy.addAll(NL_modelSeeker);
				for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
					if (scope.containsAll(c.getVariablesList())) 					
						NLy.add(c);
				}
				//--------------------------------------------


				// B projection on "scope/guilty"
				scope.remove(guilty);

				Iterator<AcqConstraint> iterator3 = B.iterator();
				bYguilty.clear();
				while( iterator3.hasNext()) {
					AcqConstraint c= (AcqConstraint) iterator3.next();

					if (scope.containsAll(c.getVariablesList())) 				
						bYguilty.add(c);

				}
				//-----------------------------------------------------


				// B projection on "scope"
				scope.add(guilty);

				Iterator<AcqConstraint> iterator4 = B.iterator();
				bY.clear();
				while( iterator4.hasNext()) {
					AcqConstraint c= (AcqConstraint) iterator4.next();

					if (scope.containsAll(c.getVariablesList())) 				
						bY.add(c);

				}
				//-----------------------------------------------------

				//select e in sol(CL[scp] U B[scp/guilty])[scp] maximazing B[scp]	

				NLy.addAll(bYguilty);
				e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 

				pas=10;
				while(e==null&& pas<100)
				{
					solver.setTimeLimit(1000*pas);
					e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
					pas*=10;
				}
				solver.setTimeLimit(1000);

				if(e==null){		
					guiltyflag=true;
					NLy.removeAll(bYguilty);

					e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 

					pas=10;
					while(e==null&& pas<100)
					{
						solver.setTimeLimit(1000*pas);
						e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
						pas*=10;
					}
					solver.setTimeLimit(1000);


				}

				if(e==null)
					e=solver.buildSolution(NLy);
				e=e.getProjection(scope);		

				//////////////// END 	GENERATE EXAMPLE

				/*
				 Finite subsequences of the Luby-sequence:

				 0: 1
				 1: 1 1 2
				 2: 1 1 2 1 1 2 4
				 3: 1 1 2 1 1 2 4 1 1 2 1 1 2 4 8
				 ...


				 */

				if(restart){



					if(nb_negatives>threshold){

						if(luby){
							luby_restart=luby(2,luby_seq);
							luby_seq++;
							threshold=luby_restart*(factor);
						}

						if(geo)
							threshold*=coeff;

						//	Toolkit.getDefaultToolkit().beep();

						nb_negatives=0;
						System.err.println("																=============================================================");
						System.err.println("																================       RESTART         ======================");
						System.err.println("																================         NB:"+nb_negatives+"           ======================");
						System.err.println("																================         TH:"+threshold+"           ======================");
						System.err.println("																=============================================================");
						if(luby)
							System.err.println("threshold= "+threshold+" nb IT= "+nbIt);
						if(mylog)
							myLog("Threshold="+threshold);
						nb_restart++;
						scope.remove(guilty);
						remainSet.addAll(scope);
						scope.clear();
						//	if(shuffle)
						//		Collections.shuffle(remainSet);

						scope.add(guilty);
						if(r_lex)
							Collections.reverse(remainSet);
						else if(c_lex)
						{
							scope.clear();
							remainSet.clear();
							remainSet.addAll(bias.getVariablesList());

						}

						for(int i=0; i<min_arity-1; i++)  // from 0 to min-arity-1
						{
							AcqVariable v= remainSet.get(0);
							remainSet.remove(0);
							scope.add(v);
						}

						if(c_lex)
						{
							AcqVariable v= remainSet.get(0);
							remainSet.remove(0);
							scope.add(v);
						}
						guiltyflag=true;


						//GENERATE EXAMPLE ON THE NEW SCOPE AFTER A RESTART

						guilty= remainSet.get(0);
						remainSet.remove(0);  // line 5
						scope.add(guilty);				//line 6						// s <-- scope(e) 

						///////////////// BEGIN  GENERATE EXAMPLE

						// NL projection on "scope"
						NLy.clear();
						for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
							if (scope.containsAll(c.getVariablesList())) 					
								NLy.add(c);
						}
						//--------------------------------------------


						// B projection on "scope/guilty"
						scope.remove(guilty);

						Iterator<AcqConstraint> iterator5 = B.iterator();
						bYguilty.clear();
						while( iterator5.hasNext()) {
							AcqConstraint c= (AcqConstraint) iterator5.next();

							if (scope.containsAll(c.getVariablesList())) 				
								bYguilty.add(c);

						}
						//-----------------------------------------------------


						// B projection on "scope"
						scope.add(guilty);

						Iterator<AcqConstraint> iterator6 = B.iterator();
						bY.clear();
						while( iterator6.hasNext()) {
							AcqConstraint c= (AcqConstraint) iterator6.next();

							if (scope.containsAll(c.getVariablesList())) 				
								bY.add(c);

						}
						//-----------------------------------------------------

						//select e in sol(CL[scp] U B[scp/guilty])[scp] maximazing B[scp]	

						NLy.addAll(bYguilty);
						e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 

						pas=10;
						while(e==null && pas<100)
						{
							solver.setTimeLimit(1000*pas);
							e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
							pas*=10;
						}
						solver.setTimeLimit(1000);

						if(e==null){		
							guiltyflag=true;
							NLy.removeAll(bYguilty);

							e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 

							pas=10;
							while(e==null&& pas<100)
							{
								solver.setTimeLimit(1000*pas);
								e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
								pas*=10;
							}
							solver.setTimeLimit(1000);


						}


						e=e.getProjection(scope);		

						//////////////// END 	GENERATE EXAMPLE

					}
				}
				if(luby&& false){

					//		if(nbIt>(luby_restart*(min_arity*5))) 
					if(cumul_queries>(luby_restart*(100))) 
					{
						nbIt=0;
						System.err.println("luby restart!! "+cumul_queries);
						cumul_queries=0;

						//	scope.clear();
						//	remainSet.clear();

						//	remainSet.addAll(bias.getVariablesList());

						// PRECONDITION:  SCOPE U REMAINSET = X
						scope.remove(guilty);
						remainSet.addAll(scope);
						scope.clear();
						if(shuffle)
							Collections.shuffle(remainSet);

						scope.add(guilty);
						for(int i=0; i<min_arity-1; i++)  // from 0 to min-arity-1
						{
							AcqVariable v= remainSet.get(0);
							remainSet.remove(0);
							scope.add(v);
						}
						guiltyflag=true;


						//GENERATE EXAMPLE ON THE NEW SCOPE AFTER A RESTART

						guilty= remainSet.get(0);
						remainSet.remove(0);  // line 5
						scope.add(guilty);				//line 6						// s <-- scope(e) 

						///////////////// BEGIN  GENERATE EXAMPLE

						// NL projection on "scope"
						NLy.clear();
						for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
							if (scope.containsAll(c.getVariablesList())) 					
								NLy.add(c);
						}
						//--------------------------------------------


						// B projection on "scope/guilty"
						scope.remove(guilty);

						Iterator<AcqConstraint> iterator5 = B.iterator();
						bYguilty.clear();
						while( iterator5.hasNext()) {
							AcqConstraint c= (AcqConstraint) iterator5.next();

							if (scope.containsAll(c.getVariablesList())) 				
								bYguilty.add(c);

						}
						//-----------------------------------------------------


						// B projection on "scope"
						scope.add(guilty);

						Iterator<AcqConstraint> iterator6 = B.iterator();
						bY.clear();
						while( iterator6.hasNext()) {
							AcqConstraint c= (AcqConstraint) iterator6.next();

							if (scope.containsAll(c.getVariablesList())) 				
								bY.add(c);

						}
						//-----------------------------------------------------

						//select e in sol(CL[scp] U B[scp/guilty])[scp] maximazing B[scp]	

						NLy.addAll(bYguilty);
						e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 

						pas=10;
						while(e==null && pas<100)
						{
							solver.setTimeLimit(1000*pas);
							e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
							pas*=10;
						}
						solver.setTimeLimit(1000);

						if(e==null){		
							guiltyflag=true;
							NLy.removeAll(bYguilty);

							e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 

							pas=10;
							while(e==null&& pas<100)
							{
								solver.setTimeLimit(1000*pas);
								e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
								pas*=10;
							}
							solver.setTimeLimit(1000);


						}


						e=e.getProjection(scope);		

						//////////////// END 	GENERATE EXAMPLE


					}		
				}

				//STAT
				nb_queries_It=learner.getNbQuery()-cumul_queries;
				nb_queries_lv+=nb_queries_It;
				queryPerSize[0][e.size()-1]++;
				queryPerSize[1][e.size()-1]+=nb_queries_It;


				System.err.println("line 609 nbq: "+nb_queries_It);
				cumul_queries+=nb_queries_It;
				if(mylog)
				{
					//	myLog(nb_queries_It,"QperIt");
					queryPerIt.add(nb_queries_It);
					myLog(cumul_queries,"cumul");
				}


				// memory check
				if(with_m){
					Example ex=checkMemory(e);
					if(ex!=null)
					{
						valueOfEx= ex.isPositif();
						askedQ++;
						//	if(valueOfEx)
						//		asked=true;
						if(printMemory)  
							printMemory(e,ex,true);
					}
					else
					{
						valueOfEx=learner.classify(e,false);
						memory.add(e);
						if(printSize)
							printQsize(e.size());

					}
				}
				else
				{
					valueOfEx=learner.classify(e,false);

					if(printSize)
						printQsize(e.size());

				}

			}


			if(restart)
				nb_negatives=0;

			//STAT
			//myLog(scope.size(), "scope");

			Collection<AcqConstraint> ke = getKappaB(e);
			B.removeAll(ke);
			//STAT
			//	nb_queries=learner.getNbQuery();
			//	myLog(nb_queries,"cumul");

			// call of modelSeeker simulator on partial positives

			System.out.println("partial positive e:"+e);
			if(remainSet.size()!=0)
				NL.addAll(modelSeeker(scope,modelSeeker));
			else
				System.out.println("> > > > > SOLUTION FOUND!!");


		}

		System.err.println("====================================================================================");
		System.err.println("====================================================================================");
		System.err.println("SOLUTION:	"+solver.buildSolution(NL));
		System.err.println("SOLUTION:	"+NL);
		System.err.println("====================================================================================");		
		System.err.println("====================================================================================");

		BiasSize[1]=B.size();
		if(mylog)
			myLog("nb-restart: "+nb_restart+"\n -------------------");
		return true;
	}

	/****************************************************************
	 * BBE 01/2014
	 *
	 * @see algo2: BBE in  (Gelain, Pini, Rossi, Venable, & Walsh, 2010) 
	 * on Hard CSP with Incompletness=100% and the following parameters 
	 * fixed to : 
	 * WHEN=node, 
	 * WHAT=worst, 
	 * WHO=dpi
	 * 
	 * @author lazaarnadjib
	 ****************************************************************/

	public boolean BBE() {		//Disable



		HashSet<AcqVariable> scope= new HashSet<AcqVariable>();
		List<AcqVariable> X = new ArrayList<AcqVariable>();
		List<AcqVariable> remainSet= new ArrayList<AcqVariable>();
		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();
		Collection<AcqConstraint> ke;


		/***
		 * begin INITIALIZATION   (line1 to 3) 
		 */


		boolean backtrack=false;			// I backtrack or not? yes if a given domain is empty
		boolean flag=false;					// true: we use the special findscope, the standard one otherwise

		//STAT
		BiasSize[0]=B.size();

		boolean asked=false;
		int min_arity=bias.getMinArity();
		Example e= new Example(); 

		SortedMap<AcqVariable, ArrayList<Integer>> Bvars = boundToFD(bias.getVariablesList());  // backtrack variables
		remainSet.addAll(bias.getVariablesList());

		if(shuffle)
			Collections.shuffle(remainSet);

		for(int i=0; i<min_arity-1; i++)
		{
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			//OR: int value= Bvars.get(v).get(0);	
			//	OR: int value= one_rand(v.getMin(), v.getMax());			// a random value from min to max included
			int value;
			if(rand_value)
				value= Bvars.get(v).get((int)(Math.random() * (Bvars.get(v).size())));
			else
				value= Bvars.get(v).get(0);
			e.setValue(v, value);
			scope.add(v);
		}

		//  Let us go to line 4 !!

		/***
		 * First loop (line 4) "while e partial do"
		 */

		while(!remainSet.isEmpty()){


			//WHY?
			//	if(e.isPositif()){
			//		ke = getKappaB(e);
			//		B.removeAll(ke);
			//myLog("170=="+ke);  
			//		logger.debug("REMOVED FROM B==>  "+ke);
			//	}

			if(shuffle)
				Collections.shuffle(remainSet);

			// line 5:   x<-- pick(X\scope), here we have lex order on vars
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			scope.add(v);										// s <-- scope(e) 
			int value;

			/************
			 * BEGIN
			 * GENERATE E
			 ***********/

			//---------------------------------------------
			//NL projection on "scope"

			NLy.clear();
			for(AcqConstraint c : NL){								// projection of the scope Y on constraint of NL (i.e., NL[Y])
				if (scope.containsAll(c.getVariablesList())) 					
					NLy.add(c);
			}
			//--------------------------------------------

			// B projection on "scope"
			Iterator<AcqConstraint> iterator = B.iterator();
			bY.clear();
			while( iterator.hasNext()) {
				AcqConstraint c= (AcqConstraint) iterator.next();

				if (scope.containsAll(c.getVariablesList())) 				
					bY.add(c);

			}
			//--------------------------------------------

			// B projection on "scope\x"
			List<AcqConstraint> bYx=new ArrayList<AcqConstraint>();
			scope.remove(v);
			iterator = bY.iterator();
			while( iterator.hasNext()) {
				AcqConstraint c= (AcqConstraint) iterator.next();

				if (scope.containsAll(c.getVariablesList())) 				
					bYx.add(c);

			}		

			scope.add(v);		// we restore our scope!!
			//-----------------------------------------------------

			//choose e st e |= CL[s] U B[s\x] and sat a max cstr in B[s]

			if(fixedPart){
				for(AcqVariable x: scope){
					if(x!=v)
						NLy.add(new AcqConstraint(acqConstraints.Type.EqualX, x, e.getValue(x)));
				}
			}
			NLy.addAll(bYx);		// CL[s] U B[s\x]

			Example e1=solver.singletonMax(NLy,bY, heuristic.MIN_H); 


			// Try again?
			int pas=10;
			while(e1==null && pas<1000000)
			{
				solver.setTimeLimit(1000000000);
				e1=solver.singletonMax(NLy,bY, heuristic.MIN_H);
				pas*=10;
			}

			solver.setTimeLimit(1000);

			if(e1!=null)
			{		
				e=e1;
				if(scope.size()<=min_arity)
					flag=false;
				else
					flag=true;
			}
			else			// line 8: no such example, so choose a simple one!!
			{
				NLy.removeAll(bYx);
				e=solver.singletonMax(NLy, bY, heuristic.MIN_H);
				flag=false;
			}

			// Try again?
			pas=10;
			while(e==null && pas<1000000000)
			{
				solver.setTimeLimit(1000*pas);
				System.err.println("Let try again with :"+ pas);
				e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
				pas*=10;
			}
			if(e==null)
			{
				NL.removeAll(cleanNL());
				e=solver.buildSolution(NL);

			}

			solver.setTimeLimit(1000);


			// Just a projection on scope, because the solve returns a complete example
			if(e==null)
				return BBE();
			e=e.getProjection(scope);
			/************
			 * END
			 * GENERATE E
			 ***********/



			/*****
			 * Second loop (line 10): "while backtrack or ask(e)=no or scope<min_arity do"
			 */
			while( !learner.classify(e,false) || scope.size()<min_arity){


				if( remainSet.isEmpty())		//STAT
					completeNQuery++;

				/*******
				 * line 12 : if not backtrack and scope>=min_arity => learn!
				 */
				AcqConstraint c_delta = null;
				if((!backtrack)  && scope.size()>=min_arity){
					////// LEARNING STEP  --BEGIN
					X.clear();
					emptySet.clear();
					X.addAll(scope);

					if(flag){
						X.remove(v);
						emptySet.add(v);
					}
					/*******************************************************************************************/
					List<AcqVariable> temp= new ArrayList<AcqVariable>(); 
					if(scope.size()==min_arity)
						temp.addAll(scope);
					else
					{temp= findScope(e,X,false, emptySet);
					temp.add(v);
					}
					c_delta = findCstr(temp,e);			// learnt constraint
					/*******************************************************************************************/

					if(c_delta!=null)
						if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
						{
							//	printExample(e,true,c_delta);

							ArrayList<AcqConstraint> checkNT= new ArrayList<AcqConstraint>();
							checkNT.addAll(NT);
							checkNT.add(c_delta);
							if(solver.isSoluble(checkNT))
								NL.add(c_delta);
							else{
								System.out.println("NL:"+NL);
								System.out.println("delta"+c_delta);
								System.out.println("scope"+scope);
							}
							System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
							logger.debug("REMOVED FROM B==>  "+c_delta);
							logger.debug(" B==>  "+B);
						}
						else{

							NL.add(c_delta);
						}
					else
					{
						c_delta = findCstr(findScope(e,X,false, emptySet),e);			// learnt constraint

						//		printExample(e, false,c_delta);
						logger.debug("==========================  ");
						logger.debug(" B==>  "+B);
						logger.debug("NL ==> "+NL);
						logger.debug("example==>"+e);
						logger.debug("==========================  ");
						System.err.println("<<<<<<<<<<<<   NO CSTR "+c_delta+"    >>>>>>>>>>>>>>");
						//		Toolkit.getDefaultToolkit().beep();
					}

					/*********************************one constraint learnt*************************************/
					/*******************************************************************************************/


					System.out.println("BBE=> B size: "+ B.size()+ " NL size:" + NL.size()+ "asked Q: "+ askedQ);
					//////// LEARNING STEP --- END
				}

				/**************************************    BACKTRACK STEP   ********************************/
				/*******************************************************************************************/

				if(c_delta!=null)
					v=c_delta.getVariables()[0]; 			// we branch on the first variable of the learnt constraint
				value=e.getValue(v);	
				Bvars.get(v).remove((Object)value);						// remove the inconsistant value
				NL.add(new AcqConstraint(acqConstraints.Type.DiffX, v, value));
				//	backtrack=false;		//line15 in algo branch&ask


				// if the domain is reduced to empty :
				if(Bvars.get(v).isEmpty()|| !solver.isSoluble(NL)){

					//			backtrack=true;
					// re-initialize the domain of v
					Bvars.get(v).clear();
					Bvars.get(v).addAll(unfold_it(v));
					NL.removeAll(cleanNL(v));
					e.removeVar(v);

					// remove v from e scope
					scope.remove(v);

					// return v to the remainSet
					remainSet.add(v);

					if(remainSet.size()==bias.getVariablesList().size())
					{
						System.err.println("==========   NO SOLUTION!!  ============"+!solver.isSoluble(NL));
						if(solver.isSoluble(NL))
							return BBE();
						else
							return false;

					}

					//choose another v from e
					v=e.getLastVar();  
					//value=e.getValue(v);

				}


				/************
				 * BEGIN
				 * GENERATE E
				 ***********/

				//---------------------------------------------
				//NL projection on "scope"

				NLy.clear();
				for(AcqConstraint c : NL){								// projection of the scope Y on constraint of NL (i.e., NL[Y])
					if (scope.containsAll(c.getVariablesList())) 					
						NLy.add(c);
				}
				//--------------------------------------------

				// B projection on "scope"
				iterator = B.iterator();
				bY.clear();
				while( iterator.hasNext()) {
					AcqConstraint c= (AcqConstraint) iterator.next();

					if (scope.containsAll(c.getVariablesList())) 				
						bY.add(c);

				}
				//--------------------------------------------

				// B projection on "scope\x"
				bYx.clear();
				bYx=new ArrayList<AcqConstraint>();
				scope.remove(v);
				iterator = bY.iterator();
				while( iterator.hasNext()) {
					AcqConstraint c= (AcqConstraint) iterator.next();

					if (scope.containsAll(c.getVariablesList())) 				
						bYx.add(c);

				}		

				scope.add(v);		// we restore our scope!!
				//-----------------------------------------------------

				//choose e st e |= CL[s] U B[s\x] and sat a max cstr in B[s]

				if(fixedPart){
					for(AcqVariable x: scope){
						if(x!=v)
							NLy.add(new AcqConstraint(acqConstraints.Type.EqualX, x, e.getValue(x)));
					}
				}
				NLy.addAll(bYx);		// CL[s] U B[s\x]
				e1=null;
				e1=solver.singletonMax(NLy,bY, heuristic.MIN_H); 


				// Try again?
				pas=10;
				while(e1==null && pas<1000000)
				{
					solver.setTimeLimit(1000000000);
					e1=solver.singletonMax(NLy,bY, heuristic.MIN_H);
					pas*=10;
				}

				solver.setTimeLimit(1000);

				if(e1!=null)
				{		
					e=e1;
					if(scope.size()<=min_arity)
						flag=false;
					else
						flag=true;
				}
				else			// line 8: no such example, so choose a simple one!!
				{
					NLy.removeAll(bYx);
					e=solver.singletonMax(NLy, bY, heuristic.MIN_H);
					flag=false;
				}

				// Try again?
				pas=10;
				while(e==null && pas<1000000)
				{
					solver.setTimeLimit(1000000000);
					e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
					pas*=10;
				}

				solver.setTimeLimit(1000);


				// Just a projection on scope, because the solve returns a complete example
				if(e==null)
					return BBE();
				e=e.getProjection(scope);
				/************
				 * END
				 * GENERATE E
				 ***********/




			}
			//last line in the first loop

			ke = getKappaB(e);
			B.removeAll(ke);
			logger.debug("REMOVED FROM B==>  "+ke);


		}
		//	System.out.println("NL:  "+NL);
		System.err.println("============ SOLUTION FOUND!! =================");
		NL.removeAll(cleanNL());
		System.err.println(e);

		BiasSize[1]=B.size();

		return true;

	}

	/****************************************************************
	 * backtrack-conacq 02/2014
	 *
	 * @see algo in Bessiere et al. JFPC12
	 * 
	 * 
	 * @author lazaarnadjib
	 ****************************************************************/
	public boolean bconacq(){
		//Disable



		HashSet<AcqVariable> scope= new HashSet<AcqVariable>();
		List<AcqVariable> X = new ArrayList<AcqVariable>();
		List<AcqVariable> remainSet= new ArrayList<AcqVariable>();
		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();
		Collection<AcqConstraint> ke;
		List<Collection<AcqConstraint>> T = new ArrayList<Collection<AcqConstraint>>() ;



		/***
		 * begin INITIALIZATION   (line1 to 3) 
		 */


		boolean backtrack=false;			// I backtrack or not? yes if a given domain is empty
		boolean flag=false;					// true: we use the special findscope, the standard one otherwise

		//STAT
		BiasSize[0]=B.size();

		boolean asked=false;
		int min_arity=bias.getMinArity();
		Example e= new Example(); 

		SortedMap<AcqVariable, ArrayList<Integer>> Bvars = boundToFD(bias.getVariablesList());  // backtrack variables
		remainSet.addAll(bias.getVariablesList());

		if(shuffle)
			Collections.shuffle(remainSet);

		for(int i=0; i<min_arity-1; i++)
		{
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			//OR: int value= Bvars.get(v).get(0);	
			//	OR: int value= one_rand(v.getMin(), v.getMax());			// a random value from min to max included
			int value;
			if(rand_value)
				value= Bvars.get(v).get((int)(Math.random() * (Bvars.get(v).size())));
			else
				value= Bvars.get(v).get(0);
			e.setValue(v, value);
			scope.add(v);
		}

		//  Let us go to line 4 !!

		/***
		 * First loop (line 4) "while e partial do"
		 */

		while(!remainSet.isEmpty()){


			//WHY?
			//	if(e.isPositif()){
			//		ke = getKappaB(e);
			//		B.removeAll(ke);
			//myLog("170=="+ke);  
			//		logger.debug("REMOVED FROM B==>  "+ke);
			//	}

			if(shuffle)
				Collections.shuffle(remainSet);

			// line 5:   x<-- pick(X\scope), here we have lex order on vars
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			scope.add(v);										// s <-- scope(e) 
			int value= Bvars.get(v).get(0);;
			e.setValue(v, value);
			/************
			 * BEGIN
			 * GENERATE E
			 ***********/

			/*			//---------------------------------------------
			//NL projection on "scope"

			NLy.clear();
			for(AcqConstraint c : NL){								// projection of the scope Y on constraint of NL (i.e., NL[Y])
				if (scope.containsAll(c.getVariablesList())) 					
					NLy.add(c);
			}
			//--------------------------------------------


			//choose e st e |= CL[s]

			if(fixedPart){
				for(AcqVariable x: scope){
					if(x!=v)
						NLy.add(new AcqConstraint(AcqConstraint.Type.EqualX, x, e.getValue(x)));
				}
			}

			e=solver.buildSolution(NLy); 

			e=e.getProjection(scope);*/
			boolean notFound=true;
			while(notFound && e.size()!= min_arity-1)
			{

				//		myLog(e, "exemple");
				//	e=solver.buildSolution(NLy); 

				// Just a projection on scope, because the solve returns a complete example

				e=e.getProjection(scope);
				int compt2=0;
				for(Collection<AcqConstraint> kappa: T){
					compt2++;
					int compt=0;
					for(AcqConstraint c: kappa){
						try {
							int[] tuple = e.getTuple(c.getVariables());
							if ( !c.satisfies(tuple)) {
								compt++;
							}
						} catch (NotInstanciatedException ex) {}

					}
					if(compt==kappa.size())
					{

						/********BACKTRACK****************/
						value=e.getValue(v);	
						Bvars.get(v).remove((Object)value);						// remove the inconsistant value
						NL.add(new AcqConstraint(acqConstraints.Type.DiffX, v, value));
						//	backtrack=false;		//line15 in algo branch&ask

						if(mylog)
							myLog("1-var="+v+" val="+value, "pruningLog");

						// if the domain is reduced to empty :
						while(Bvars.get(v).isEmpty()|| !solver.isSoluble(NL)){

							//			backtrack=true;
							// re-initialize the domain of v
							Bvars.get(v).clear();
							Bvars.get(v).addAll(unfold_it(v));
							NL.removeAll(cleanNL(v));
							e.removeVar(v);

							// remove v from e scope
							scope.remove(v);

							// return v to the remainSet
							remainSet.add(v);

							if(remainSet.size()==bias.getVariablesList().size())
							{
								System.err.println("==========   1) NO SOLUTION!!  ============"+!solver.isSoluble(NL));
								return false;

							}

							//choose another v from e
							v=e.getLastVar();  
							//value=e.getValue(v);
							compt2--;

							value=e.getValue(v);	
							Bvars.get(v).remove((Object)value);						// remove the inconsistant value
							NL.add(new AcqConstraint(acqConstraints.Type.DiffX, v, value));

						}

						value= Bvars.get(v).get(0);
						e.setValue(v, value);
						compt2--;



						break;

					}

				}
				if(compt2==T.size())
					notFound=false;

			}
			/************
			 * END
			 * GENERATE E
			 ***********/



			/*****
			 * Second loop (line 10): "while backtrack or ask(e)=no or scope<min_arity do"
			 */


			ke = getKappaB(e);

			if(ke.isEmpty())
				e.setPositive();
			else{
				boolean mutex=false;
				for(int i=0; i<T.size()&& !mutex; i++)
					if(ke.containsAll(T.get(i)))
					{
						//		myLog(ke+"\n\n"+T.get(i), "test");
						mutex=true;
					}

				if(mutex)
				{
					if(ke.size()==1)
						NL.addAll(ke);
					e.setNegative();
				}

				else
					learner.classify(e, false);
				if(e.isNegatif())
					T.add(ke);	



			}

			if(e.isPositif())
				B.removeAll(ke);


			while( e.isNegatif() ){	//|| scope.size()<min_arity
				System.err.println("====> T: size="+T.size());

				System.out.println("====> e: size="+e.size()+" value="+e.isPositif());

				if( remainSet.isEmpty())		//STAT
					completeNQuery++;

				/*******
				 * line 12 : if not backtrack and scope>=min_arity => learn!
				 */

				/**************************************    BACKTRACK STEP   ********************************/
				/******************************************************************************************

				value=e.getValue(v);	
				Bvars.get(v).remove((Object)value);						// remove the inconsistant value
				NL.add(new AcqConstraint(AcqConstraint.Type.DiffX, v, value));
				//	backtrack=false;		//line15 in algo branch&ask
				myLog("2-var="+v+" val="+value, "pruningLog");


				// if the domain is reduced to empty :
				if(Bvars.get(v).isEmpty()|| !solver.isSoluble(NL)){

					//			backtrack=true;
					// re-initialize the domain of v
					Bvars.get(v).clear();
					Bvars.get(v).addAll(unfold_it(v));
					NL.removeAll(cleanNL(v));
					e.removeVar(v);

					// remove v from e scope
					scope.remove(v);

					// return v to the remainSet
					remainSet.add(v);

					if(remainSet.size()==bias.getVariablesList().size())
					{
						System.err.println("==========   2) NO SOLUTION!!  ============"+!solver.isSoluble(NL));
						return false;

					}

					//choose another v from e
					v=e.getLastVar();  
					//value=e.getValue(v);

				}


				/************
				 * BEGIN
				 * GENERATE E
				 ***********//*

				//---------------------------------------------
				//NL projection on "scope"

				NLy.clear();
				for(AcqConstraint c : NL){								// projection of the scope Y on constraint of NL (i.e., NL[Y])
					if (scope.containsAll(c.getVariablesList())) 					
						NLy.add(c);
				}

				//choose e st e |= CL[s] 

				if(fixedPart){
					for(AcqVariable x: scope){
						if(x!=v)
							NLy.add(new AcqConstraint(AcqConstraint.Type.EqualX, x, e.getValue(x)));
					}
				}

				e=solver.buildSolution(NLy); 
				// Just a projection on scope, because the solve returns a complete example
				if(e==null)
					return false;
				e=e.getProjection(scope);
				  */


				notFound=true;
				while(notFound && e.size()!=min_arity-1)
				{
					notFound=false;

					// Just a projection on scope, because the solve returns a complete example

					e=e.getProjection(scope);
					int compt2=0;
					for(Collection<AcqConstraint> kappa: T){
						compt2++;
						int compt=0;
						for(AcqConstraint c: kappa){
							try {
								int[] tuple = e.getTuple(c.getVariables());
								if ( !c.satisfies(tuple)) {
									compt++;
								}
							} catch (NotInstanciatedException ex) {}

						}
						if(compt==kappa.size())
						{

							/********BACKTRACK****************/
							value=e.getValue(v);	
							Bvars.get(v).remove((Object)value);						// remove the inconsistant value
							NL.add(new AcqConstraint(acqConstraints.Type.DiffX, v, value));
							//	backtrack=false;		//line15 in algo branch&ask


							// if the domain is reduced to empty :
							while(Bvars.get(v).isEmpty()|| !solver.isSoluble(NL)){

								//			backtrack=true;
								// re-initialize the domain of v
								Bvars.get(v).clear();
								Bvars.get(v).addAll(unfold_it(v));
								NL.removeAll(cleanNL(v));
								e.removeVar(v);

								// remove v from e scope
								scope.remove(v);

								// return v to the remainSet
								remainSet.add(v);

								if(remainSet.size()==bias.getVariablesList().size())
								{
									System.err.println("2) ==========   NO SOLUTION!!  ============"+!solver.isSoluble(NL));
									return false;

								}

								//choose another v from e
								v=e.getLastVar();  
								//value=e.getValue(v);

							}
							value= Bvars.get(v).get(0);
							e.setValue(v, value);	



							break;

						}

					}
					if(compt2==T.size())
						notFound=false;

				}


				//		myLog(e,"tester");
				/************
				 * END
				 * GENERATE E
				 ***********/


				ke = getKappaB(e);
				if(ke.isEmpty())
					e.setPositive();
				else{
					boolean mutex=false;
					for(int i=0; i<T.size()&& !mutex; i++)
						if(ke.containsAll(T.get(i)))
						{
							//		myLog(ke+"\n\n"+T.get(i), "test");
							mutex=true;
						}
					if(mutex)
					{
						if(ke.size()==1)
							NL.addAll(ke);
						e.setNegative();
					}
					else
						learner.classify(e, false);
					if(e.isNegatif())
						T.add(ke);		
				}

				if(e.isPositif())
					B.removeAll(ke);


			}
			//last line in the first loop



		}
		//	System.out.println("NL:  "+NL);
		System.err.println("============ SOLUTION FOUND!! =================");
		NL.removeAll(cleanNL());
		System.err.println(e);

		BiasSize[1]=B.size();

		return true;


	}

	/****************************************************************
	 * Branch&Ask 04/2013
	 * 
	 * @author lazaarnadjib
	 ****************************************************************/

	public boolean branchAskV1() {			// disable

		//System.out.println(removeDuplicates(B));
		System.out.println(solver.isSoluble(NT));

		HashSet<AcqVariable> scope= new HashSet<AcqVariable>();
		List<AcqVariable> X = new ArrayList<AcqVariable>();
		List<AcqVariable> remainSet= new ArrayList<AcqVariable>();
		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();
		Collection<AcqConstraint> ke;


		//	List<AcqConstraint> exampleCstr = new ArrayList<AcqConstraint>();

		/***
		 * begin INITIALIZATION   (line1 to 4) 
		 */


		boolean backtrack=false;			// I backtrack or not? yes if a given domain is empty

		//	List<AcqVariable> Qvars = new ArrayList<AcqVariable>(bias.getVariablesList());		// quacq variables

		boolean asked=false;
		int min_arity=bias.getMinArity();
		Example e= new Example(); 
		//printExample(e,true,null);		//LOG

		SortedMap<AcqVariable, ArrayList<Integer>> Bvars = boundToFD(bias.getVariablesList());  // backtrack variables
		remainSet.addAll(bias.getVariablesList());

		if(shuffle)
			Collections.shuffle(remainSet);

		for(int i=0; i<min_arity-1; i++)
		{
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			//	int value= one_rand(v.getMin(), v.getMax());			// a random value from min to max included
			int value= Bvars.get(v).get((int)(Math.random() * (Bvars.get(v).size())));	
			e.setValue(v, value);
			scope.add(v);
		}



		/***
		 * First loop (line 5) "while e partial do"
		 */
		while(!remainSet.isEmpty()){

			if(e.isPositif()){
				ke = getKappaB(e);
				B.removeAll(ke);
				//myLog("170=="+ke);  
				logger.debug("REMOVED FROM B==>  "+ke);
			}

			if(shuffle)
				Collections.shuffle(remainSet);
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			scope.add(v);										// s <-- scope(e) 
			int value;
			/*******
			 * 	int value= one_rand(v.getMin(), v.getMax());	

			//	int value= Bvars.get(v).get((int)(Math.random() * (Bvars.get(v).size())));	

			//e.setValue(v, value);								// e<--extend(e)
			// Block replaced by the below code on 22 oct. 13  */

			//---------------------------------------------
			//NL projection on "scope"

			NLy.clear();
			for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
				if (scope.containsAll(c.getVariablesList())) 					
					NLy.add(c);
			}
			//--------------------------------------------

			// B projection on "scope"

			Iterator<AcqConstraint> iterator = B.iterator();
			bY.clear();
			while( iterator.hasNext()) {
				AcqConstraint c= (AcqConstraint) iterator.next();

				if (scope.containsAll(c.getVariablesList())) 				
					bY.add(c);

			}
			//-----------------------------------------------------
			//choose e in sol(CL) such that sat a max cstr in B[s]

			if(fixedPart){
				for(AcqVariable x: scope){
					if(x!=v)
						NLy.add(new AcqConstraint(acqConstraints.Type.EqualX, x, e.getValue(x)));
				}
			}
			Example e1=solver.singletonMax(NLy,bY, heuristic.MIN_H); 


			int pas=10;
			while(e1==null && pas<1000000)
			{
				solver.setTimeLimit(1000000000);
				e1=solver.singletonMax(NLy,bY, heuristic.MIN_H);
				pas*=10;
			}

			solver.setTimeLimit(1000);

			if(e1!=null)
				e=e1;

			else
			{
				// ?! 
				Bvars.get(v).clear();
				asked=true;
				e=solver.buildSolution(NL);
			}
			//	System.out.println(e.size()+"  "+e.isPositif());

			e=e.getProjection(scope);
			//	System.out.println(e.size()+"  "+e.isPositif());					

			/*****
			 * Second loop (line 10): "while backtrack or ask(e)=no or scope<min_arity do"
			 */
			while(backtrack || !learner.classify(e,false) || scope.size()<min_arity){

				//printExample(e,true,null);		//LOG

				if( remainSet.isEmpty())		//STAT
					completeNQuery++;

				/*******
				 * line 11 : if not backtrack and scope>=min_arity => learn!
				 */
				if((!backtrack)  && scope.size()>=min_arity){
					////// LEARNING STEP  --BEGIN
					X.clear();
					emptySet.clear();
					X.addAll(scope);
					X.remove(v);
					emptySet.add(v);
					/*******************************************************************************************/
					List<AcqVariable> temp= findScope(e,X,false, emptySet);
					temp.add(v);
					AcqConstraint c_delta = findCstr(temp,e);			// learnt constraint
					/*******************************************************************************************/

					if(mylog) {
						myLog("learnt==> "+c_delta);
						myLog("from: "+e);
					}
					//myLog("==> "+c_delta);

					if(c_delta!=null)
						if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
						{
							//	printExample(e,true,c_delta);

							ArrayList<AcqConstraint> checkNT= new ArrayList<AcqConstraint>();
							checkNT.addAll(NT);
							checkNT.add(c_delta);
							if(solver.isSoluble(checkNT))
								NL.add(c_delta);
							else{
								System.out.println("NL:"+NL);
								System.out.println("delta"+c_delta);
								System.out.println("scope"+scope);
							}
							System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
							logger.debug("REMOVED FROM B==>  "+c_delta);
							logger.debug(" B==>  "+B);
						}
						else{

							NL.add(c_delta);
						}
					else
					{
						c_delta = findCstr(findScope(e,X,false, emptySet),e);			// learnt constraint

						//		printExample(e, false,c_delta);
						logger.debug("==========================  ");
						logger.debug(" B==>  "+B);
						logger.debug("NL ==> "+NL);
						logger.debug("example==>"+e);
						logger.debug("==========================  ");
						System.err.println("<<<<<<<<<<<<   NO CSTR "+c_delta+"    >>>>>>>>>>>>>>");
						//		Toolkit.getDefaultToolkit().beep();
					}



					System.out.println("ITERATION=> B size: "+ B.size()+ " NL size:" + NL.size()+ "asked Q: "+ askedQ);
					//////// LEARNING STEP --- END
				}
				//			System.out.print(">>"+e);

				if(printSize)									// printf function on Query size
					printQsize(e.size());

				value=e.getValue(v);
				Bvars.get(v).remove((Object)value);						// remove the inconsistant value
				NL.add(new AcqConstraint(acqConstraints.Type.DiffX, v, value));
				// myLog("(var, val) == "+v+"  "+value);
				backtrack=false;		//line15 in algo branch&ask


				// if the domain is reduced to empty :
				if(Bvars.get(v).isEmpty()|| !solver.isSoluble(NL)){

					backtrack=true;
					// re-initialize the domain of v
					Bvars.get(v).clear();
					Bvars.get(v).addAll(unfold_it(v));
					NL.removeAll(cleanNL(v));

					// remove v from e
					e.removeVar(v);

					// remove v from e scope
					scope.remove(v);

					// return v to the remainSet
					remainSet.add(v);

					if(remainSet.size()==bias.getVariablesList().size())
					{
						System.err.println("==========   NO SOLUTION!!  ============"+!solver.isSoluble(NL));
						if(solver.isSoluble(NL))
							return askSolve();
						else
							return false;

					}

					//choose another v from e
					v=e.getLastVar();  
					// set v to the last value in the search-tree
					value=e.getValue(v);
					// remove from v the last value
					//	Bvars.get(v).remove((Object)e.getValue(v));

					// reset v to the next value
					//		e.setValue(v, Bvars.get(v).get(0));

				}
				else
				{
					//value=Bvars.get(v).get(0);
					//e.setValue(v, value);			replaced by the below code!!

					// NL projection on "scope"
					NLy.clear();
					for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
						if (scope.containsAll(c.getVariablesList())) 					
							NLy.add(c);
					}
					// add fixed part of e as constraints !!
					if(fixedPart){
						for(AcqVariable x: scope){
							if(x!=v)
								NLy.add(new AcqConstraint(acqConstraints.Type.EqualX, x, e.getValue(x)));
						}
					}
					//--------------------------------------------


					// B projection on "scope"

					Iterator<AcqConstraint> iterator2 = B.iterator();
					bY.clear();
					while( iterator2.hasNext()) {
						AcqConstraint c= (AcqConstraint) iterator2.next();

						if (scope.containsAll(c.getVariablesList())) 				
							bY.add(c);

					}
					//-----------------------------------------------------
					//choose e in sol(CL) such that sat a max cstr in B[s]


					e1=solver.singletonMax(NLy,bY, heuristic.MIN_H); 


					pas=10;
					while(e1==null && pas<1000000)
					{
						solver.setTimeLimit(1000000000);
						e1=solver.singletonMax(NLy,bY, heuristic.MIN_H);
						pas*=10;
					}
					solver.setTimeLimit(1000);

					if(e1!=null)
						e=e1;

					else
					{
						e=solver.buildSolution(NL);
						//				Bvars.get(v).clear();
						//			asked=true;
					}
					//	System.out.println(e.size()+"  "+e.isPositif());
					//		if(e!=null)
					e=e.getProjection(scope);

					//	System.out.println(e.size()+"  "+e.isPositif());					
				}

				/***********  FAULT !!
			ke = getKappaB(e);
				B.removeAll(ke);
				logger.debug("REMOVED FROM B==>  "+ke);

				 ******************/

			}


		}
		//	System.out.println("NL:  "+NL);
		System.err.println("============ SOLUTION FOUND!! =================");
		NL.removeAll(cleanNL());
		System.err.println(e);

		return true;
	}


	private int removeDuplicates(List<AcqConstraint> b2) {
		int duplicates = 0;

		for ( int i = 0; i < b2.size(); i++ )
		{
			for ( int j = 0; j < b2.size(); j++ )
			{
				if ( i == j )
				{
					// i & j refer to same entry so do nothing
				}

				else if ( b2.get( j ).equals( b2.get( i ) ) )
				{
					//	myLog(b2.get(j));
					b2.remove( j );
					duplicates++;
				}
			}
		}

		return duplicates;
	}

	/****************************************************************
	 * Learn&Solve with an extend and a reduce function  03/2013
	 * 
	 * @author lazaarnadjib
	 ****************************************************************/



	public boolean learnSolve_disable() {			// enable

		// definitions
		List<AcqVariable> scope= new ArrayList<AcqVariable>();
		List<AcqVariable> remainSet= new ArrayList<AcqVariable>();
		List<AcqVariable> variables= new ArrayList<AcqVariable>(bias.getVariablesList());
		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();

		int min_arity=bias.getMinArity();

		BiasSize[0]=B.size();


		Example e= solver.buildSolution(NL);
		// Initialization :

		scope.addAll(bias.getVariablesList());

		if(shuffle)
			Collections.shuffle(scope);

		if(printSize)
			printQsize(e.size());

		while(!learner.classify(e,false))
		{

			AcqVariable v= scope.get(0);
			scope.remove(0);
			remainSet.add(v);
			e=e.getProjection(scope);

			System.out.println("\n DOWN: e==> "+e.size()+ " scope ==> "+ scope.size());
			if(printSize)
				printQsize(e.size());

		}

		if(scope.size()==bias.getNbVariables())
			return true;
		else{

			Collection<AcqConstraint> ke = getKappaB(e);
			B.removeAll(ke);

			boolean mutex1,mutex2;
			while(!remainSet.isEmpty()){

				if(shuffle)
					Collections.shuffle(remainSet);
				AcqVariable v= remainSet.get(0);
				remainSet.remove(0);
				int value= one_rand(v.getMin(), v.getMax());	
				e.setValue(v, value);								// e<--extend(e)
				//scope.add(v);										// s <-- scope(e) 


				emptySet.add(v);
				mutex1=false;
				mutex2=true;

				if(printSize)
					printQsize(e.size());


				while(!learner.classify(e,false)){



					if(remainSet.isEmpty())
						completeNQuery++;
					if(mutex1)
						emptySet.clear();

					AcqConstraint c_delta = findCstr(findScope(e,scope,false, emptySet),e);			// learnt constraint
					if (c_delta!=null) 
					{
						NL.add(c_delta);
						System.out.println("<<<<<<<<<<<<   "+c_delta+"    >>>>>>>>>>>>>>");
					}
					else
					{
						System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
						Toolkit.getDefaultToolkit().beep();

					}
					System.out.println("ITERATION=> B size: "+ B.size()+ " NL size:" + NL.size()+ "asked Q: "+ askedQ);

					// NL projection on "scope"
					NLy.clear();
					for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
						if (scope.containsAll(c.getVariablesList())) 					
							NLy.add(c);
					}
					//--------------------------------------------


					// B projection on "scope"

					Iterator<AcqConstraint> iterator = B.iterator();
					bY.clear();
					while( iterator.hasNext()) {
						AcqConstraint c= (AcqConstraint) iterator.next();

						if (scope.containsAll(c.getVariablesList())) 				
							bY.add(c);

					}
					//-----------------------------------------------------
					//choose e in sol(CL) such that sat a max cstr in B[s]

					e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
					e=e.getProjection(scope);
					mutex1=true;
					if(mutex2)
						scope.add(v);
					mutex2=false;
					if(printSize)
						printQsize(e.size());

				}

				ke = getKappaB(e);
				B.removeAll(ke);

			}

		}

		BiasSize[1]=B.size();

		return true;
	}


	/****************************************************************
	 * Learn&Solve : only with a extand function
	 *  
	 * @author lazaarnadjib
	 ****************************************************************/



	public boolean learnSolve() {

		// definitions
		HashSet<AcqVariable> scope= new HashSet<AcqVariable>();
		List<AcqVariable> X = new ArrayList<AcqVariable>();
		List<AcqVariable> remainSet= new ArrayList<AcqVariable>();
		List<AcqVariable> variables= new ArrayList<AcqVariable>(bias.getVariablesList());
		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();

		BiasSize[0]=B.size();

		///////////////////////
		/// TEST PART

		System.out.println(solver.buildSolution(NT));
		///////////////////////

		int min_arity=bias.getMinArity();

		Example e= new Example(); 
		//			solver.buildSolution(NL);

		// Initialization :

		remainSet.addAll(bias.getVariablesList());

		if(shuffle)
			Collections.shuffle(remainSet);

		for(int i=0; i<min_arity-1; i++)  // from 0 to min-arity-1
		{
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			int value= one_rand(v.getMin(), v.getMax());
			e.setValue(v, value);
			scope.add(v);
		}

		while(!remainSet.isEmpty()){

			if(shuffle)
				Collections.shuffle(remainSet);
			AcqVariable v= remainSet.get(0);
			remainSet.remove(0);
			int value= one_rand(v.getMin(), v.getMax());	
			e.setValue(v, value);								// e<--extend(e)
			scope.add(v);										// s <-- scope(e) 


			while(!learner.classify(e,false)){

				if(printSize)
					printQsize(e.size());


				if(remainSet.isEmpty())
					completeNQuery++;
				X.clear();
				X.addAll(scope);
				List<AcqVariable> scopeOn=findScope(e,X,false, emptySet);
				AcqConstraint c_delta = findCstr(scopeOn,e);			// learnt constraint
				// myLog(X);

				List<AcqConstraint> lc_delta = new ArrayList<AcqConstraint>();

				if(generalization){

					if (c_delta != null && c_delta.getVariables().length == 2) {
						List<TypeVar> list_type = new ArrayList<TypeVar>();
						boolean methodeComplete = true;
						if (methodeComplete)
							list_type.addAll(generalisetable2(c_delta));
						else
							list_type.addAll(generalisetable2Incomplete(c_delta));

						//						 list_type.addAll(generalisetable2Rand(c_delta));

						for (int i = 0; i < list_type.size(); i += 2) {

							TypeVar t_i = list_type.get(i);
							TypeVar t_j = list_type.get(i + 1);

							if (t_i.getName().equals(t_j.getName())) {
								for (int j = 0; j < t_i.getVars().length - 1; j++)
									for (int k = j + 1; k < t_i.getVars().length; k++)
										lc_delta.add(new AcqConstraint(c_delta
												.getType(), t_i.getVars()[j], t_i
												.getVars()[k]));
							} else
								for (AcqVariable var1 : t_i.getVars())
									for (AcqVariable var2 : t_j.getVars()) {
										if (!var1.equals(var2)) {
											AcqConstraint cst = new AcqConstraint(
													c_delta.getType(), var1, var2);
											if (!cst.isInListOfConstraint(lc_delta)
													&& !cst.isInListOfConstraint(NL))
												lc_delta.add(cst);
										}
									}
						}
					}
				}
				if(c_delta!=null)
					if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
					{
						ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
						ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
						checkNT1.addAll(NT);
						checkNT2.addAll(NT);
						checkNT1.add(c_delta);
						checkNT2.add(c_delta.negation());

						if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2))
							NL.add(c_delta);
						else{
							System.out.println("NL:"+NL);
							System.out.println("delta"+c_delta);
							System.out.println("scope"+scope);
						}
						System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
						logger.debug("REMOVED FROM B==>  "+c_delta);
					}
					else{
						NL.add(c_delta);
					}
				else
				{
					//mylog=true;
					scopeOn=findScope(e,X,false, emptySet);
					//myLog("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
					System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>"+ scopeOn);
					Toolkit.getDefaultToolkit().beep();
				}


				System.out.println("LearnSolve=> B size: "+ B.size()+ " NL size:" + NL.size()+ "asked Q: "+ askedQ);
				System.err.println(c_delta+"==>"+ new Date());


				// NL projection on "scope"
				NLy.clear();
				for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
					if (scope.containsAll(c.getVariablesList())) 					
						NLy.add(c);
				}
				//--------------------------------------------


				// B projection on "scope"

				Iterator<AcqConstraint> iterator = B.iterator();
				bY.clear();
				while( iterator.hasNext()) {
					AcqConstraint c= (AcqConstraint) iterator.next();

					if (scope.containsAll(c.getVariablesList())) 				
						bY.add(c);

				}
				//-----------------------------------------------------
				//choose e in sol(CL) such that sat a max cstr in B[s]

				e=solver.singletonMax(NLy,bY, heuristic.MIN_H); 
				int pas=10;
				while(e==null)
				{
					solver.setTimeLimit(1000*pas);
					e=solver.singletonMax(NLy,bY, heuristic.MIN_H);
					pas*=10;
				}
				solver.setTimeLimit(1000);
				e=e.getProjection(scope);

			}

			Collection<AcqConstraint> ke = getKappaB(e);
			B.removeAll(ke);
		}

		BiasSize[1]=B.size();

		return true;
	}


	/****************************************************************
	 * Learn&Solve : Cleaning a negative one
	 * 
	 * @author lazaarnadjib
	 ****************************************************************/



	public boolean learnSolve_disable2() {

		// definitions
		List<AcqVariable> scope= new ArrayList<AcqVariable>();
		List<AcqVariable> remainSet= new ArrayList<AcqVariable>();
		List<AcqVariable> variables= new ArrayList<AcqVariable>(bias.getVariablesList());
		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();
		List<AcqConstraint> oneCstr = new ArrayList<AcqConstraint>();

		int min_arity=bias.getMinArity();
		int it_count=0;
		Example e=solver.buildSolution(NL);

		// Initialization :


		while(!learner.classify(e,false))
		{

			if(printSize)
				printQsize(e.size());


			if(it_count==-1)
			{
				scope= findScope(e.getProjection(variables.subList(76, e.size())), variables.subList(76, e.size()), false, emptySet);
			}
			scope= findScope(e,variables,false, emptySet);
			AcqConstraint c_delta = findCstr(scope,e);			// learnt constraint
			System.out.println("\n e==> "+e.size()+ " scope ==> "+ scope.size());
			if (c_delta.equals(null)) 
				throw new RuntimeException("Collapsed");
			else
				NL.add(c_delta);
			Example ep= solver.buildSolution(NL).getProjection(c_delta.getVariablesList());

			System.out.println("\n c==> "+c_delta);
			System.out.println("\n ep==> "+ep);

			System.out.println("\n e==> "+e);

			e=e.merge(ep);
			System.out.println("\n e==> "+e);			

			it_count++;
		}


		return true;
	}

	
	public boolean QuAcq() {
		return  QuAcq(0);
	}

	/****************************************************************
	 * QuAcq
	 * 
	 * @author lazaarnadjib
	 * @param findAll : if true, use the findAllScopes function
	 ****************************************************************/

	public boolean QuAcq(int version) {
		recordedQueries = new ArrayList<Integer>();
		recordedTime = new ArrayList<Double>();
		recordedQSizes = new ArrayList<Integer>();
		recordedQTimes = new ArrayList<Double>();
		
		startTime = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;

		//Verify the presence of FAS files and ask the user to delete them or not
		if(QAexp.RECORDSFA){
			final File folder = new File(Config.getExpDir());
			final File[] files = folder.listFiles(new FileFilter(){
			    public boolean accept(File pathname){
			    	return (pathname.getName().startsWith("FAS") && pathname.getName().endsWith(".log"));
			    }
			});
						
			if(files.length > 0){
				Scanner sc = new Scanner(System.in);
				System.out.print("There exist FASFiles. Do you want to remove and replace them by new ones for this XP ? (y,yes/n,no/a,abort)\n> ");
				String str = sc.nextLine();
				sc.close();
				if(str.equals("yes") || str.equals("y")){
					for ( final File file : files ) {
					    if ( !file.delete() ) {
					        System.err.println( "Can't remove " + file.getAbsolutePath() );
					    }
					}
				}else if(str.equals("abort") || str.equals("a")){
					throw new RuntimeException("Aborted by user !");
				}
			}
		}
		
		
		/*//-------------------------begin  Checker
		//System.err.println("||||||||||||||||| is NT a subset of B? =>"+ NtIsinBias()+ " ||||||||||||||||");
		//printNTandB();
			System.out.println(NT);
		System.out.println(solver.buildSolution(NT));
		System.out.println(solver.buildNextSolution(NT));
		//-------------------------begin  Checker */
		System.out.println("nbSols : "+solver.getNbSolutions(NT));
		System.out.println("Solution ? -> "+solver.buildSolution(NT));
		System.out.println(B.size());
		if(modelSeeker_init)
			jumpInLearning();
		BiasSize[0]=B.size();
		ThreadMXBean thread = ManagementFactory.getThreadMXBean();
		//	printNLsize(true);																							// print the NL and B size at the first iteration
		//printBsize(true);

		while(true){

			if(mylog)
				myLog(B.size(), "bSize");

			//	printBsize(false);
			//				System.out.println("NADJIB=====>\n"+solver.buildSolution(learner.constraints));
			//				System.out.println("NADJIB=====>\n"+solver.buildNextSolution(learner.constraints));
			//	System.out.println(solver.getNbSolutions(learner.constraints));
			// swapping
			/*			if(B.size()<(BiasSize[0]/2))  // swap
			{H=heuristic.SOL_H;
				sol=true;}
			if(B.size()<(BiasSize[0]/4))  // swap
			{H=heuristic.MAX_H;
				sol=false;
				max_it=true;}*/
			/*if(NL.size()>NL_size)	{	printNLsize(false);																					// print the NL and B size at each iteratio }*/
			//	e= solver.notAllViolated(NL, B, bias.getVariablesList()););
			//		System.out.println("solution of NL: "+ solver.buildSolution(NL));
			/*			if(NL.size()==NL_size){stuck++; tabu++;}
			if(tabu==5){
				logger.debug("TABU SWAP!!");
				H=heuristic.SOL_H;
				tabu=0;}*/

			nbIt++;

			/****************************************************** STATE
			queriesPerIt=learner.getNbQuery()-queriesPerIt;
			queriesSizePerIt=learner.getSizeQ()-queriesSizePerIt;
			if(queriesPerIt==0) queriesPerIt=1;
			myLog((double)(queriesSizePerIt/queriesPerIt));
			queriesPerIt=learner.getNbQuery();
			queriesSizePerIt=learner.getSizeQ();
			 ******************************************************/


			System.out.println("QUACQ n: "+nbIt+" B size: "+ B.size()+ " NL size:" + NL.size()+ "nb peel:"+nbpeelit+ "asked Q:"+askedQ);
			logger.info("**************   ITERATION "+ nbIt+"   *********************");
			logger.debug("NL=> B:"+ B.size());
			logger.debug("NL=> learnt:"+ NL.size()+ NL);
			logger.debug("NL=> target:"+ learner.constraints.size());
			logger.info("********************************************** " );

			Example e = null;
			if(peelIt){
				System.out.println("====== peelit now =====");
				double peelTime = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
				e=peelIt(NL);
				System.err.println("Peel duration : " + ((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - peelTime) + " sec.");
				nbpeelit++;
				peelIt=false;
				solver.setTimeLimit(QAexp.TIMEOUT);
			}
			else if(randomFirstEx){
				e = new Example();
				Random c = new Random();
				List<AcqVariable> varList = bias.getVariablesList();
				for(AcqVariable v : varList){
					e.setValue(v, c.nextInt(v.getMax() - v.getMin()) + v.getMin());
				}
				randomFirstEx = false;
			}
			else e=solver.singletonMax(NL,B,H);
			
			max=solver.max;
			
			/* if(e==null && solver.isSoluble(NL)){
				solver.setTimeLimit(10000);
				e = solver.singletonMax(NL,B,heuristic.SOL_H);			// convergence false
				solver.setTimeLimit(1000);	}*/		
			if (e == null && B.size()!=0) {  // no solution
				logger.info("\n\n\n*******************************************   TIMEOUT   **********************************");
				logger.debug("NL=> B:"+ B.size() + B);
				logger.debug("NL=> learnt:"+ NL.size()+ NL);
				logger.debug("NL=> target:"+ learner.constraints.size()+ learner.constraints);
				logger.info("\n********************************************************************************************\n\n\n");
				//	logger.debug("NL soluble?: "+solver.isSoluble(NL));
				//	logger.debug("NT is soluble?"+ solver.isSoluble(NT));
				//		logger.debug("B is soluble?"+ solver.isSoluble(B));
				//		logger.debug("NL minus NT:");
				//		print_minus(NL,NT);
				//	logger.debug("NT minus NL:");
				//	print_minus(NT,NL);
				solver.setTimeLimit(1000*100);
				if(solver.isSoluble(NL))
				{
					solver.setTimeLimit(1000);
					peelIt = true;	
				}

				else throw new RuntimeException("Collapsed");
			}
			else {	

				if(max==B.size()|| B.size()==0)   //convergence of B on Nt
				{
					solver.setTimeLimit(1000*100);
					if(solver.isSoluble(NL))
					{
						solver.setTimeLimit(1000);
					}

					else throw new RuntimeException("Collapsed");
					
					if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
					if(QAexp.RECORDSFA) recordFAS_Q((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - timeRemoved - startTime, learner.getNbQuery() - qRemoved);


					BiasSize[1]=0;
					logger.info("\n\n\n*******************************************   CQFT   **********************************");
					logger.debug("NL=> B:"+ B.size() + B);
					logger.debug("NL=> learnt:"+ NL.size()+ NL);
					logger.debug("NL=> target:"+ learner.constraints.size()+ learner.constraints);
					logger.info("\n********************************************************************************************\n\n\n");
					System.err.println("************************************************************");
					System.err.println("************************************************************");
					System.err.println("***************   NL Convergence !!  ***********************");
					System.err.println("************************************************************");
					System.err.println("************************************************************");
					System.out.println("REDUNDANT: "+ redundant);
					System.out.println("solution of NT: "+ solver.buildSolution(NT));
					System.out.println("solution of NL: "+ solver.buildSolution(NL));
					System.out.println("asked Q: "+ askedQ + " nbQ+:"+learner.getYanswer()+ " nbQ-:"+learner.getNanswer()+ " nbQueries:"+learner.getNbQuery());


					/*					print_NT_minus_NL();
					System.out.println("\n\n\n$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n\n");
					print_NT_inter_B();
					System.out.println(solver.buildNextSolution(NL));*/
					logger.debug("NL soluble?: "+solver.isSoluble(NL));
					logger.debug("B is soluble?"+ solver.isSoluble(B));
					
					//System.err.println("Total number of recursive calls to FAS : " + nbVisitedNodes);
					
					if(QAexp.RECORDSTD){
						System.out.println("Computing averages and standard deviations...");
						setAndprintStandardDeviation();
						System.out.println("FAS Files : "+fasFiles);
					}
					
					if(QAexp.RECORDSFA) System.out.println("FAS Files : "+fasFiles);
					
					return true;
				}

				if(printSize)
					printQsize(e.size());
				
				//record the size of the next query
				if(noFindCQueries) qRemoved++;
				else queryUpdate(e.size(), true);

				if (learner.classify(e,false)) {
					if(mylog)
						myLog("10", "exp");
					completePQuery++;
					partialEx=false;
					Collection<AcqConstraint> ke=getKappaB(e);
					B.removeAll(ke);
					logger.debug("REMOVED FROM B==>  "+ke);
					if(succes)
					{
						BiasSize[1]=B.size();
						System.err.println("***************   Positive solution !!  ***********************");
						logger.debug("NL soluble?: "+solver.isSoluble(NL));
						logger.debug("B is soluble?"+ solver.isSoluble(B));
						/*	double end = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime();				// remove commant if need to have (succes/convergence) in same time
						end = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime();
						QAexp.s_times[QAexp.current_exp]=end-QAexp.start;
						QAexp.s_query[QAexp.current_exp]=QuickAcq.learner.getNbQuery();
						QAexp.s_queryCPQ[QAexp.current_exp]=QuickAcq.getCompletePQuery();
						QAexp.s_queryCNQ[QAexp.current_exp]=QuickAcq.getCompleteNQuery();
						QAexp.s_queryPPQ[QAexp.current_exp]=QuickAcq.learner.getYanswer();
						QAexp.s_queryPNQ[QAexp.current_exp]=QuickAcq.learner.getNanswer();
						QAexp.s_sizeQ[QAexp.current_exp]=QuickAcq.learner.getSizeQ();
						QAexp.s_reducB[QAexp.current_exp]=getBsize(1);
						QAexp.c_histo_Q[QAexp.current_exp]=QuickAcq.learner.getHistoQ();
						QAexp.s_NL[QAexp.current_exp]=QuickAcq.getNLsize();

						//query[i]=Conacq.learner.getNbQuery();
						if(QAexp.current_exp== QAexp.s_sizeQ.length-1)
						{SimpleDateFormat dateFormat = new SimpleDateFormat(  
									"yyyy-MM-dd" );  
							GregorianCalendar calendar = new GregorianCalendar();
							String fileName=Config.getExpDir()+
									dateFormat.format(calendar.getTime())+".log";
							QAexp.printResults(average(QAexp.s_times),average(QAexp.s_NL), average(QAexp.s_query), average(QAexp.s_sizeQ),average(QAexp.s_queryCPQ), average(QAexp.s_queryCNQ), 
									average(QAexp.s_queryPPQ), average(QAexp.s_queryPNQ), 0, average(QAexp.s_reducB),QAexp.printHisto(false),fileName);
						}*/

						return true;			// @ADVICE put in commont if succes/con needed in same time
					}
				} else {

					if(mylog)
						myLog("0", "exp");

					if(checklearning && NL.size()!=NL_size) 		// DEBUG step: checking stuck state
						stuck=0;
					NL_size=NL.size();
					if(stuck==100)
					{
						QAexp.setStuck();
						return true;
					}
					//	if(!pick_heuristic(e))				// TODO !!
					
					//RA: we choose the right version to launch here
					if(version == 1){
						try{
							learnt=quAcqFindAll(e);
						}catch(TimeoutException re1){
							//System.err.println("Timed out !");

							if(reverseRestart){
								reverseVars = true;
								
								double now = (ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0);

								if(QAexp.RECORDSFA) recordFAS_Q(now - timeRemoved - startTime, learner.getNbQuery() - qRemoved);
								lastFASTime = now;

								if(queryTimeOut == 1) lastQRecorded = now;
								if (MNSTimeOut == 1) lastMNSRecorded = now;
								
								try{
									learnt=quAcqFindAll(e);
								}catch(TimeoutException re2){
									//System.err.println("Timed out !");
									learnt = true;
								}
								
								reverseVars = false;
								
							}else{
								learnt = true;
							}
						}
						
						double now = (ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0);
						if(QAexp.RECORDSFA) recordFAS_Q(now - timeRemoved - startTime, learner.getNbQuery() - qRemoved);
						lastFASTime = now;
					}
					else if(version == 2){
						learnt=quAcqRev(e);
					}
					else learnt=quAcq(e);
					
					
					if(learnt)
						completeNQuery++;
				}
			}
			/*													// NL: TIMEOUT OFF
				if((end-start)/1000000000>600)
				{
				TIMEOUT=true;
				return false;
				}*/

		}


	}




	/****************************************************************
	 * modelSeeker simulator 04/2014
	 *
	 * @version 1
	 * 
	 * @author lazaarnadjib
	 ****************************************************************/



	private  List<AcqConstraint> modelSeeker(Collection<AcqVariable> scope, boolean flag) {

		List<AcqConstraint> model= new ArrayList<AcqConstraint>();

		if(flag){
			List<AcqConstraint> delta= minus(NT,NL);
			for(AcqConstraint c : delta){
				if (scope.containsAll(c.getVariablesList())) 					
				{
					model.add(c);
					System.out.println("ModelSeeker ==> "+c);
					cst_MS++;
				}
			}
		}
		//System.err.println("======================================================================================================> MS:"+cst_MS);
		return model;

	}

	/*****************************************
	 * 21-05-2013 (Cork)
	 * 
	 * 
	 * @author lazaarnadjib
	 */

	private void jumpInLearning() {
		System.out.println(MS);
		B.removeAll(MS);
		NL.addAll(MS);					// comment on sudoku and zebra like
		for(AcqConstraint c : MS){
			//		if(c.getType().equals(Type.AllDiff))
			//		NL.add(c);
			AcqConstraint oppositeC = bias.getOpposite(c);	
			if(oppositeC!=null)
				B.remove(oppositeC);	
		}
	}


	/****************************************************************
	 * pick_heuristic   COMEBACKTOME
	 * @param e: a complete negative example
	 * 
	 * optim (Manu heuristic):
	 * @author lazaarnadjib
	 * @throws NotInstanciatedException 
	 ****************************************************************/


	private boolean pick_heuristic(Example e) throws NotInstanciatedException {
		Random generator = new Random();
		int rand = generator.nextInt(B.size()-1);

		AcqConstraint c= B.get(rand);
		int[] tuple = e.getTuple(c.getVariables());
		if (!c.satisfies(tuple)) 
		{
			NL.add(c);
			B.remove(rand);
			return true;
		}
		B.remove(c);
		return false;
	}


	/****************************************************************
	 * QUICKACQ
	 * @param e: a complete negative example
	 * 
	 * @author lazaarnadjib
	 ****************************************************************/

	private boolean quAcq(Example e){

		List<AcqVariable> emptySet = new ArrayList<AcqVariable>();
		List<AcqVariable> scope=new ArrayList<AcqVariable>();
		List<AcqVariable> variables= new ArrayList<AcqVariable>(bias.getVariablesList());

		boolean flag=true;

		while(flag && e.isNegatif()){

			partialEx = false;
			bigKappa = getKappaB(e.getProjection(variables));


			if(shuffle_scope)
				Collections.shuffle(variables);

			if(elucidate)
				scope= elucidate(e);
			else
				scope= findScopeRev(e,variables,false, emptySet);

			if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());


			//printScope(scope);

			/*		
		if(scopeTabu)
			scope= findScope(e,variables,false, emptySet);
		else 
		{
			scope=elucidate(e);
			if(scope.isEmpty())
			{
				Collections.shuffle(variables);
				scope= findScope(e,variables,false, emptySet);
			}
			scopeTabu=true;
		}*/


			AcqConstraint c_delta = findCstrRev(scope,e);			// learnt constraint

			/*if(scopeSave.equals(scope))					// tabu search on the found scope
			scopeTabu=false;
		scopeSave=scope;
		logger.debug("swap splitting: "+scopeTabu);*/
			//BEGIn ZEBRA

			List<AcqConstraint> lc_delta = new ArrayList<AcqConstraint>();

			if(generalization){

				if (c_delta != null && c_delta.getVariables().length == 2) {
					List<TypeVar> list_type = new ArrayList<TypeVar>();
					boolean methodeComplete = true;
					if (methodeComplete)
						list_type.addAll(generalisetable2(c_delta));
					else
						list_type.addAll(generalisetable2Incomplete(c_delta));

					//						 list_type.addAll(generalisetable2Rand(c_delta));

					for (int i = 0; i < list_type.size(); i += 2) {

						TypeVar t_i = list_type.get(i);
						TypeVar t_j = list_type.get(i + 1);

						if (t_i.getName().equals(t_j.getName())) {
							for (int j = 0; j < t_i.getVars().length - 1; j++)
								for (int k = j + 1; k < t_i.getVars().length; k++)
									lc_delta.add(new AcqConstraint(c_delta
											.getType(), t_i.getVars()[j], t_i
											.getVars()[k]));
						} else
							for (AcqVariable var1 : t_i.getVars())
								for (AcqVariable var2 : t_j.getVars()) {
									if (!var1.equals(var2)) {
										AcqConstraint cst = new AcqConstraint(
												c_delta.getType(), var1, var2);
										if (!cst.isInListOfConstraint(lc_delta)
												&& !cst.isInListOfConstraint(NL))
											lc_delta.add(cst);
									}
								}
					}
				}
			}
			
			
			return checkC_Delta(c_delta, scope, e);
			

//			if(c_delta!=null)
//				if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
//				{
//					
//					double now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
//					timeRemoved -= now;
//					
//					ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
//					ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
//					checkNT1.addAll(NT);
//					checkNT2.addAll(NT);
//					checkNT1.add(c_delta);
//					checkNT2.add(c_delta.negation());
//
//					if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2))
//
//						if (lc_delta.size() != 0)
//							NL.addAll(lc_delta);
//						else
//							NL.add(c_delta);
//
//
//					else{
//						System.out.println("NL:"+NL);
//						System.out.println("=>>>>>> delta"+c_delta);
//						System.out.println("scope"+scope);
//						
//						now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
//						timeRemoved += now;
//						return false;
//					}
//					System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
//					//B.removeAll(delta);
//					///END ZEBRA
//
//
//					//NL.add(c_delta);
//					B.remove(c_delta);							// Nadjib: here an inscruction commented before=> find why
//					logger.debug("REMOVED FROM B==>  "+c_delta);
//					
//					now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
//					timeRemoved += now;
//				}
//				else{
//					if (lc_delta.size() != 0)
//						NL.addAll(lc_delta);
//					else
//						NL.add(c_delta);
//
//				}
//			else
//			{
//				System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
//				Toolkit.getDefaultToolkit().beep();
//				return false;
//			}
//
//			if(allScope_flag){
//
//				for(AcqVariable v : scope)
//					e.removeVar(v);
//				variables.removeAll(scope);
//				if(variables.isEmpty())
//					flag=false;
//				else{
//					if(noFindCQueries) qRemoved++;
//					else queryUpdate(e.size(), true);
//					learner.classify(e,false);
//				}
//
//			}
//			else
//				flag=false;

		}
		return true;
	}

	/****************************************************************
	 *  FindScope procedure
	 * @param e : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @param Delta : aside variables
	 * @param Bgd : background variables
	 * @return  variable scope
	 * 
	 * Author: NL
	 ****************************************************************/
	private List<AcqVariable> findScope(Example e, List<AcqVariable> X, boolean mutex, List<AcqVariable> Bgd){

		//System.out.println(">>"+e+"  |  "+X+ "  |  "+Bgd);
		//if(mylog) myLog("findScope= "+X+" "+mutex+" "+Bgd);
		if (mutex 
				//&& Bgd.size()>= bias.getMinArity()  <- FIXME : minArity doesnt have the good value !!
				) {
			Example eBgd = e.getProjection(Bgd);      // projection e|Bgd
			partialEx=false;
			boolean valueOfEx=true;		// the valueOf the partial example
			boolean asked=false;
			Collection<AcqConstraint> ke=getKappaB(eBgd); 

			if(check_singleton(eBgd))			// pour les cases remplis du sudoku 
				//1	if( ke.size()!=0)
				//1	{
				if(with_m){
					Example ex=checkMemory(eBgd);
					if(ex!=null)
					{
						valueOfEx= ex.isPositif();
						askedQ++;
						if(valueOfEx)
							asked=true;

						if(printMemory)  
							printMemory(eBgd,ex,true);
					}
					else
					{
						queryUpdate(eBgd.size(), true);
						valueOfEx=learner.classify(eBgd,complexQ_flag);
						memory.add(eBgd);
						if(printSize)
							printQsize(eBgd.size());

					}
				}
				else
				{
					queryUpdate(eBgd.size(), true);
					valueOfEx=learner.classify(eBgd,complexQ_flag);

					if(printSize)
						printQsize(eBgd.size());

				}

			//	if(mylog) myLog("     ask("+eBgd+")="+valueOfEx);

			if(!valueOfEx) {

				List<AcqVariable> emptySet= new ArrayList<AcqVariable>();

				//		if(restart)
				//			nb_negatives++;

				return emptySet;
			} else {

				if(!complexQ_flag && ke.size()==0 && !asked) {   // optim
					learner.Yanswer--;
				}

				B.removeAll(ke);
				logger.debug("REMOVE FROM B: using "+ eBgd + " : "+ ke);

				// call of modelSeeker simulator on partial positives

				System.out.println("partial positive e:"+eBgd);
				NL.addAll(modelSeeker(Bgd,modelSeeker));
			}
			//1	}	
		}


		if(X.size()==1) return X;

		List<AcqVariable> X1= new ArrayList<AcqVariable>();
		List<AcqVariable> X2= new ArrayList<AcqVariable>();
		List<AcqVariable> S1= new ArrayList<AcqVariable>();
		List<AcqVariable> S2= new ArrayList<AcqVariable>();
		List<AcqVariable> BgdUX2 = new ArrayList<AcqVariable>();
		List<AcqVariable> BgdUS1 = new ArrayList<AcqVariable>();
		List<AcqVariable> S1US2 = new ArrayList<AcqVariable>();


		int subSize; 			//FIXME different splitting manners can be defined here!

		//		if(scopeTabu)
		subSize = X.size() / 2; 			//FIXME different splitting manners can be defined here!


		int cpt = 0;
		for(AcqVariable v : X) {
			if(cpt++ < subSize) 
				X1.add(v);
			else 
				X2.add(v);
		}
		/*
		while ()
		X1 = new HashSet<AcqVariable>(X.subList(0, subSize));
		X2 = new HashSet<AcqVariable>(X.subList(subSize, X.size()));
		 */

		BgdUX2.addAll(Bgd);
		BgdUX2.addAll(X2);

		S1=findScope(e,X1,true,BgdUX2);    // First recursive call of quAcq

		BgdUS1.addAll(Bgd);
		BgdUS1.addAll(S1);
		mutex= !S1.isEmpty();
		S2=findScope(e,X2,mutex,BgdUS1);    // Second recursive call of quAcq

		S1US2.addAll(S1);
		S1US2.addAll(S2);

		return S1US2;
	}




	/****************************************************************
	 * QUICKACQFINDALL
	 * FindAllScopes version
	 * @param e: a complete negative example
	 * 
	 * @author Robin Arcangioli
	 ****************************************************************/

	private boolean quAcqFindAll(Example e){
		
		fasStartTime = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
		fasQ = learner.getNbQuery();


		double now = (ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0);
		
		if(noFindCQueries){
			if(firstEx){
				timeRemoved = now - startTime;
			}else{
				timeRemoved += (now - lastFASTime);
			}
		}
		
		
		if(QAexp.RECORDSFA){
			recordFASFile = Config.getExpDir() + "FASStats" + (((int) fasStartTime) % 1000) + ".log";
			recordFAS_QFile = "FAS_Q" + (((int) fasStartTime) % 1000) + ".log";
			fasFiles += recordFAS_QFile + " ";
			recordFAS_QFile = Config.getExpDir() + recordFAS_QFile;
			
			if(firstEx){
				recordFAS(0, 0);
				recordFAS_Q(0, 0);
			}else{
				recordFAS_Q(now - timeRemoved - startTime,  learner.getNbQuery() - qRemoved);
			}
		}

		if(reverseVars){
			Collections.reverse(bias.getVariablesList());
		}else{
			if(!firstEx){
				if(changeRestart == 1){
					AcqVariable x1;
					for(int i = indexOfLastVarFound - 1; i >= 0; i--){
						x1 = bias.getVariablesList().get(i);
						bias.getVariablesList().remove(i);
						bias.getVariablesList().add(x1);
					}
				}else if(changeRestart == 2){
					Collections.shuffle(bias.getVariablesList());
				}
			}
			firstEx = false;
			
			MNSes = new ArrayList<List<AcqVariable>>();
		}
		
		ArrayList<AcqVariable> variables= new ArrayList<AcqVariable>(bias.getVariablesList());
		
		System.out.println("Variables order : " + variables);
		MYSes = new ArrayList<List<AcqVariable>>();

		
		//NL: 16-11-15  shuffle on variables 
			//if(shuffle_scope)
				//Collections.shuffle(variables);


		
		//For the weight heuristic
		varWeights = new TreeMap<String, Integer>();
		for(AcqVariable v : variables) varWeights.put(v.getName(), 0);
		
		//findAllScopes(e, variables);  //RA: dicho version
		findAllScopes2(e, variables); //RA: non dicho version
		//findAllScopes2BreadthFirst(e, variables); //RA: non dicho version breadth first
		//findAllScopesBottomUp2(e, variables); //RA: from bottom version breadth first


		int sizeMNSes = MNSes.size();

		if(sizeMNSes != 0) return true;
		return false;
	}


	/****************************************************************
	 * QUACQREV
	 * FindScopeRev version
	 * @param e: a complete negative example
	 * 
	 * @author Robin Arcangioli
	 ****************************************************************/
	private boolean quAcqRev(Example e){
		
		ArrayList<AcqVariable> X = new ArrayList<AcqVariable>(bias.getVariablesList());
		
		MNSes = new ArrayList<List<AcqVariable>>(); //FIXME : toRemove, we dont need the set MNSes
		HS = new ArrayList<List<AcqVariable>>();

		partialEx = false;
		bigKappa = getKappaB(e.getProjection(X));

		//first call
		List<AcqVariable> scope = findScopeRev(e, X, false, new ArrayList<AcqVariable>());
		
		if(scope == null || scope.isEmpty()) return false;
		
		for(AcqVariable v : scope){
			List<AcqVariable> hs = new ArrayList<AcqVariable>();
			hs.add(v);
			HS.add(hs);
		}

		//System.out.println("\033[36mNew HS (i = 0) : "+HS+"\033[0m");
		
		MNSes.add(scope); //FIXME : toRemove, we dont need the set MNSes
		
		AcqConstraint c_delta = findCstrRev(scope, e);
		
		if( ! checkC_Delta(c_delta, scope, e)) return false;
		
		List<AcqVariable> vars;
		
		while(! HS.isEmpty()){
			vars = new ArrayList<AcqVariable>(X);
			vars.removeAll(HS.get(0));
			
			//System.out.println("\033[36mGenerating : "+vars+"\033[0m");
			
			List<AcqConstraint> ke = getKappaProj(vars);
			Example eProj = e.getProjection(vars);
			
			if(ke.isEmpty()){
				
				//System.out.println("\033[36mKappa empty for : "+eProj+"\033[0m");
				HS.remove(0);
				
			}else{
				
				queryUpdate(vars.size(), true);
				if(learner.classify(eProj, false)){
					//System.out.println("\033[36mNo scope to find on : "+eProj+"\033[0m");
					
					B.removeAll(ke);
					bigKappa.removeAll(ke);
					
					HS.remove(0);
					
				}else{
				
					//System.out.println("\033[36mLaunching findScopeRev on : "+vars+"\033[0m");
					
					scope = findScopeRev(e, vars, false, new ArrayList<AcqVariable>());
				
					//System.out.println("\033[36mFound : "+scope+"\033[0m");
					
					double now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;

					if(limitHit && HS.size() > limitHitSize) HS = HS.subList(0, limitHitSize);
					updateHS(scope);
					maxHSSize = Math.max(maxHSSize, HS.size());
					
					currUDTime = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0 - now;
					maxUDTime = (currUDTime > maxUDTime)? currUDTime : maxUDTime;
					cumulUDTime += currUDTime;

					System.out.println("\033[36mHS update time : "+currUDTime+"\033[0m");
					System.out.println("\033[36mHS max update time : "+maxUDTime+"\033[0m");
					System.out.println("\033[36mHS total update time : "+cumulUDTime+"\033[0m");
					System.out.println("\033[36mHS max size : "+maxHSSize+"\033[0m");
					
//					if(HS.size() > 25) System.out.println("\033[36mNew HS : "+HS.subList(0, Math.min(HS.size(), 10))+" ... "+HS.subList(HS.size()-10, HS.size())+"\033[0m");
//					else System.out.println("\033[36mNew HS : "+HS+"\033[0m");

					
					MNSes.add(scope); //FIXME : toRemove, we dont need the set MNSes
					
					partialEx = false;
					c_delta = findCstrRev(scope,e);
					
					if( ! checkC_Delta(c_delta, scope, e)) return false;
				}
			}
		}
		
		return true;
	}

	/****************************************************************
	 *  updateHS procedure
	 * @param HS : the hitting set of a set S
	 * @param scope : a new element of S
	 * @return the hitting set on S U scope
	 * 
	 * Author: RA
	 ****************************************************************/
	private void updateHS(List<AcqVariable> scope){
		
		List<AcqVariable> hsi, cloneScope;
		
		for(int i = 0; i < HS.size(); i++){
			hsi = new ArrayList<AcqVariable>(HS.get(i));
			cloneScope = new ArrayList<AcqVariable>(scope);
			
			if(! cloneScope.removeAll(hsi)){
				
				HS.remove(i);
				i--;
				
				for(AcqVariable v : scope){
					//We insert nhs = hs U v and remove subsets
					List<AcqVariable> hsiUv = new ArrayList<AcqVariable>(hsi);
					hsiUv.add(v);
					
					orderedAINS(hsiUv);
				}
			}
		}
	}


	//WARNING : Require That the elements of M are ordered by increasing size
	//AINS : Add If No Subsets
	private void orderedAINS(List<AcqVariable> Z){
		
		int sizeZ = Z.size();
		int sizeHS = HS.size();
		List<AcqVariable> hsi;
		
		for(int i = 0; i < sizeHS; i++){
			hsi = HS.get(i);
			
			if(Z.containsAll(hsi)) return;
			
			else if(hsi.size() >= sizeZ){
				HS.add(i, Z);
				return;
			}
		}
		
		HS.add(Z);
	}

	/****************************************************************
	 *  FindScopeRev procedure
	 * @param e : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @param Delta : aside variables
	 * @param Bgd : background variables
	 * @return  variable scope
	 * 
	 * Author: RA
	 ****************************************************************/
	private List<AcqVariable> findScopeRev(Example e, List<AcqVariable> X, boolean mutex, List<AcqVariable> Bgd){

//		if(containsSubset(MNSes, X)){
//			System.err.println("Bad hitting set  with : "+X+" with MNSes : "+MNSes); //FIXME : toRemove, we dont need the set MNSes
//			throw new RuntimeException("Bad Hitting Set !");
//		}
		
		if (mutex) {
			
			Collection<AcqConstraint> ke = getKappaProj(Bgd); 
			Example eBgd = e.getProjection(Bgd);

			if(! ke.isEmpty()){
				
				queryUpdate(eBgd.size(), true);
				if(learner.classify(eBgd,false)) {
	
					B.removeAll(ke);
					bigKappa.removeAll(ke);
	
				}else return new ArrayList<AcqVariable>();
			}
		}


		if(X.size()==1) return X;

		List<AcqVariable> X1= new ArrayList<AcqVariable>();
		List<AcqVariable> X2= new ArrayList<AcqVariable>();
		List<AcqVariable> S1= new ArrayList<AcqVariable>();
		List<AcqVariable> S2= new ArrayList<AcqVariable>();
		List<AcqVariable> BgdUX2 = new ArrayList<AcqVariable>();
		List<AcqVariable> BgdUS1 = new ArrayList<AcqVariable>();
		List<AcqVariable> S1US2 = new ArrayList<AcqVariable>();


		int subSize; 			//FIXME different splitting manners can be defined here!

		//		if(scopeTabu)
		subSize = X.size() / 2; 			//FIXME different splitting manners can be defined here!


		int cpt = 0;
		for(AcqVariable v : X) {
			if(cpt++ < subSize) 
				X1.add(v);
			else 
				X2.add(v);
		}
		/*
		while ()
		X1 = new HashSet<AcqVariable>(X.subList(0, subSize));
		X2 = new HashSet<AcqVariable>(X.subList(subSize, X.size()));
		 */

		BgdUX2.addAll(Bgd);
		BgdUX2.addAll(X2);

		S1=findScopeRev(e,X1,true,BgdUX2);    // First recursive call of quAcq

		BgdUS1.addAll(Bgd);
		BgdUS1.addAll(S1);
		mutex= !S1.isEmpty();
		S2=findScopeRev(e,X2,mutex,BgdUS1);    // Second recursive call of quAcq

		S1US2.addAll(S1);
		S1US2.addAll(S2);

		return S1US2;
	}
	
	
	//cheack if the constraint learnt is the right one, else it picks and returns the right one
	private boolean checkC_Delta(AcqConstraint c_delta, List<AcqVariable> scope, Example e){
		
		System.out.println("->  " + c_delta + " has been found with "+e.getProjection(scope));
		
		if(!checklearning){
			NL.add(c_delta);
			return true;
		}
		
		//If the learnt constraint accepts NT we return it with no more check
		ArrayList<AcqConstraint> checkNT = new ArrayList<AcqConstraint>(NT);
		checkNT.add(c_delta.negation());
		if(!solver.isSoluble(checkNT)){
			System.out.println("*********Check Learning********* KEPT : "+c_delta);
			bigKappa.remove(c_delta);
			B.remove(c_delta);
			NL.add(c_delta);
			return true;
		}
		

		double now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
		timeRemoved -= now;
		
		System.out.println("Delta init : "+deltaInit);
		boolean learnt = false;
		//We try to find the needed constraints in the initial delta
		for(AcqConstraint c : deltaInit){
			for(AcqConstraint cNT : NT){
				if(c.equals(cNT)){
					learnt = true;
					System.out.println("*********Check Learning********* GOT : "+c);
					bigKappa.remove(c);
					B.remove(c);
					NL.add(c);
				}
			}
		}
		
		if(learnt){
			if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
			if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
			
			now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
			timeRemoved += now;
			
			return true;
		}
		
//		//If the needed constraint is not in Delta, then we find the needed conjunction of constraints in NT...
//		List<AcqConstraint> cstConj = new ArrayList<AcqConstraint>();
//		int[] tuple;
//		for(AcqConstraint c : NT) {    
//
//			try {
//				tuple = e.getTuple(c.getVariables());
//				if (!c.satisfies(tuple)) {
//					cstConj.add(c);     
//				}
//			} catch (NotInstanciatedException ex) {
//
//			}
//		}
//		
//		//...then we try to find a conjunction in deltaInit equivalent to cstConj
//		ArrayList<AcqConstraint> toLearn = equiConj(cstConj, deltaInit);
//		
//		if(!toLearn.isEmpty()){
//			System.out.println("*********Check Learning********* GOT : "+toLearn);
//			NL.addAll(toLearn);
//			bigKappa.removeAll(toLearn);
//			B.removeAll(toLearn);
//			if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
//			if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
//			
//			now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
//			timeRemoved += now;
//			
//			return true;
//			
//		}

		ArrayList<AcqConstraint> checkNT1, checkNT2;
		//Else we try to find a needed conjunction of constraints in the initial delta
		for(AcqConstraint c : deltaInit){
			checkNT1= new ArrayList<AcqConstraint>(NT);
			checkNT2= new ArrayList<AcqConstraint>(NT);
			checkNT1.add(c);
			checkNT2.add(c.negation());
			
			if(solver.getNbSolutions(checkNT1) == solver.getNbSolutions(NT) && !solver.isSoluble(checkNT2)){
				learnt = true;
				System.out.println("*********Check Learning********* GOT : "+c);
				NL.add(c);
				bigKappa.remove(c);
				B.remove(c);
			}
		}
		
		if(learnt){
			if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
			if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
			
			now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
			timeRemoved += now;
			
			return true;
		}
		
		System.out.println("=>>>>>> delta"+c_delta);
		System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
		Toolkit.getDefaultToolkit().beep();
		
		now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
		timeRemoved += now;
			
		return false;
	}


	/****************************************************************
	 *  equiConj function
	 * @param conj : a conjunction of constraints
	 * @param cstSet : a set of constraints
	 * @return  a conjunction of constraints in cstSet equivalent to conj
	 * 
	 * Author: Robin Arcangioli
	 ****************************************************************/
	ArrayList<AcqConstraint> equiConj(List<AcqConstraint> conj, List<AcqConstraint> cstSet){
		ArrayList<AcqConstraint> equiConj = new ArrayList<AcqConstraint>();
		
		//This is a two steps function : first we try to get the maximal number of constraints
		//s.t. their conjunction is equivalent to conj, then we refine this conjunction by
		//removing non-filtering constraints
		
		//First step :
		ArrayList<AcqConstraint> checkNet = new ArrayList<AcqConstraint>();
		for(AcqConstraint c : cstSet){
			checkNet = new ArrayList<AcqConstraint>(conj);
			checkNet.add(c.negation());
			
			if(solver.isSoluble(checkNet)){
				equiConj.add(c);
			}
		}
		
		//Second step :
		int i = 0;
		while(i < equiConj.size()){
			checkNet = new ArrayList<AcqConstraint>(equiConj);
			checkNet.remove(i);
			checkNet.add(equiConj.get(i).negation());
			
			if(!solver.isSoluble(checkNet)){
				equiConj.remove(i);
				
			}else i++;
		}
		
		return equiConj;
	}

	/****************************************************************
	 *  FindAllScopes procedure
	 * @param e : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @return  variable scopes
	 * 
	 * Author: Robin Arcangioli
	 ****************************************************************/

	private void findAllScopes(Example e, List<AcqVariable> X){

		partialEx = false;
		bigKappa = getKappaB(e.getProjection(X));

		int size = X.size();
		if(size%2 != 0) size = (size/2) + 1;
		else size /= 2;

		List<AcqVariable> Z = firstSubset(X, size);

		while(!Z.isEmpty()){
			//System.out.println(Z);

			List<AcqVariable> XmoinsZ = new ArrayList<AcqVariable>();
			XmoinsZ.addAll(X);
			XmoinsZ.removeAll(Z);
			
			//RA: WARNING, works but may be incomplete (need proof)
			//XmoinsZ = removeAllInf(XmoinsZ, Z.get(0));

			findMNSes(e, Z, new ArrayList<AcqVariable>(), XmoinsZ);
			Z = nextSubset(Z, X);
		}
		//System.err.println("Number of recursive calls to FAS : " + nbVisitedNodes);
	}

	private boolean findMNSes(Example e, List<AcqVariable> R, List<AcqVariable> S, List<AcqVariable> Y){

		//System.out.print(".");
		
		//nbVisitedNodes++;
		
		//To always have the same order on variables
		SortedSet<AcqVariable> sortedRUS = new TreeSet<AcqVariable>(new VariableComparator());
		//Variables are inserted using the comparator
		sortedRUS.addAll(R);
		sortedRUS.addAll(S);
		List<AcqVariable> RUS = new ArrayList<AcqVariable>();
		RUS.addAll(sortedRUS);

		if(MNSes.contains(RUS)) return false;

		Example eProj = e.getProjection(RUS);
		if(useKappaCondition){
			List<AcqVariable> RUSUY = new ArrayList<AcqVariable>();
			RUSUY.addAll(RUS);
			RUSUY.addAll(Y);
			if(getKappaProj(RUSUY).isEmpty()) return true;
		}
		
		int rusSize = RUS.size();

		boolean containsSub = (rusSize == 1)? false : containsSubset(MNSes, RUS);

		if(!containsSub){
			boolean yes = false;
			if(!useKappaCondition){
				if(containsSuperset(MYSes, RUS)){
					yes = true;
				}else if(learner.classify(eProj,complexQ_flag)){

					queryUpdate(rusSize, false);
					MYSes = addAndRemoveSubsets(MYSes, RUS);
					
					yes = true;
					
					Collection<AcqConstraint> ke=getKappaProj(RUS);
					
					if(!ke.isEmpty()){
						B.removeAll(ke);
						bigKappa.removeAll(ke);
						System.out.println("Removed (with positive " + eProj + ") : "+ke);
					}
				}
			}else{
				Collection<AcqConstraint> ke=getKappaProj(RUS);
				
				if(ke.isEmpty() || learner.classify(eProj,complexQ_flag)){
					yes = true;
				
					if(!ke.isEmpty()){
						queryUpdate(rusSize, false);
						B.removeAll(ke);
						bigKappa.removeAll(ke);
						System.out.println("Removed (with positive " + eProj + ") : "+ke);
					}
				}
			}
			
			if(yes){
	
				int size = Y.size();
	
				if(size <= 1) return true;
				
				if(size%2 != 0) size = (size/2) + 1;
				else size /= 2;
				List<AcqVariable> Z = firstSubset(Y, size);
				
				boolean returnValue = true;
	
				List<AcqVariable> SUZ;
				List<AcqVariable> YmoinsZ;
				
				while(!Z.isEmpty()){
					SUZ = new ArrayList<AcqVariable>();
					SUZ.addAll(S);
					SUZ.addAll(Z);
	
					YmoinsZ = new ArrayList<AcqVariable>();
					YmoinsZ.addAll(Y);
					YmoinsZ.removeAll(Z);
	
					returnValue = (findMNSes(e, R, SUZ, YmoinsZ) && returnValue);
					Z = nextSubset(Z, Y);
				}
	
				return returnValue;
			}

			queryUpdate(rusSize, false);
		}

		boolean isMNS = true;
		int sizeS = S.size();
		int sizeR = R.size();

		if(sizeS > 1){

			int size;
			if(sizeS%2 != 0) size = (sizeS/2) + 1;
			else size = sizeS/2;

			List<AcqVariable> Z = firstSubset(S, size);

			List<AcqVariable> SmoinsZ;
			
			while(!Z.isEmpty()){
				SmoinsZ = new ArrayList<AcqVariable>();
				SmoinsZ.addAll(S);
				SmoinsZ.removeAll(Z);
				
				//RA: WARNING, not complete !
				//SmoinsZ = removeAllInf(SmoinsZ, (R.isEmpty())? Z.get(0) : ((Z.get(0).compareTo(R.get(0)) < 0)? Z.get(0) : R.get(0)));

				isMNS = (findMNSes(e, R, Z, SmoinsZ) && isMNS);
				Z = nextSubset(Z, S);
			}
		}

		if(sizeR == 1 && isMNS) isMNS = (findMNSes(e, new ArrayList<AcqVariable>(), S, new ArrayList<AcqVariable>()) && isMNS);

		else if(sizeR > 1 && isMNS){

			int size;
			if(sizeR%2 != 0) size = (sizeR/2) + 1;
			else size = sizeR/2;

			List<AcqVariable> Z = firstSubset(R, size);

			List<AcqVariable> RmoinsZ;
			
			while(!Z.isEmpty()){
				RmoinsZ = new ArrayList<AcqVariable>();
				RmoinsZ.addAll(R);
				RmoinsZ.removeAll(Z);
				
				//RA: WARNING, not complete !
				//RmoinsZ = removeAllInf(RmoinsZ, (S.isEmpty())? Z.get(0) : ((Z.get(0).compareTo(S.get(0)) < 0)? Z.get(0) : S.get(0)));

				isMNS = (findMNSes(e, Z, S, RmoinsZ) && isMNS);
				Z = nextSubset(Z, R);
			}
		}

		if(isMNS && !containsSub){

			List<AcqVariable> RUSclone = new ArrayList<AcqVariable>(RUS);
			MNSes.add(RUSclone);
			System.out.println("Added : " + e.getProjection(RUS));

			partialEx = false;
			AcqConstraint c_delta = findCstr(RUS,e);
			
			if(c_delta!=null)
				if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
				{
					ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
					ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
					checkNT1.addAll(NT);
					checkNT2.addAll(NT);
					checkNT1.add(c_delta);
					checkNT2.add(c_delta.negation());
					
					if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2)) NL.add(c_delta);
					
					else{
						System.out.println("NL:"+NL);
						System.out.println("=>>>>>> delta"+c_delta);
						System.out.println("scope"+RUS);
						return false;
					}
					System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
					//B.removeAll(delta);
					///END ZEBRA
					if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
					if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);


					//NL.add(c_delta);
					B.remove(c_delta);							// Nadjib: here an inscruction commented before=> find why
					bigKappa.remove(c_delta);
					logger.debug("REMOVED FROM B==>  "+c_delta);
				}
				else{

					if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
					if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
					NL.add(c_delta);
				}
			else
			{
				System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
				Toolkit.getDefaultToolkit().beep();
				
				//FIXME: It's dangerous here ! Uncomment me !
				//throw new RuntimeException("Collapsed");
			}
		}

		return false;
	}
	
	
	private boolean isSameArity(List<AcqConstraint> C){
		int arity = C.get(0).getVariablesList().size();
		int sizeC = C.size();
		for(int i = 1; i < sizeC; i++){
			if(C.get(i).getVariablesList().size() != arity) return false;
		}
		
		return true;
	}
	

	private List<AcqVariable> removeAllInf(List<AcqVariable> X, AcqVariable y){
		List<AcqVariable> toRemove = new ArrayList<AcqVariable>();

		for(AcqVariable x : X){
			if(x.getName().compareTo(y.getName()) < 0) toRemove.add(x);
		}

		X.removeAll(toRemove);
		return X;
	}

	//WARNING : Require a global order on variables of lists Z and Mi !
	//(better than the naive method using containsAll but there should
	//even be a better method using differents structures)
	private boolean containsSuperset(List<List<AcqVariable>> M, List<AcqVariable> Z){
		int i, j, sizeZ, sizeMi;
		sizeZ = Z.size();
		for(List<AcqVariable> Mi : M){
			sizeMi = Mi.size();
			i = -1; j = -1;
			while(i<sizeZ && j<sizeMi){
				i++; j++;
				if(i < sizeZ){
					while(j<sizeMi && (!Mi.get(j).getName().equals(Z.get(i).getName()))){
						if(Mi.get(j).getName().compareTo(Z.get(i).getName()) > 0) j = sizeMi;
						j++;
					}
				}
			}
			if(i == sizeZ){
				return true;
			}
		}

		return false;
	}

	//WARNING : Require a global order on variables of lists Z and Mi !
	//(better than the naive method using containsAll but there should
	//even be a better method using differents structures)
	private boolean containsSubset_disable(List<List<AcqVariable>> M, List<AcqVariable> Z){
		int i, j, sizeZ, sizeMi;
		sizeZ = Z.size();
		for(List<AcqVariable> Mi : M){
			sizeMi = Mi.size();
			i = -1; j = -1;
			while(i<sizeMi && j<sizeZ){
				i++; j++;
				if(i < sizeMi){
					while(j<sizeZ && (!Z.get(j).getName().equals(Mi.get(i).getName()))){
						if(Z.get(j).getName().compareTo(Mi.get(i).getName()) > 0) j = sizeZ;
						j++;
					}
				}
			}
			if(i == sizeMi){
				return true;
			}
		}

		return false;
	}
	
	//Not to use, too slow (but doesnt require any ordering)
	private boolean containsSubset(List<List<AcqVariable>> M, List<AcqVariable> Z){
		for(List<AcqVariable> Mi : M){
			if(Z.containsAll(Mi)) return true;
		}
		
		return false;
	}

	//WARNING : Require a global order on variables of lists Z and Mi !
	//(better than the naive method using containsAll but there should
	//even be a better method using differents structures)
	private List<List<AcqVariable>> addAndRemoveSubsets(List<List<AcqVariable>> M, List<AcqVariable> Z){
		List<List<AcqVariable>> toRemove = new ArrayList<List<AcqVariable>>();
		int i, j, sizeZ, sizeMi;
		sizeZ = Z.size();
		for(List<AcqVariable> Mi : M){
			sizeMi = Mi.size();
			i = -1; j = -1;
			while(i<sizeMi && j<sizeZ){
				i++; j++;
				if(i < sizeMi){
					while(j<sizeZ && (!Z.get(j).getName().equals(Mi.get(i).getName()))){
						if(Z.get(j).getName().compareTo(Mi.get(i).getName()) > 0) j = sizeZ;
						j++;
					}
				}
			}
			if(i == sizeMi) toRemove.add(Mi);
		}

		M.removeAll(toRemove);
		M.add(Z);
		return M;
	}

	private List<List<AcqVariable>> addAndRemoveSupersets(List<List<AcqVariable>> M, List<AcqVariable> Z){
		List<List<AcqVariable>> toRemove = new ArrayList<List<AcqVariable>>();
		for(List<AcqVariable> Mi : M){
			if(Mi.containsAll(Z)) toRemove.add(Mi);
		}

		M.removeAll(toRemove);
		M.add(Z);
		return M;
	}

	private List<AcqVariable> firstSubset(List<AcqVariable> X, int size){
		List<AcqVariable> Z = new ArrayList<AcqVariable>();
		for(int i = 0; i < size; i++) Z.add(X.get(i));
		return Z;
	}

	//Compute and return the next subset of Z of size |Z| in X (Z has to be a subset of X)
	private List<AcqVariable> nextSubset(List<AcqVariable> Z, List<AcqVariable> X){
		int sizeZ = Z.size();
		int sizeX = X.size();
		int index;

		for(index = 1; index <= sizeZ; index++){
			if(!(Z.get(sizeZ - index).getName().equals(X.get(sizeX - index).getName()))) break;
		}

		if(index > sizeZ) return new ArrayList<AcqVariable>();

		int indOfVarX = X.indexOf(Z.get(sizeZ - index)) + 1;

		Z.set(sizeZ - index, X.get(indOfVarX));

		for(int i = sizeZ - index + 1; i < sizeZ; i++){
			for(int j = indOfVarX + 1; j < sizeX; j++){
				AcqVariable x = X.get(j);
				if(!(Z.contains(x))){
					Z.set(i, x);
					break;
				}
			}
		}

		return Z;
	}
	
	private List<List<AcqVariable>> computeSubsets(List<AcqVariable> X){
		List<List<AcqVariable>> subsets = new ArrayList<List<AcqVariable>>();
		int i = X.size();
		
		List<AcqVariable> clone;
		while(--i >= 0){
			clone = new ArrayList<AcqVariable>(X);
			clone.remove(i);
			
			subsets.add(new ArrayList<AcqVariable>(clone));
		}
		
		return subsets;
	}
	
	private int getWeight(List<AcqVariable> set){
		int weight = 0;
		for(AcqVariable x : set){
			weight += varWeights.get(x.getName());
		}
		return weight;
	}
	
	private List<AcqVariable> popLessWeighted(List<List<AcqVariable>> X){
		int sizeX = X.size();
		if(sizeX == 0) return new ArrayList<AcqVariable>();
		
		List<AcqVariable> Xi = X.get(0);
		
		if(sizeX == 1){
			X.remove(0);
			return Xi;
		}
		
		if(getWeight(Xi) == 0){
			X.remove(0);
			return Xi;
		}
		
		int min = getWeight(Xi);
		int indice = 0;
		int weighti;
		
		int i = 1;
		for(; i < sizeX; i++){
			Xi = X.get(i);
			weighti = getWeight(Xi);
			
			if(weighti == 0){
				X.remove(i);
				return Xi;
			}
			
			if(weighti < min) indice = i;
		}
		
		Xi = X.get(indice);
		X.remove(indice);
		return Xi;
	}
	


	/**********************************************************************************
	 * getKappaProj
	 * @param scope
	 * @return
	 **********************************************************************************/

	public List<AcqConstraint> getKappaProj(List<AcqVariable> scope) {

		List<AcqConstraint> ke = new ArrayList<AcqConstraint>();

		for(AcqConstraint c : bigKappa) {
			if (scope.containsAll(c.getVariablesList())) {
				ke.add(c);     
			}
		}
		
		//System.out.println("bigK : " + bigKappa + "\nscope : " + scope + " - ke : " + ke);
		
		return ke;
	}


	/****************************************************************
	 *  FindAllScopes2 procedure
	 * @param e : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @return  variable scopes
	 * 
	 * Author: Robin Arcangioli
	 ****************************************************************/

	private void findAllScopes2(Example e, List<AcqVariable> X){


		partialEx = false;
		bigKappa = getKappaB(e.getProjection(X));

		if(varWeightsHeuristic){
			List<List<AcqVariable>> subsets = computeSubsets(X);
			
			List<AcqVariable> set;
			while(!subsets.isEmpty()){
				set = popLessWeighted(subsets);
				
				findMNSes2(e, set, 0);
			}
			
		}else{
			int lrg = 0;
			int i = X.size();
			
			List<AcqVariable> Z;
			while(--i >= 0){
				Z = new ArrayList<AcqVariable>(X);
				Z.remove(i);
	
				findMNSes2(e, Z, lrg);
				lrg++;
			}

		}
		
		//System.err.println("Number of recursive calls to FAS : " + nbVisitedNodes);
	}
	

	private boolean findMNSes2(Example e, List<AcqVariable> X, int lrg){

		//nbVisitedNodes++;
		//System.out.println("cutoff==> "+cutoff);
		//myLog(nbVisitedNodes,"counter");
		
		checkTimeOut();
		
		//System.out.println("Visited : " + X + " with lrg : " + lrg);
		//System.err.print(".");
		
		if(MNSes.contains(X)) return false;
		
		Example eProj = e.getProjection(X);
		Collection<AcqConstraint> ke=getKappaProj(X);
		
		if(ke.isEmpty()){
			//System.out.println("ke empty");
			return true;
		}

		int xsize = X.size();
		boolean containsSub = ((xsize == 1)? false : containsSubset(MNSes, X));

		if(!containsSub){
			
			//Updating timers and recording query size and time if needed
			queryUpdate(xsize, true);
			
			if(learner.classify(eProj,complexQ_flag)){
				consPosQ++;
				B.removeAll(ke);
				bigKappa.removeAll(ke);
				System.out.println("Removed (with positive example : "+eProj+") : "+ke);

				return true;
			}else{
				consPosQ = 0;
			}
		}
		//System.out.println("example false");

		boolean isMNS = true;

		
		if(varWeightsHeuristic){
			List<List<AcqVariable>> subsets = computeSubsets(X);
			
			List<AcqVariable> set;
			
			while(!subsets.isEmpty()){
				set = popLessWeighted(subsets);
				
				isMNS = (findMNSes2(e, set, 0) && isMNS);
			}
			
		}else{
			int i = xsize - lrg;
			List<AcqVariable> Z;
			
			while(--i >= 0){
				Z = new ArrayList<AcqVariable>(X);
				Z.remove(i);
				
				isMNS = (findMNSes2(e, Z, lrg) && isMNS);
				
				lrg++;
			}

		}

		if(isMNS && !containsSub){
			//MNSes = addAndRemoveSupersets(MNSes, X);
			List<AcqVariable> Xclone = new ArrayList<AcqVariable>(X);
			indexOfLastVarFound = bias.getVariablesList().indexOf(Xclone.get(0)) + 1;
			//Xclone = reverse(Xclone);
			MNSes.add(0, Xclone);
			//System.out.println("Added : " + e.getProjection(X));
			
			double now;
			
			if(noFindCQueries){
				now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
				timeRemoved -= now;
			}

			partialEx = false;
			AcqConstraint c_delta = findCstrRev(X,e);

			now = (ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0);
			if(queryTimeOut == 1) lastQRecorded = now;
			if (MNSTimeOut == 1) lastMNSRecorded = now;
			
			if(noFindCQueries) timeRemoved += now;
			
			checkC_Delta(c_delta, X, e);
			
			
		}
		
		return false;
	}


	/****************************************************************
	 *  FindAllScopes2 procedure
	 * @param e : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @return  variable scopes
	 * 
	 * Author: Robin Arcangioli
	 ****************************************************************/

	private void findAllScopes2BreadthFirst(Example e, List<AcqVariable> X){

		partialEx = false;
		bigKappa = getKappaB(e.getProjection(X));
		
		findMNSes2BreadthFirst(e, X, 0, true);
		
		//System.err.println("Number of recursive calls to FAS : " + nbVisitedNodes);
	}
	
//	private boolean findMNSes2BreadthFirst(Query e, List<AcqVariable> X, int lrg){
//		
//		checkTimeOut();
//		
//		if(MNSes.contains(X)){
//			return false;
//		}
//
//		Query eProj = e.getProjection(Y);
//		Collection<AcqConstraint> ke=getKappaProj(Y);
//		
//		if(ke.isEmpty()){
//			//System.out.println("ke empty");
//			lrg++;
//			return true;
//		}
//		
//		int xsize = X.size();
//		boolean containsSub = ((xsize == 1)? false : containsSubset(MNSes, Y));
//
//		List<List<AcqVariable>> S = new ArrayList<List<AcqVariable>>();
//		List<Integer> L = new ArrayList<Integer>();
//		List<AcqVariable> Y;
//		int ysize;
//		
//		int i = xsize - lrg;
//		boolean isMNS = true;
//		
//		while(--i >= 0){
//
//			Y = new ArrayList<AcqVariable>(X);
//			Y.remove(i);
//
//			if(!containsSub){
//				
//				//Updating timers and recording query size and time if needed
//				queryUpdate(ysize, true);
//				
//				if(learner.classify(eProj,complexQ_flag)){
//					consPosQ++;
//					B.removeAll(ke);
//					bigKappa.removeAll(ke);
//					System.out.println("Removed (with positive example : "+eProj+") : "+ke);
//
//					lrg++;
//					continue;
//				}
//			}
//			
//			isMNS = false;
//			S.add(Y);
//			L.add(lrg);
//			lrg++;
//		}
//		
//		if(! isMNS){
//			i = 0;
//			for(List<AcqVariable> sub : S){
//				System.out.println(sub);
//				findMNSes2BreadthFirst(e, sub, L.get(i));
//			}
//			
//		//X is an MNS
//		}else{
//
//			MNSes.add(0, new ArrayList<AcqVariable>(X));
//			//System.out.println("Added : " + e.getProjection(X));
//			
//			double now;
//			
//			if(noFindCQueries){
//				now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
//				timeRemoved -= now;
//			}
//
//			partialEx = false;
//			AcqConstraint c_delta = findCstr(X,e);
//			
//			System.out.println("->  " + c_delta + " has been found with "+e.getProjection(X));
//			
//			if(c_delta!=null)
//				if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
//				{
//					ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
//					ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
//					checkNT1.addAll(NT);
//					checkNT2.addAll(NT);
//					checkNT1.add(c_delta);
//					checkNT2.add(c_delta.negation());
//					
//					if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2)) NL.add(c_delta);
//					
//					else{
//						System.out.println("NL:"+NL);
//						System.out.println("=>>>>>> delta"+c_delta);
//						System.out.println("scope"+X);
//						
//					}
//					System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
//					//B.removeAll(delta);
//					///END ZEBRA
//					if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
//					if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
//
//
//					//NL.add(c_delta);
//					bigKappa.remove(c_delta);
//					B.remove(c_delta);							// Nadjib: here an inscruction commented before=> find why
//					logger.debug("REMOVED FROM B==>  "+c_delta);
//				}
//				else{
//					if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
//					if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
//					NL.add(c_delta);
//				}
//			else
//			{
//				System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
//				Toolkit.getDefaultToolkit().beep();
//				
//				//FIXME: It's dangerous here ! Uncomment me !
//				//throw new RuntimeException("Collapsed");
//			}
//
//			now = (ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0);
//			if(queryTimeOut == 1) lastQRecorded = now;
//			if (MNSTimeOut == 1) lastMNSRecorded = now;
//			
//			if(noFindCQueries) timeRemoved += now;
//		}
//	}
//	
	

	
	private void findMNSes2BreadthFirst(Example e, List<AcqVariable> X, int lrg, boolean cbMNS){
		
		List<List<AcqVariable>> S = new ArrayList<List<AcqVariable>>();
		List<Integer> L = new ArrayList<Integer>();
		List<AcqVariable> Y;
		List<Boolean> BB = new ArrayList<Boolean>();
		
		int ysize;
		boolean containsSub;

		Example eProj;
		Collection<AcqConstraint> ke;
		
		boolean isMNS = (cbMNS && (!containsSubset(MNSes, X)));
		
		int xsize = X.size();
		int i  = xsize - lrg;
		
		while(--i >= 0){
			
			checkTimeOut();

			Y = new ArrayList<AcqVariable>(X);
			Y.remove(i);
			
//			if(MNSes.contains(Y)){
//				isMNS = false;
//				lrg++;
//				continue;
//			}
			
			eProj = e.getProjection(Y);
			ke=getKappaProj(Y);
			
			if(ke.isEmpty()){
				//System.out.println("ke empty");
				lrg++;
				continue;
			}

			ysize = Y.size();
			containsSub = ((ysize == 1)? false : containsSubset(MNSes, Y));

			if(!containsSub){
				
				//Updating timers and recording query size and time if needed
				queryUpdate(ysize, true);
				
				if(learner.classify(eProj,complexQ_flag)){
					B.removeAll(ke);
					bigKappa.removeAll(ke);
					System.out.println("Removed (with positive example : "+eProj+") : "+ke);

					lrg++;
					continue;
				}
			}
			
			isMNS = false;
			S.add(Y);
			L.add(lrg);
			BB.add(!containsSub);
			lrg++;
		}
		
		if(! isMNS){
			i = 0;
			for(List<AcqVariable> sub : S){
				findMNSes2BreadthFirst(e, sub, L.get(i), BB.get(i));
				i++;
			}
			
		//X is an MNS
		}else{

			MNSes.add(0, new ArrayList<AcqVariable>(X));
			//System.out.println("Added : " + e.getProjection(X));
			
			double now;
			
			if(noFindCQueries){
				now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
				timeRemoved -= now;
			}

			partialEx = false;
			AcqConstraint c_delta = findCstr(X,e);
			
			System.out.println("->  " + c_delta + " has been found with "+e.getProjection(X));
			
			if(c_delta!=null)
				if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
				{
					ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
					ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
					checkNT1.addAll(NT);
					checkNT2.addAll(NT);
					checkNT1.add(c_delta);
					checkNT2.add(c_delta.negation());
					
					if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2)) NL.add(c_delta);
					
					else{
						System.out.println("NL:"+NL);
						System.out.println("=>>>>>> delta"+c_delta);
						System.out.println("scope"+X);
						
					}
					System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
					//B.removeAll(delta);
					///END ZEBRA
					if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
					if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);


					//NL.add(c_delta);
					bigKappa.remove(c_delta);
					B.remove(c_delta);							// Nadjib: here an inscruction commented before=> find why
					logger.debug("REMOVED FROM B==>  "+c_delta);
				}
				else{
					if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
					if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
					NL.add(c_delta);
				}
			else
			{
				System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
				Toolkit.getDefaultToolkit().beep();
				
				//FIXME: It's dangerous here ! Uncomment me !
				//throw new RuntimeException("Collapsed");
			}

			now = (ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0);
			if(queryTimeOut == 1) lastQRecorded = now;
			if (MNSTimeOut == 1) lastMNSRecorded = now;
			
			if(noFindCQueries) timeRemoved += now;
		}
	}
	
	
	private void checkTimeOut(){
		double now = ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0;
		//Condition to wait for both : if(MNSes.size() > 0 && ((queryTimeOut == 1 && lastQRecorded + currQueryTimeOutRange < now && ((MNSTimeOut == 1)? (lastMNSRecorded + currMNSTimeOutRange < now) : true)) || (MNSTimeOut == 1 && lastMNSRecorded + currMNSTimeOutRange < now && ((queryTimeOut == 1)? (lastQRecorded + currQueryTimeOutRange < now) : true)))){
		//Condition to wait for the first : 
		if(MNSes.size() > 0 && ((queryTimeOut > 0 && lastQRecorded + currQueryTimeOutRange < now && (queryTimeOut == 1 || consPosQ >= consPosQLimit)) || (MNSTimeOut == 1 && lastMNSRecorded + currMNSTimeOutRange < now))){
			System.err.println("Timed out at " + now + " (query : " + (now - lastQRecorded) + " - MNS : " + (now - lastMNSRecorded) + ")");
			if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
			
			if(variantVersion > 0){
				if(variantVersion == 1){
					currQueryTimeOutRange *= variantCoeff;
					currMNSTimeOutRange *= variantCoeff;
				}else if(variantVersion == 2){
					currQueryTimeOutRange = queryTimeOutRange * ((double) luby(2, lubyCpt++));
					currMNSTimeOutRange = MNSTimeOutRange * ((double) luby(2, lubyCpt++));
				}
				System.err.println("New timeout ranges - query : " + currQueryTimeOutRange + " - MNS : " + currMNSTimeOutRange);
			}
			
			nbTimeOut++;
			
			throw new TimeoutException();
		}
	}
	
	//hitSets receive the minimal hitting sets of the set M
	public List<List<AcqVariable>> getHittingSets(List<List<AcqVariable>> hitSets, List<List<AcqVariable>> M, List<AcqVariable> HS){
		
		if(M.isEmpty()){
			List<AcqVariable> HSclone = new ArrayList<AcqVariable>(HS);
			hitSets.add(HSclone);
			return hitSets;
		}
		
		List<AcqVariable> m = M.get(0);
		M.remove(0);

		List<List<AcqVariable>> Mprime;
		List<List<AcqVariable>> newM;
		
		for(AcqVariable x : m){
			Mprime = new ArrayList<List<AcqVariable>>();
			newM = new ArrayList<List<AcqVariable>>();
			
			for(List<AcqVariable> mi : M){
				if(!mi.contains(x)) Mprime.add(new ArrayList<AcqVariable>(mi));
			}
			HS.add(x);
			getHittingSets(hitSets, Mprime, new ArrayList<AcqVariable>(HS));
			HS.remove(x);
			
			for(List<AcqVariable> mi : M){
				mi.remove(x);
				if(!mi.isEmpty()) newM.add(new ArrayList<AcqVariable>(mi));
			}
			
			M = new ArrayList<List<AcqVariable>>(newM);
		}
		
		return hitSets;
	}
	
	
	/****************************************************************
	 *  FindAllScopesFromBottom procedure
	 * @param e : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @return  variable scopes
	 * 
	 * Author: Robin Arcangioli
	 ****************************************************************/

	private void findAllScopesBottomUp(Example e, List<AcqVariable> X){
		
		partialEx = false;
		bigKappa = getKappaB(e.getProjection(X));
				
		List<AcqVariable> Y;
		ArrayList<List<AcqVariable>> sets = new ArrayList<List<AcqVariable>>();
		for(AcqVariable v : bias.getVariablesList()){
			Y = new ArrayList<AcqVariable>();
			Y.add(v);
			
			sets.add(Y);
		}
		
		while(! sets.isEmpty()){
			
			Y = sets.get(0);
			sets.remove(0);

			//System.out.println("Visited : " + Y);
			
			//nbVisitedNodes++;
			
			//just a test, to comment
			if(MNSes.contains(Y)){
				System.err.println("CRITICAL ERROR : "+ Y + " visited twice !!! FIX IT !!!");
				throw new RuntimeException();
			}else{
				
				int ysize = Y.size();
				boolean containsSub = ((ysize == 1)? false : containsSubset(MNSes, Y));

				if(!containsSub){
					
					Example eProj = e.getProjection(Y);
					Collection<AcqConstraint> ke=getKappaProj(Y);
					
					//Case : Z is not an MNS
					if(ke.isEmpty() || learner.classify(eProj,complexQ_flag)){
						
						if(! ke.isEmpty()) queryUpdate(ysize, false);


						B.removeAll(ke);
						bigKappa.removeAll(ke);
						
						int i = bias.getVariablesList().indexOf(Y.get(ysize - 1));
						int nVars = bias.getVariablesList().size();
						ArrayList<AcqVariable> Z;
						for(++i; i < nVars; i++){
							//Compute strict and ordered supersets
							Z = new ArrayList<AcqVariable>(Y);
							Z.add(bias.getVariablesList().get(i));
							
							sets.add(Z);
						}
						
					//If it is an MNS
					}else{
						queryUpdate(ysize, false);

						List<AcqVariable> Yclone = new ArrayList<AcqVariable>(Y);
						MNSes.add(Yclone);
						//System.out.println("Added : " + e.getProjection(Y));

						partialEx = false;
						AcqConstraint c_delta = findCstr(Y,e);
						
						System.out.println("->  " + c_delta + " has been found with "+eProj);
						
						if(c_delta!=null)
							if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
							{
								ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
								ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
								checkNT1.addAll(NT);
								checkNT2.addAll(NT);
								checkNT1.add(c_delta);
								checkNT2.add(c_delta.negation());
								
								if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2)) NL.add(c_delta);
								
								else{
									System.out.println("NL:"+NL);
									System.out.println("=>>>>>> delta"+c_delta);
									System.out.println("scope"+Y);
								}
								System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
								//B.removeAll(delta);
								///END ZEBRA
								if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
								if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);


								//NL.add(c_delta);
								B.remove(c_delta);							// Nadjib: here an inscruction commented before=> find why
								bigKappa.remove(c_delta);
								logger.debug("REMOVED FROM B==>  "+c_delta);
							}
							else{
								if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
								if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
								NL.add(c_delta);
							}
						else
						{
							System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
							Toolkit.getDefaultToolkit().beep();
							
							//FIXME: It's dangerous here ! Uncomment me !
							//throw new RuntimeException("Collapsed");
						}
					}
				}
			}
		}
	}
	
	
	private void findAllScopesBottomUp2(Example e, List<AcqVariable> X){

		partialEx = false;
		bigKappa = getKappaB(e.getProjection(X));

		List<AcqVariable> Z;

		for(AcqVariable x : X){
			Z = new ArrayList<AcqVariable>();
			Z.add(x);

			findMNSesBottomUp2(e, Z);
		}
		
		//System.err.println("Number of recursive calls to FAS : " + nbVisitedNodes);
	}
	
	/****************************************************************
	 *  FindMNSesFromBottom procedure
	 * @param e : a complete negative example
	 * @param X : problem variables (and/or) foreground variables
	 * @return  variable scopes
	 * 
	 * Author: Robin Arcangioli
	 ****************************************************************/

	private void findMNSesBottomUp2(Example e, List<AcqVariable> X){
		
		//System.out.println("Visited : "+ X);
		
		//just a test, to comment
		if(MNSes.contains(X)){
			System.err.println("CRITICAL ERROR : "+ X + " visited twice !!! FIX IT !!!");
			throw new RuntimeException();
		}else{
			
			int xsize = X.size();
			boolean containsSub = ((xsize == 1)? false : containsSubset(MNSes, X));

			if(!containsSub){
				
				Example eProj = e.getProjection(X);
				Collection<AcqConstraint> ke=getKappaProj(X);
				
				//Case : Z is not an MNS
				if(ke.isEmpty() || learner.classify(eProj,complexQ_flag)){
					
					if(! ke.isEmpty()) queryUpdate(xsize, false);


					B.removeAll(ke);
					bigKappa.removeAll(ke);
					
					int first = bias.getVariablesList().indexOf(X.get(0));
					ArrayList<AcqVariable> Z;
					
					for(int i = 0; i < first; i++){
						//Compute strict and ordered supersets
						Z = new ArrayList<AcqVariable>(X);
						Z.add(0, bias.getVariablesList().get(i));
						
						findMNSesBottomUp2(e, Z);
					}
					
				//If it is an MNS
				}else{
					queryUpdate(xsize, false);

					List<AcqVariable> Xclone = new ArrayList<AcqVariable>(X);
					MNSes.add(Xclone);
					//System.out.println("Added : " + e.getProjection(Y));

					partialEx = false;
					AcqConstraint c_delta = findCstr(X,e);
					
					System.out.println("->  " + c_delta + " has been found with "+eProj);
					
					if(c_delta!=null)
						if(checklearning)					// TEST steps: checking if the learned constraint keep correctness
						{
							ArrayList<AcqConstraint> checkNT1= new ArrayList<AcqConstraint>();
							ArrayList<AcqConstraint> checkNT2= new ArrayList<AcqConstraint>();
							checkNT1.addAll(NT);
							checkNT2.addAll(NT);
							checkNT1.add(c_delta);
							checkNT2.add(c_delta.negation());
							
							if(solver.isSoluble(checkNT1)&& !solver.isSoluble(checkNT2)) NL.add(c_delta);
							
							else{
								System.out.println("NL:"+NL);
								System.out.println("=>>>>>> delta"+c_delta);
								System.out.println("scope"+X);
							}
							System.out.println("HHHHHHHHHHHHHHHH   "+c_delta+ "   HHHHHHHHHHHHHHHH");
							//B.removeAll(delta);
							///END ZEBRA
							if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
							if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);


							//NL.add(c_delta);
							B.remove(c_delta);							// Nadjib: here an inscruction commented before=> find why
							bigKappa.remove(c_delta);
							logger.debug("REMOVED FROM B==>  "+c_delta);
						}
						else{
							if(QAexp.RECORDSTD) recordMNSes((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0), learner.getNbQuery());
							if(QAexp.RECORDSFA) recordFAS((ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0) - fasStartTime, learner.getNbQuery() - fasQ);
							NL.add(c_delta);
						}
					else
					{
						System.err.println("<<<<<<<<<<<<   NO CSTR    >>>>>>>>>>>>>>");
						Toolkit.getDefaultToolkit().beep();
						
						//FIXME: It's dangerous here ! Uncomment me !
						//throw new RuntimeException("Collapsed");
					}
				}
			}
		}
	}
	
	
	private void recordTimeAndQ(String filename, double time, int nbQ){
		File file = new File(filename);
		try {

			if (!file.exists()) {

				file.createNewFile();
			}

			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			writer.append(time + " " + nbQ + " " + NL.size() + "\n");
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void recordMNSes(double time, int nbQ){
		recordedTime.add(time);
		recordedQueries.add(nbQ);
	}
	
	private void recordFAS(double time, int nbQ){
		recordTimeAndQ(recordFASFile, time, nbQ);
	}
	
	private void recordFAS_Q(double time, int nbQ){
		recordTimeAndQ(recordFAS_QFile, time, nbQ);
	}
	
	private void queryUpdate(int qsize, boolean before){
		double now = (ManagementFactory.getThreadMXBean().getCurrentThreadCpuTime()/1000000000.0);
		if(queryTimeOut == 1) lastQRecorded = now;
		
		if(QAexp.RECORDSTD) recordSSize(qsize, now - timeRemoved - startTime);
		
		int supplQuery = 0;
		if(before) supplQuery = 1;
		if(QAexp.RECORDSFA) recordFAS_Q(now - timeRemoved - startTime, learner.getNbQuery() + supplQuery - qRemoved);
		
	}
	
	private void recordSSize(int size, double time){
		recordedQSizes.add(size);
		recordedQTimes.add(time);
	}
	
	public void setRecords(ArrayList<Integer> rQ, ArrayList<Double> rT, ArrayList<Integer> rQS){
		recordedQueries = rQ;
		recordedTime = rT;
		recordedQSizes = rQS;
	}
	
	private void setAndprintStandardDeviation(){
		int nbQ = recordedQSizes.size();
		int nbMNS = recordedQueries.size() - 1;
		double averageS = 0, averageQ = 0, averageT = 0, averageTQ = 0;
		double totalTQ = recordedQTimes.get(nbQ - 1);
				
		for(int size : recordedQSizes){
			averageS += size;
		}
		
		averageS /= nbQ;
		averageQ = (double) nbQ/nbMNS;
		averageT = totalTQ/nbMNS;
		averageTQ = totalTQ/nbQ;
		
		
		double currT = 0, precT = 0, currTQ = 0, precTQ = 0;
		int currQ = 0, precQ = 0;
		
		for(int size : recordedQSizes){
			stDevS += Math.pow(size - averageS, 2);
		}
		
		for(int i = 0; i < nbQ; i++){
			currTQ = recordedQTimes.get(i);
			
			if(currTQ < precTQ){
				precTQ = 0;
			}
			
			stDevQT += Math.pow(currTQ - precTQ - averageTQ, 2);
			
			precTQ = currTQ;
		}
		
		for(int i = 0; i <= nbMNS; i++){
			currQ = recordedQueries.get(i);
			currT = recordedTime.get(i);
			
			if(currQ < precQ){
				precQ = 0;
				precT = 0;
			}

			stDevQ += Math.pow(currQ - precQ - averageQ, 2);
			stDevT += Math.pow(currT - precT - averageT, 2);
			
			precQ = currQ;
			precT = currT;
		}
		
		stDevS = Math.sqrt(stDevS/nbQ);
		stDevQ = Math.sqrt(stDevQ/nbMNS);
		stDevT = Math.sqrt(stDevT/nbMNS);
		stDevQT = Math.sqrt(stDevQT/nbQ);
		
		System.out.println("Total queries :"+nbQ+", query average size : " + averageS + ", standard deviation : "+ stDevS);
		System.out.println("Total MNS : "+nbMNS+", queries per MNS : " + averageQ + ", standard deviation : "+ stDevQ);
		System.out.println("Total time : "+totalTQ + ", time per MNS : " + averageT + ", standard deviation : "+ stDevT);
		System.out.println("Time per Query : " + averageTQ + ", standard deviation : "+ stDevQT);
		System.err.println("Number of Time Out : "+nbTimeOut);
	}
	
	public double getStDevQT(){
		return stDevQT;
	}

	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	/*************************************
	 *   tools for the Branch&Ask approach 
	 *************************************/


	private Collection<AcqConstraint> cleanNL(AcqVariable v) {
		Collection<AcqConstraint> toRemove= new ArrayList<AcqConstraint>();

		for(AcqConstraint c: NL){
			if(c.typeTex().equals("DiffX") && c.getVariables()[0]==v)
				toRemove.add(c);
		}

		return toRemove;
	}

	private Collection<AcqConstraint> cleanNL() {
		Collection<AcqConstraint> toRemove= new ArrayList<AcqConstraint>();
		for(AcqVariable v: bias.getVariables())
			for(AcqConstraint c: NL){
				if(c.typeTex().equals("DiffX") && c.getVariables()[0]==v)
					toRemove.add(c);
			}

		return toRemove;
	}
	public boolean reduceDomain(AcqVariable x){
		int min= x.getMin();
		int max=x.getMin();
		if(min!=max)
		{
			x.setMin(min-1);
			return true;
		}
		return false;		// emptySet
	}




	public SortedMap<AcqVariable, ArrayList<Integer>> boundToFD(List<AcqVariable> bound_vars){
		SortedMap<AcqVariable, ArrayList<Integer>> fd_vars = new TreeMap<AcqVariable, ArrayList<Integer>>();

		for(AcqVariable v: bound_vars){
			fd_vars.put(v, unfold_it(v));

		}

		return fd_vars;
	}



	private ArrayList<Integer> unfold_it(AcqVariable v) {

		ArrayList<Integer> fd_vals= new ArrayList<Integer>();

		for(int i = v.getMin(); i<v.getMax()+1; i++)
			fd_vals.add(i);

		return fd_vals;
	}


	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////

	/*************************
	 * INTERSECTION between two sets (resolve a problem with the api retainAll)
	 * 
	 * @author lazaarnadjib
	 ************************/

	public List<AcqConstraint> intersection(List<AcqConstraint> listA,List<AcqConstraint> listB){
		int sizeA=listA.size();
		int sizeB= listB.size();

		List<AcqConstraint> listC= new ArrayList<AcqConstraint>();
		listC.addAll(listA);
		for(int i=0; i<sizeA; i++)
			for(int j=0; j<sizeB; j++)
				if(!listA.get(i).equals(listB.get(j)))
					listC.remove(listA.get(i));

		return listC;
	}


	/*************************
	 * MINUS between two sets 
	 * 
	 * @author lazaarnadjib
	 ************************/

	public List<AcqConstraint> minus(List<AcqConstraint> listA,List<AcqConstraint> listB){

		List<AcqConstraint> results=new ArrayList<AcqConstraint>();

		results.addAll(listA);
		for(AcqConstraint c: listB){
			if(results.contains(c))
			{
				results.remove(c);
				System.out.println(c+ " already learned !!");
			}
		}
		return results;

	}
	private void stateBefore() {
		learner.Nanswer-= learner.NforOneIt;
		learner.Yanswer-= learner.YforOneIt;
		learner.nbQuery-= (learner.NforOneIt+learner.YforOneIt);
		learner.NforOneIt=0;
		learner.YforOneIt=0;
		completeNQuery--;


	}

	private void reset() {
		learner.NforOneIt=0;
		learner.YforOneIt=0;


	}


	private Example checkMemory(Example eBgd) {

		for (Example ex : memory) {
			if(ex.values.equals(eBgd.values))
				return ex;
		}

		return null;
	}
	/****************************************************************
	 *  ELUCIDATE procedure (TESTME)
	 * @param e a complete negative example 
	 * 
	 * Author: RC
	 * @return 
	 * ****************************************************************/

	private List<AcqVariable> elucidate(Example e) {
		//P <- X
		P = new ArrayList<List<AcqVariable>>();
		List<AcqVariable> scope= new ArrayList<AcqVariable>();
		P.add(bias.getVariablesList()); 
		//
		boolean found = false;

		while(!P.isEmpty() && !found) {
			Y = P.get(0); P.remove(0); 					//Y <- pick(P)
			if(P.size() == 0 && Y.size() >=3) {
				int subSize = Y.size() / 3; 			//FIXME gerer le cas � 5
				P.clear();

				Y1 = Y.subList(0, subSize);
				Y2 = Y.subList(subSize,2*subSize);
				Y3 = Y.subList(2*subSize, Y.size());

				P.clear(); 								//TESTME A ton perdu les Y ?
				assert(Y1.size() == subSize);
				assert(Y2.size() == subSize);
				assert(Y3.size() >= subSize);

				Y1UY2 = new ArrayList<AcqVariable>();
				Y1UY2.addAll(Y1);
				Y1UY2.addAll(Y2);
				P.add(Y1UY2);

				Y1UY3 = new ArrayList<AcqVariable>();
				Y1UY3.addAll(Y1);
				Y1UY3.addAll(Y3);
				P.add(Y1UY3);

				Y2UY3 = new ArrayList<AcqVariable>();
				Y2UY3.addAll(Y2);
				Y2UY3.addAll(Y3);
				P.add(Y2UY3);
			}
			else {
				Example eY = e.getProjection(Y);      // e[Y]
				if(printSize)
					printQsize(eY.size());

				queryUpdate(eY.size(), true);
				if(learner.classify(eY,complexQ_flag)) {


					partialEx=false;
					B.removeAll(getKappaB(eY));
				}
				else {
					if (Y.size() > 2) {					// (|Y| != 2)
						P.clear();
						P.add(Y);

					}
					else {
						scope.addAll(Y);
						return scope;
					}
				}
			}
		}
		return scope;

	}


	/****************************************************************
	 *  FindCstr procedure (TESTED)
	 *  RA : New version, deals with small arity constraints
	 * @param y4 a given scope
	 * 
	 * Author: NL
	 * ****************************************************************/

	private AcqConstraint findCstr(Collection<AcqVariable> y4, Example e) {

		boolean valueOfEx;
		logger.debug("SIZE of B at detect==> "+B.size());
		assert y4.size() > 1;
		logger.debug("NL=> FindCstr on scope ="+y4+")");
		bY.clear();
		NLy.clear();
		List<AcqConstraint> nTUnotC = new ArrayList<AcqConstraint>();

		for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
			List<AcqVariable> temp_vars=c.getVariablesList();
			boolean mutex=true;
			for(AcqVariable v: temp_vars)
				if(!y4.contains(v))
					mutex=false;
			if(mutex)
				NLy.add(c);

			if(c.getVariablesList().size()==2 && y4.containsAll(c.getVariablesList()))					// Golomb rulers case
				NLy.add(c);
			else if(y4.containsAll(c.getVariablesList())&& c.getVariablesList().containsAll(y4))			// C subset of Y4 : Golomb rulers  x4 -X2= x2 - x1   scope x4 x2 x3 x1
				NLy.add(c);
		}
		nTUnotC.addAll(NLy);


		Iterator<AcqConstraint> iterator = B.iterator();

		while( iterator.hasNext()) {
			AcqConstraint c= (AcqConstraint) iterator.next();
			//		if (c.getVariablesList().containsAll(y4)&& y4.containsAll(c.getVariablesList())) {					//c \in B[Y]; GOLOMB	

			if (y4.containsAll(c.getVariablesList())) {					

				/*			List<AcqVariable> temp_vars=c.getVariablesList();
			boolean mutex=true;
			for(AcqVariable v: temp_vars)
				if(!y4.contains(v))
					mutex=false;
			if(mutex){*/


				AcqConstraint oppositeC = bias.getOpposite(c);		
				if(remove_redundant && oppositeC!=null){
					nTUnotC.add(oppositeC); 								//begin(temp) le temps du test
					if (!solver.isSoluble(nTUnotC)) {
						logger.debug("REMOVE FROM B: (implied c)" + c);
						redundant++;
						iterator.remove();									//on retire c si c \in B[Y] and Nt |= c
					}
					else
						bY.add(c);

					nTUnotC.remove(oppositeC); 								//end(temp) TEMP on le retire 
				}
				else 
					bY.add(c);
			}

		}

		delta = new ArrayList<AcqConstraint>();
		delta.addAll(bY);

		assert(delta.size()==bY.size());

		if (delta.isEmpty()) {
			//NL: before	throw new RuntimeException("Collapsed");
			AcqConstraint c=null;
			return c;

		}


		partialEx=true;
		List<AcqConstraint> ke2= getKappaB(e);

		delta.retainAll(ke2);




		while(true) {

			int temp=delta.size();

			Example e2 = solver.notAllViolated(NLy, delta,y4);

			assert(delta.size()==temp);

			if (e2 == null) {

				if(delta.size()==0)
					return null;

				logger.debug("Learnt from B==> "+delta.get(0));
				logger.debug("Removed from B==> "+delta.get(0));
				// NL: previously return delta
				/////////
				/*		ArrayList<AcqConstraint> checkNT= new ArrayList<AcqConstraint>();
				checkNT.addAll(NT);
				checkNT.addAll(delta);
				if(solver.isSoluble(checkNT))
				{
				 */			B.removeAll(delta);
							
				 return delta.get(0);
				 /*		}
				else {
					B.removeAll(delta);
					logger.debug("REMOVED FROM B==>  "+delta);
					return new ArrayList<AcqConstraint>();
				}
				  */		/////////
			}
			partialEx=true;
			ke2 = getKappaB(e2);

			if(ke2.size()==0)
				throw new RuntimeException("Collapsed");

			if(with_m){
				Example ex=checkMemory(e2);
				if(ex!=null)
				{
					valueOfEx= ex.isPositif();
					askedQ++;
					if(printMemory)  printMemory(e2,ex,false);

				}
				else
				{
					if(noFindCQueries) qRemoved++;
					else queryUpdate(e2.size(), true);
					
					valueOfEx=learner.classify(e2,false);
					if(printSize)
						printQsize(e2.size());

					memory.add(e2);
				}
			}
			else
			{
				if(noFindCQueries) qRemoved++;
				else queryUpdate(e2.size(), true);
				
				valueOfEx=learner.classify(e2,false);
				if(printSize)
					printQsize(e2.size());

			}

			if (valueOfEx) {

				//System.out.println("partial positive e:"+e2);
				//System.out.println("NL:"+NL.size());
				NL.addAll(modelSeeker(y4,modelSeeker));
				//System.out.println("NL:"+NL.size());

				B.removeAll(ke2);
				logger.debug("REMOVED FROM B==>  "+ke2);
				delta.removeAll(ke2);

			}
			else{
				partialEx = false;
				List<AcqConstraint> kinf = getKappaB(e2);
				boolean doDescend = false;
				int y4size = y4.size();
				for(AcqConstraint c : kinf){
					if(y4size > c.getVariablesList().size()){
						doDescend = true;
						break;
					}
				}
				
				if(doDescend){
					List<AcqVariable> beginScope = new ArrayList<AcqVariable>(y4);
					List<AcqVariable> scope= findScope(e2,beginScope,false, new ArrayList<AcqVariable>());
					
					if(!scope.containsAll(y4)){
						findCstr(scope, e2);
						//System.err.println("Found " + scope + " shorter than " + y4 + " giving us " + cFound);
						
					}else{
						logger.debug("DELTA1="+delta);
						logger.debug("ke    ="+ke2);
						delta.retainAll(ke2);
						//	delta=intersection(delta,ke2);
						logger.debug("DELTA2="+delta);
		
						//	if(restart)
						//		nb_negatives++;
					}
					
				}else{
					
					logger.debug("DELTA1="+delta);
					logger.debug("ke    ="+ke2);
					delta.retainAll(ke2);
					//	delta=intersection(delta,ke2);
					logger.debug("DELTA2="+delta);
	
					//	if(restart)
					//		nb_negatives++;
				}

			}
		}
	}
	
	
	private AcqConstraint findCstrRev(Collection<AcqVariable> y4, Example e) {

		boolean valueOfEx;
		logger.debug("SIZE of B at detect==> "+B.size());
		assert y4.size() > 1;
		logger.debug("NL=> FindCstr on scope ="+y4+")");
		bY.clear();
		NLy.clear();
		List<AcqConstraint> nTUnotC = new ArrayList<AcqConstraint>();

		for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
			List<AcqVariable> temp_vars=c.getVariablesList();
			boolean mutex=true;
			for(AcqVariable v: temp_vars)
				if(!y4.contains(v))
					mutex=false;
			if(mutex)
				NLy.add(c);

			if(c.getVariablesList().size()==2 && y4.containsAll(c.getVariablesList()))					// Golomb rulers case
				NLy.add(c);
			else if(y4.containsAll(c.getVariablesList())&& c.getVariablesList().containsAll(y4))			// C subset of Y4 : Golomb rulers  x4 -X2= x2 - x1   scope x4 x2 x3 x1
				NLy.add(c);
		}
		nTUnotC.addAll(NLy);


		Iterator<AcqConstraint> iterator = B.iterator();

		while( iterator.hasNext()) {
			AcqConstraint c= (AcqConstraint) iterator.next();
			//		if (c.getVariablesList().containsAll(y4)&& y4.containsAll(c.getVariablesList())) {					//c \in B[Y]; GOLOMB	

			if (y4.containsAll(c.getVariablesList())) {					

				/*			List<AcqVariable> temp_vars=c.getVariablesList();
			boolean mutex=true;
			for(AcqVariable v: temp_vars)
				if(!y4.contains(v))
					mutex=false;
			if(mutex){*/


				AcqConstraint oppositeC = bias.getOpposite(c);		
				if(remove_redundant && oppositeC!=null){
					nTUnotC.add(oppositeC); 								//begin(temp) le temps du test
					if (!solver.isSoluble(nTUnotC)) {
						logger.debug("REMOVE FROM B: (implied c)" + c);
						redundant++;
						iterator.remove();									//on retire c si c \in B[Y] and Nt |= c
					}
					else
						bY.add(c);

					nTUnotC.remove(oppositeC); 								//end(temp) TEMP on le retire 
				}
				else 
					bY.add(c);
			}

		}

		delta = new ArrayList<AcqConstraint>();
		delta.addAll(bY);

		assert(delta.size()==bY.size());

		if (delta.isEmpty()) {
			//NL: before	throw new RuntimeException("Collapsed");
			AcqConstraint c=null;
			return c;

		}


		partialEx=true;
		List<AcqConstraint> ke2= getKappaB(e);

		delta.retainAll(ke2);
		
		removeDomainEquivalent(delta);

		deltaInit = new ArrayList<AcqConstraint>(delta);


		while(true) {

			int temp=delta.size();

			Example e2 = solver.notAllViolated(NLy, delta,y4);

			assert(delta.size()==temp);

			if (e2 == null) {

				if(delta.size()==0)
					return null;

				logger.debug("Learnt from B==> "+delta.get(0));
				logger.debug("Removed from B==> "+delta.get(0));
				// NL: previously return delta
				/////////
				/*		ArrayList<AcqConstraint> checkNT= new ArrayList<AcqConstraint>();
				checkNT.addAll(NT);
				checkNT.addAll(delta);
				if(solver.isSoluble(checkNT))
				{
				 */			B.removeAll(delta);
							
				 bigKappa.removeAll(delta);
				 return delta.get(0);
				 /*		}
				else {
					B.removeAll(delta);
					logger.debug("REMOVED FROM B==>  "+delta);
					return new ArrayList<AcqConstraint>();
				}
				  */		/////////
			}
			partialEx=true;
			ke2 = getKappaB(e2);

			if(ke2.size()==0)
				throw new RuntimeException("Collapsed");

			if(with_m){
				Example ex=checkMemory(e2);
				if(ex!=null)
				{
					valueOfEx= ex.isPositif();
					askedQ++;
					if(printMemory)  printMemory(e2,ex,false);

				}
				else
				{
					if(noFindCQueries) qRemoved++;
					else queryUpdate(e2.size(), true);
					
					valueOfEx=learner.classify(e2,false);
					if(printSize)
						printQsize(e2.size());

					memory.add(e2);
				}
			}
			else
			{
				if(noFindCQueries) qRemoved++;
				else queryUpdate(e2.size(), true);
				
				valueOfEx=learner.classify(e2,false);
				if(printSize)
					printQsize(e2.size());

			}

			if (valueOfEx) {

				//System.out.println("partial positive e:"+e2);
				//System.out.println("NL:"+NL.size());
				NL.addAll(modelSeeker(y4,modelSeeker));
				//System.out.println("NL:"+NL.size());

				B.removeAll(ke2);
				bigKappa.removeAll(ke2);
				logger.debug("REMOVED FROM B==>  "+ke2);
				delta.removeAll(ke2);

			}
			else{
				partialEx = false;
				List<AcqConstraint> kinf = getKappaB(e2);
				boolean doDescend = false;
				int y4size = y4.size();
				for(AcqConstraint c : kinf){
					if(y4size > c.getVariablesList().size()){
						doDescend = true;
						break;
					}
				}
				
				if(doDescend){
					List<AcqVariable> beginScope = new ArrayList<AcqVariable>(y4);
					List<AcqVariable> scope= findScopeRev(e2,beginScope,false, new ArrayList<AcqVariable>());
					
					if(!scope.containsAll(y4)){
						findCstrRev(scope, e2);
						//System.err.println("Found " + scope + " shorter than " + y4 + " giving us " + cFound);
						
					}else{
						logger.debug("DELTA1="+delta);
						logger.debug("ke    ="+ke2);
						delta.retainAll(ke2);
						//	delta=intersection(delta,ke2);
						logger.debug("DELTA2="+delta);
		
						//	if(restart)
						//		nb_negatives++;
					}
					
				}else{
					
					logger.debug("DELTA1="+delta);
					logger.debug("ke    ="+ke2);
					delta.retainAll(ke2);
					//	delta=intersection(delta,ke2);
					logger.debug("DELTA2="+delta);
	
					//	if(restart)
					//		nb_negatives++;
				}

			}
		}
	}
	
	
	/****************************************************************
	 *  FindCstr procedure REVISED (but still needs fixes)
	 * @param y4 a given scope
	 * 
	 * Author: NL
	 * ****************************************************************/

	private AcqConstraint advanced_findCstr(Collection<AcqVariable> y4, Example e) {

		boolean valueOfEx;
		logger.debug("SIZE of B at detect==> "+B.size());
		assert y4.size() > 1;
		logger.debug("NL=> FindCstr on scope ="+y4+")");
		bY.clear();
		NLy.clear();
		List<AcqConstraint> nTUnotC = new ArrayList<AcqConstraint>();

		for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
			List<AcqVariable> temp_vars=c.getVariablesList();
			boolean mutex=true;
			for(AcqVariable v: temp_vars)
				if(!y4.contains(v))
					mutex=false;
			if(mutex)
				NLy.add(c);

			if(c.getVariablesList().size()==2 && y4.containsAll(c.getVariablesList()))					// Golomb rulers case
				NLy.add(c);
			else if(y4.containsAll(c.getVariablesList())&& c.getVariablesList().containsAll(y4))			// C subset of Y4 : Golomb rulers  x4 -X2= x2 - x1   scope x4 x2 x3 x1
				NLy.add(c);
		}
		nTUnotC.addAll(NLy);


		Iterator<AcqConstraint> iterator = B.iterator();

		while( iterator.hasNext()) {
			AcqConstraint c= (AcqConstraint) iterator.next();
			//		if (c.getVariablesList().containsAll(y4)&& y4.containsAll(c.getVariablesList())) {					//c \in B[Y]; GOLOMB	

			if (y4.containsAll(c.getVariablesList())) {					

				/*			List<AcqVariable> temp_vars=c.getVariablesList();
			boolean mutex=true;
			for(AcqVariable v: temp_vars)
				if(!y4.contains(v))
					mutex=false;
			if(mutex){*/


				AcqConstraint oppositeC = bias.getOpposite(c);		
				if(remove_redundant && oppositeC!=null){
					nTUnotC.add(oppositeC); 								//begin(temp) le temps du test
					if (!solver.isSoluble(nTUnotC)) {
						logger.debug("REMOVE FROM B: (implied c)" + c);
						redundant++;
						iterator.remove();									//on retire c si c \in B[Y] and Nt |= c
					}
					else
						bY.add(c);

					nTUnotC.remove(oppositeC); 								//end(temp) TEMP on le retire 
				}
				else{
					if(remove_irrelevant){
						nTUnotC.add(c); 
						if (!solver.isSoluble(nTUnotC)){
							iterator.remove();
							
						}else bY.add(c);
						
						nTUnotC.remove(c);
						
					}else bY.add(c);
				}
			}

		}

		deltaR = new ArrayList<AcqConstraint>();
		deltaA = new ArrayList<AcqConstraint>();
		deltaR.addAll(bY);

		assert(deltaR.size()==bY.size());

		if (deltaR.isEmpty()) {
			//NL: before	throw new RuntimeException("Collapsed");
			AcqConstraint c=null;
			return c;

		}


		partialEx=true;
		List<AcqConstraint> ke2= getKappaB(e);

		deltaR.retainAll(ke2);
		deltaA.addAll(bY);
		deltaA.removeAll(deltaR);

		boolean notRefined = true;

		while(true) {
			
//			if(B.size()<1540){
//				System.out.println("Un autre tour with "+deltaR);
//			}

			removeDomainEquivalent(deltaR);
			//remove from deltaA all constraints that intersect none in deltaR
			removeDeltaAIrrelevants();
			
			int temp=deltaR.size();
			
			List<AcqConstraint> ctemp = new ArrayList<AcqConstraint>(NLy);
			ctemp.addAll(deltaA);

			Example e2 = solver.notAllViolated(ctemp, deltaR,y4);

			assert(deltaR.size()==temp);

			if (e2 == null) {

				if(deltaR.size()==1){
					AcqConstraint cFound = deltaR.get(0);
					logger.debug("Learnt from B==> "+cFound);
					logger.debug("Removed from B==> "+cFound);
					
					B.remove(cFound);
		 			if(bigKappa != null) bigKappa.remove(cFound);
					
					return cFound;
					
				}else if(deltaR.size() == 0) return null;
				
				else{
					
					boolean totalImplication = totalImplicationDeltaA();
					//boolean implying = removeDeltaADomainImplyingConstraints();
					
					//if((!domainEquiv) && (!totalImplication)) return null;
					
					if((!totalImplication)/* && (!implying)*/){
						
						//refineDeltaRRev();
						
						boolean doRefine = false;
						
						if(notRefined){
							doRefine = true;
							notRefined = false;
							
						}
//						else{
//							//try to find c in deltaR s.t. opposite(c) is not in deltaA nor deltaR
//							for(AcqConstraint c : deltaR){
//								if(!deltaA.contains(c.negation()) && !deltaR.contains(c.negation())){
//									deltaA.add(c.negation());
//									doRefine = true;
//									break;
//								}
//							}
//						}
						
						//then we refine deltaR or quit
						if(doRefine){
							System.err.println("Refining deltaR...");
							
							refineDeltaR();
							
						//if no replacement was done, then collapse
						}else return null;
					}
				}
				
			}else{
				
				partialEx=true;
				ke2 = getKappaB(e2);
	
				if(ke2.size()==0)
					throw new RuntimeException("Collapsed");
	
				if(with_m){
					Example ex=checkMemory(e2);
					if(ex!=null)
					{
						valueOfEx= ex.isPositif();
						askedQ++;
						if(printMemory)  printMemory(e2,ex,false);
	
					}
					else
					{
						if(noFindCQueries) qRemoved++;
						else queryUpdate(e2.size(), true);
						
						valueOfEx=learner.classify(e2,false);
						if(printSize)
							printQsize(e2.size());
	
						memory.add(e2);
					}
				}
				else
				{
					if(noFindCQueries) qRemoved++;
					else queryUpdate(e2.size(), true);
					
					valueOfEx=learner.classify(e2,false);
					if(printSize)
						printQsize(e2.size());
	
				}
	
				if (valueOfEx) {
	
					System.out.println("partial positive e:"+e2);
					System.out.println("NL:"+NL.size());
					NL.addAll(modelSeeker(y4,modelSeeker));
					System.out.println("NL:"+NL.size());
	
					B.removeAll(ke2);
					if(bigKappa != null) bigKappa.removeAll(ke2);
					logger.debug("REMOVED FROM B==>  "+ke2);
					deltaR.removeAll(ke2);
	
				}
				else{
					partialEx = false;
					List<AcqConstraint> kinf = getKappaB(e2);
					boolean doDescend = false;
					int y4size = y4.size();
					for(AcqConstraint c : kinf){
						if(y4size > c.getVariablesList().size()){
							doDescend = true;
							break;
						}
					}
					
					if(doDescend){
						List<AcqVariable> beginScope = new ArrayList<AcqVariable>(y4);
						List<AcqVariable> scope= findScope(e2,beginScope,false, new ArrayList<AcqVariable>());
						
						if(!scope.containsAll(y4)){
							AcqConstraint cFound = findCstr(scope, e2);
							//System.err.println("Found " + scope + " shorter than " + y4 + " giving us " + cFound);
							NL.add(cFound);
							
						}else{
							logger.debug("DELTA1="+deltaR);
							logger.debug("ke    ="+ke2);
							deltaR.retainAll(ke2);
							//	delta=intersection(delta,ke2);
							logger.debug("DELTA2="+deltaR);
			
							//	if(restart)
							//		nb_negatives++;
						}
						
					}else{
						
						logger.debug("DELTA1="+deltaR);
						logger.debug("ke    ="+ke2);
						deltaR.retainAll(ke2);
						//	delta=intersection(delta,ke2);
						logger.debug("DELTA2="+deltaR);
		
						//	if(restart)
						//		nb_negatives++;
					}
	
				}
			}
		}
	}
	
	
	private List<AcqConstraint> union(Collection<AcqConstraint> A, Collection<AcqConstraint> B){
		List<AcqConstraint> C = new ArrayList<AcqConstraint>(A);
		C.addAll(B);
		return C;
	}
	
	//S'il existe une contrainte de deltaA qui n'est compatible avec NL U aucune contrainte de deltaR, on la supprime
	private void removeDeltaAIrrelevants(){
		List<AcqConstraint> cUdeltaAc = new ArrayList<AcqConstraint>();
		boolean toRemove = false;
		for(int i = 0; i < deltaA.size(); i++){
			toRemove = true;
			cUdeltaAc.addAll(NL);
			cUdeltaAc.add(deltaA.get(i));
			
			for(AcqConstraint c : deltaR){
				cUdeltaAc.add(c);
				if(solver.isSoluble(cUdeltaAc)){
					toRemove = false;
					break;
				}
				cUdeltaAc.remove(c);
			}

			cUdeltaAc.clear();
			
			if(toRemove){
				System.err.println("Removed irrelevant constraint from deltaA : "+deltaA.get(i));
				B.remove(deltaA.get(i));
				if(bigKappa != null) bigKappa.remove(deltaA.get(i));
				deltaA.remove(i);
				i--;
			}
		}
	}
	
	
	private void refineDeltaR(){
		List<AcqConstraint> NLUcc = new ArrayList<AcqConstraint>();
		//boolean existImplied = false;
		
		for(int i = 0; i < deltaR.size(); i++){
			NLUcc.addAll(NL);
			AcqConstraint c = deltaR.get(i);
			NLUcc.add(c);
			
			for(AcqConstraint cPrime : deltaA){
				NLUcc.add(cPrime.negation());
				
				//if c implies cPrime, we refine deltaR
				if(!solver.isSoluble(NLUcc)){
					deltaR.remove(i);
					deltaR.add(cPrime);
					deltaA.remove(cPrime);
					i--;
					
					NLUcc.clear();
					
					break;
				}
				NLUcc.remove(cPrime.negation());
			}
			NLUcc.clear();
		}
	}
	
	
	private boolean refineDeltaRRev(){
		List<AcqConstraint> cnet = new ArrayList<AcqConstraint>();
		for(AcqConstraint cR : deltaR){
			for(AcqConstraint cA1 : deltaA){
				for(AcqConstraint cA2 : deltaA){
					
					cnet.add(cR);
					cnet.add(cA2.negation());
					if(!solver.isSoluble(cnet)){
						cnet.remove(cR);
						cnet.add(cA1);
						if(!solver.isSoluble(cnet)){
							cnet.clear();
							cnet.add(cR.negation());
							cnet.add(cA1.negation());
							cnet.add(cA2);
							if(!solver.isSoluble(cnet)){
								deltaR.remove(cR);
								deltaA.remove(cA1);
								deltaA.remove(cA2);
								deltaR.add(cA2);
								System.err.println("Replaced "+cR+" by "+cA2+" and deleted "+cA1);
								
								return true;
							}
						}
					}
					cnet.clear();
				}
			}
		}
		return false;
	}
	
	
	private void removeDomainEquivalent(List<AcqConstraint> delta){
		AcqConstraint c1, c2;
		List<AcqConstraint> cnet = new ArrayList<AcqConstraint>();
		
		for(int i = 0; i < delta.size()-1; i++){
			c1 = delta.get(i);
			for(int j = i + 1; j < delta.size(); j++){
				c2 = delta.get(j);
				cnet.add(c1);
				cnet.add(c2.negation());
				//cnet.addAll(NL);
				if(!solver.isSoluble(cnet)){
					cnet.clear();
					cnet.add(c1.negation());
					cnet.add(c2);
					//cnet.addAll(NL);
					if(!solver.isSoluble(cnet)){
						B.remove(c2);
						if(bigKappa != null) bigKappa.remove(c2);
						System.err.println("There were domain equivalent constraints in B ! Removed : " + c2 + " and kept : " + c1);
						delta.remove(j);
						j--;
					}
				}
				cnet.clear();
			}
		}
	}
	
	
	private void removeDeltaREquivalent(){		//Remove equivalent constraint in deltaR
		List<AcqConstraint> cnet = new ArrayList<AcqConstraint>();
		for(int i = 0; i < deltaR.size() - 1; i++)
			for(int j = i + 1; j < deltaR.size(); j++){
				cnet.add(deltaR.get(i));
				cnet.add(deltaR.get(j).negation());
				if(!solver.isSoluble(cnet)){
					cnet.clear();
					cnet.add(deltaR.get(i).negation());
					cnet.add(deltaR.get(j));
					if(!solver.isSoluble(cnet)){
						B.remove(deltaR.get(j));
						if(bigKappa != null) bigKappa.remove(deltaR.get(j));
						System.err.println("There were equivalent constraints in B ! Removed : " + deltaR.get(j) + " and kept : " + deltaR.get(i));
						deltaR.remove(j);
						j--;
					}
				}
				cnet.clear();
			}
	}
	
	//supprime de deltaA toute contrainte qui implique NL U toute contrainte de deltaR
	private boolean totalImplicationDeltaA(){
		boolean totalImply = false;
		boolean toRemove;
		Iterator<AcqConstraint> iterator = deltaA.iterator();
		List<AcqConstraint> NLUcc = new ArrayList<AcqConstraint>();

		while(iterator.hasNext()){
			toRemove = true;
			AcqConstraint cA = (AcqConstraint) iterator.next();
			NLUcc.addAll(NL);
			NLUcc.add(cA.negation());
			
			for(AcqConstraint c : deltaR){
				NLUcc.add(c);
				if(solver.isSoluble(NLUcc)){
					toRemove = false;
					break;
				}
				NLUcc.remove(c);
			}
			
			NLUcc.clear();
			
			if(toRemove){
				totalImply = true;
				B.remove(cA);
				if(bigKappa != null) bigKappa.remove(cA);
				System.err.println("Removed total implying constraint from deltaA : "+cA);
				iterator.remove();
			}
		}
		
		return totalImply;
	}
	
	//supprime de deltaA toute contrainte qui implique NL U toute contrainte de deltaA
	private boolean removeDeltaADomainImplyingConstraints(){
		boolean removed = false;
		AcqConstraint c1, c2;
		List<AcqConstraint> cnet = new ArrayList<AcqConstraint>();
		
		for(int i = 0; i < deltaA.size(); i++){
			c1 = deltaA.get(i);
			for(int j = i + 1; j < deltaA.size(); j++){
				c2 = deltaA.get(j);
				cnet.addAll(NL);
				cnet.add(c1);
				cnet.add(c2.negation());
				if(!solver.isSoluble(cnet)){
					System.err.println("Removed domain implying constraint" + c2 + " that was implying " + c1);
					deltaA.remove(j);
					j--;
					removed = true;
				}else{
					cnet.remove(c1);
					cnet.remove(c2.negation());
					cnet.add(c1.negation());
					cnet.add(c2);
					if(!solver.isSoluble(cnet)){
						System.err.println("Removed domain implying constraint" + c1 + " that was implying " + c2);
						deltaA.remove(i);
						removed = true;
						i--;
						cnet.clear();
						break;
					}
				}
				cnet.clear();
			}
		}
		
		return removed;
	}


	/****************************************************************
	 *  FindAllCstr procedure (TESTME)
	 * @param y4 a given scope
	 * 
	 *
	 * Author: NL
	 * ****************************************************************/

	private List<AcqConstraint> FindAllCstr(Collection<AcqVariable> y4) {

		List<AcqConstraint> By= new ArrayList<AcqConstraint>();		// B_Y in algo
		bY.clear();														// B[Y] in algo
		NLy.clear();
		List<AcqConstraint> nTUnotC = new ArrayList<AcqConstraint>();
		nTUnotC.addAll(NL);

		Iterator<AcqConstraint> iterator = B.iterator();

		while( iterator.hasNext()) {
			AcqConstraint c= (AcqConstraint) iterator.next();
			if (y4.containsAll(c.getVariablesList())&& c.getVariablesList().containsAll(y4)) {					//c \in B[Y];
				AcqConstraint oppositeC = bias.getOpposite(c);		
				nTUnotC.add(oppositeC); 								//begin(temp) le temps du test
				if (!solver.isSoluble(nTUnotC)) {
					iterator.remove();									//on retire c si c \in B[Y] and Nt |= c
				}
				else
					bY.add(c);

				nTUnotC.remove(oppositeC); 								//end(temp) TEMP on le retire 
			}

		}
		By.addAll(bY);
		delta = new ArrayList<AcqConstraint>();					// delta <-- emptyset


		if (By.isEmpty()) {
			throw new RuntimeException("Collapsed");
		}

		for(AcqConstraint c : NL){								// projection of the scope Y on constraint of Nt (i.e., Nt[Y])
			if(y4.containsAll(c.getVariablesList())&&c.getVariablesList().containsAll(y4))			
				NLy.add(c);
		}



		while(true) {

			Example e2 = solver.notAllViolated(NLy, By,y4);


			if (e2 == null) {
				if(By.size()==1){
					NLy.addAll(By);
					if(solver.isSoluble(NLy)){
						delta.addAll(By);
						return delta;
					}
					else
						NLy.removeAll(By);
				}

				return delta;
			}


			//			Collection<AcqConstraint> ke2 = bias.getKe(e2);
			partialEx=true;
			Collection<AcqConstraint> ke2 = getKappaB(e2);

			if(ke2.size()==0)
				throw new RuntimeException("Collapsed");

			queryUpdate(e2.size(), true);
			if (learner.classify(e2,complexQ_flag)) {
				if(printSize)
					printQsize(e2.size());

				B.removeAll(ke2);
				logger.debug("REMOVED FROM B==>  "+ke2);
				By.removeAll(ke2);
			}
			else{

				By.retainAll(ke2); 

			}

			if(By.size()==1)
			{
				delta.addAll(By);
				NLy.addAll(By);
				By.clear();
				By.addAll(bY);
				By.removeAll(delta);			

			}
		}


	}
	/**********************************************************************************
	 *luby (restart policy)
	 * 12/2013
	 *
	 * 			
				 Finite subsequences of the Luby-sequence:

				 0: 1
				 1: 1 1 2
				 2: 1 1 2 1 1 2 4
				 3: 1 1 2 1 1 2 4 1 1 2 1 1 2 4 8
				 ...


	 *@author lazaarnadjib 
	 *********************************************************************************/

	static int luby(int y, int x) {

		// Find the finite subsequence that contains index 'x', and the
		// size of that subsequence:
		int size, seq;
		for (size = 1, seq = 0; size < x + 1; seq++, size = 2 * size + 1)
			;

		while (size - 1 != x) {
			size = (size - 1) >> 1;
		seq--;
		x = x % size;
		}

		return (int)Math.pow(y, seq);
	}

	/**********************************************************************************
	 * getKappaB
	 * @param ex
	 * @return
	 **********************************************************************************/

	public List<AcqConstraint> getKappaB(Example ex) {

		List<AcqConstraint> ke = new ArrayList<AcqConstraint>();
		if(partialEx)

			for(AcqConstraint c : bY) {    

				try {
					int[] tuple = ex.getTuple(c.getVariables());
					if (!c.satisfies(tuple)) {
						ke.add(c);     
					}
				} catch (NotInstanciatedException e) {

				}
			}
		else
			for(AcqConstraint c : B) {    

				try {
					int[] tuple = ex.getTuple(c.getVariables());
					if (!c.satisfies(tuple)) {
						ke.add(c);     
					}
				} catch (NotInstanciatedException e) {

				}
			}

		return ke;
	}
	
	
	public  void printNLsize(boolean firstWrite){
		File file = new File(Config.getExpDir()+QAexp.getInstance()+".log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			if(firstWrite)
				writer.write(String.valueOf(NL.size())+"\n");
			else
				writer.append(String.valueOf(NL.size())+"\n");
			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public  void printBsize(boolean firstWrite){
		File file = new File(Config.getExpDir()+QAexp.getInstance()+"_Bsize.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			if(firstWrite)
				writer.write(String.valueOf(B.size())+"\n");
			else
				writer.append(String.valueOf(B.size())+"\n");
			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/*****************************************************************************
	 * peel-It to return an example that violate at lea
	 * 
	 * @param nL
	 * @param b
	 * @return
	 *
	 *@author lazaarnadjib
	 ****************************************************************************/

	private Example peelIt(List<AcqConstraint> A) {
		Example e=null;
		List<AcqConstraint> nL= new ArrayList<AcqConstraint>();
		nL.addAll(A);
		solver.setTimeLimit(120000);	//1800000
		Iterator<AcqConstraint> iterator = B.iterator();

		while( iterator.hasNext()) {
			AcqConstraint c= (AcqConstraint) iterator.next();
			AcqConstraint oppositC=QuickAcq.bias.getOpposite(c);
			//		if(oppositC==null)
			//			continue;
			if(oppositC.toString().contains("NotNeighbour"))
			{
				iterator.remove();
				continue;
			}
			nL.add(oppositC);
			e=solver.buildSolution(nL);
			if(e==null)
			{
				iterator.remove();
				nL.remove(oppositC);	
			}
			else
				return e;


		}

		return null;
	}

	/*****************************************************************
	 * check_singleton:
	 * return true if atleast the domain of one variable is singleton
	 * false otherwise 
	 * 
	 * @param e
	 * @return
	 ****************************************************************/

	private boolean check_singleton(Example e) {
		for (AcqVariable v : e.values.keySet()) 
			if(v.getDomain().length>1)
				return true;

		return false;
	}



	public List<AcqConstraint> getBias(){
		return B;
	}

	public void setBias(List<AcqConstraint> B_inc){
		B.clear();
		B.addAll(B_inc);
	}

	public void setSucces(boolean value){
		this.succes=value;
	}
	/**************************************************************************************
	 * ************************************************************************************
	 * ************************************************************************************
	 *
	 * 									STAT's FUNCTIONS
	 * 
	 * ************************************************************************************
	 * ************************************************************************************
	 * ************************************************************************************
	 */

	public static void printSTAT(){
		File file = new File(Config.getExpDir()+QAexp.getInstance()+"STAT.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));

			writer.append("******************************* STATs ***************************************\n");

			writer.append("NL :"+ NL.size()+"\n"+NL);
			writer.append("\n_____________________________________________________________________________\n");

			writer.append("NT :"+ NT.size()+"\n"+NT);
			writer.append("\n_____________________________________________________________________________\n");

			writer.append("all sol of NL are of NT: "+QAexp.cptest(QuickAcq.NL, QuickAcq.NT)+"\n");
			writer.append("all sol of NT are of NL: "+QAexp.cptest(QuickAcq.NT, QuickAcq.NL));

			writer.append("\n_____________________________________________________________________________\n");
			writer.append("NL minus NT:");
			writer.append(print_minus(NL,NT));
			writer.append("\n_____________________________________________________________________________\n");
			writer.append("NT minus NL:");
			writer.append(print_minus(NT,NL));
			writer.append("\n_____________________________________________________________________________\n");
			writer.append("nb NL solutions:"+solver.getNbSolutions(QuickAcq.NL)+"\n");
			writer.append("nb NT solutions:"+solver.getNbSolutions(QuickAcq.NT));
			System.out.println("\n*******************************************************************************");



			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public  void printScope(List<AcqVariable> scope){
		File file = new File(Config.getExpDir()+QAexp.getInstance()+"SCOPE.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			writer.append(scope.toString()+"\n");

			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public static  void myLog(Object something, String somewhere){
		File file = new File(Config.getExpDir()+"LOG/"+somewhere+".log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			writer.append(something.toString()+"\n");

			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
	public static  void myLog(Object something){
		File file = new File(Config.getExpDir()+"AmyLOG.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			writer.append(something.toString()+"\n");

			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}


	public  void printExample(Example e, boolean learnt, AcqConstraint cst){
		File file = new File(Config.getExpDir()+QAexp.getInstance()+"Examples.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			if(cst!=null)
				writer.append(learnt+ "  "+e+" ==>  "+cst+"\n");
			else
				writer.append(learnt+ "  "+e+"\n");

			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException ex) {
			ex.printStackTrace();
		}

	}


	public  void printQsize(int size){
		File file = new File(Config.getExpDir()+QAexp.getInstance()+"Qsize.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			writer.append(size+"\n");

			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}


	public  void printMemory(Example in, Example out,boolean findscope){
		File file = new File(Config.getExpDir()+"AskedQ.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file, true));
			if(findscope)
				writer.append("FS: ");
			else
				writer.append("FC: ");

			writer.append(in+" is equal to "+out+"\n");

			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public static int getAskedQ(){
		return askedQ;
	}
	public static int getCompletePQuery(){
		return completePQuery;
	}
	public static void setCompletePQuery(int nb){
		completePQuery=nb;
	}

	public static int getCompleteNQuery(){
		return completeNQuery;
	}
	public static void setCompleteNQuery(int nb){
		completeNQuery=nb;
	}

	public double[] getCpuTime() {
		return cpuTime;
	}
	public int getCspSolverCall() {
		return (int) solver.nbEQ;
	}
	public static int getBsize(int i){
		return BiasSize[i];
	}
	public static int getNbIt(){
		return nbIt;
	}
	public static int getNLsize(){
		return NL.size();
	}

	public static boolean getConv(){
		return succes;
	}
	public static int getPeel(){
		return nbpeelit;
	}
	public static heuristic getHeuristic(){
		return H;
	}
	private boolean NtIsinBias() {
		for(AcqConstraint c: learner.constraints){
			if(!B.contains(c))
			{
				System.err.println(c);
				return false;
			}
		}
		return true;
	}

	public static String print_minus(List<AcqConstraint> nL2, List<AcqConstraint> nT2 ) {
		String results="";
		for(AcqConstraint c: nL2){
			if(!nT2.contains(c))
			{
				results+="\n"+c.toString();

			}
		}
		return results;

	}

	private void print_NT_inter_B() {
		for(AcqConstraint c: learner.constraints){
			if(B.contains(c))
			{
				System.err.println(c);

			}
		}

	}

	private void print_NT_minus_B() {
		for(AcqConstraint c: learner.constraints){
			if(!B.contains(c))
			{
				System.err.println(c);

			}
		}

	}

	private void print_NL_minus_NT() {
		for(AcqConstraint c: NL){
			if(!learner.constraints.contains(c))
			{
				System.err.println(c);

			}
		}

	}

	private List<AcqVariable> reverse(List<AcqVariable> list) {
		List<AcqVariable> reversed= new ArrayList<AcqVariable>();
		for(int i=list.size(); i>0; i--)
			reversed.add(list.get(i-1));
		return reversed;
	}

	public static double average(double[] results){
		double sum=results[0];
		for( int i=1; i< results.length; i++)
			if(results[i]<0)
				return -1;
			else
				sum+=results[i];


		return (double) sum/results.length;

	}


	public static double average(int[] results){
		double sum=results[0];
		for( int i=1; i< results.length; i++)
			if(results[i]<0)
				return -1;
			else
				sum+=results[i];


		return sum/results.length;

	}

	/**************************************************************
	 * clean_up B if there is singleton domain variables
	 * @author lazaarnadjib
	 **************************************************************/
	public void clean_up() {
		boolean mutex=false;
		Iterator<AcqConstraint> iterator = B.iterator();

		while( iterator.hasNext()) {
			//nadjib
			AcqConstraint c= iterator.next();
			if(c.getType().equals(Type.EqualXY))
			{		for( AcqVariable v: c.getVariables())
				if(v.getDomain().length>1)
				{
					mutex=true;
					continue;
				}

			if(!mutex){
				System.out.println("removed: "+c);
				iterator.remove();

			}
			mutex=false;
			}
		}
	}

	/**************************************************************
	 * one_rand from min included to max excluded
	 * 
	 * TESTME
	 * @author lazaarnadjib
	 **************************************************************/

	public int one_rand(int min, int max) {
		return (int)(Math.random() * (max-min)) + min;
	}

	/**************************************************************
	 * one_rand from min included to max excluded
	 * 
	 * TESTME
	 * @author lazaarnadjib
	 **************************************************************/

	public int one_rand(int min, int max, SortedMap<AcqVariable, ArrayList<Integer>> c, AcqVariable v) {

		ArrayList<Integer> dom= new ArrayList<Integer>();
		ArrayList<AcqConstraint> c_projection= new ArrayList<AcqConstraint>();

		for(int i=min; i<=max; i++)
			dom.add(i);


		for(int i=0; i<c.get(v).size();i++);
		//			dom.remove((Integer)(c.get(i).getCst()));


		return dom.get((int)(Math.random() * (dom.size())));
	}

	public static boolean getShuffle() {
		// TODO Auto-generated method stub
		return shuffle;
	}

	public static boolean getFixed() {
		// TODO Auto-generated method stub
		return fixedPart;
	}

	public static boolean getrandV() {
		// TODO Auto-generated method stub
		return rand_value;
	}

	public static double getThreshold() {
		// TODO Auto-generated method stub
		return threshold;
	}

	public static int getMS() {
		// TODO Auto-generated method stub
		return cst_MS;
	}
	public static int[] getQperLevel() {
		// TODO Auto-generated method stub
		return queryPerLevel;
	}

	public static ArrayList<Integer> getQperIt() {
		// TODO Auto-generated method stub
		return queryPerIt;
	}

	public static Integer[][] getQperSize() {
		// TODO Auto-generated method stub
		return queryPerSize;
	}


	/**********************************************************
	 * 					GENERALIZATION PART
	 * 
	 * @author LAZAAR
	 * @date 20-04-15
	 *********************************************************/



	/****************************************************************
	 * isNegativeQuery
	 * 
	 * @author ZakDaoudi
	 ****************************************************************/
	private boolean isNegativeQuery(TypeVar T1, TypeVar T2, AcqConstraint cons) {
		CoupleT cpl = new CoupleT(T1, T2, cons);
		for (CoupleT c : negativeQuery)
			if (c.equals(cpl))
				return true;
		return false;
	}

	private boolean isNegativeQuery3(TypeVar T1, TypeVar T2, TypeVar T3,
			AcqConstraint cons) {
		CoupleT3 cpl = new CoupleT3(T1, T2, T3, cons);
		for (CoupleT3 c : negativeQuery3)
			if (c.equals(cpl))
				return true;
		return false;
	}

	private boolean isNegativeQuery4(TypeVar T1, TypeVar T2, TypeVar T3,
			TypeVar T4, AcqConstraint cons) {
		CoupleT4 cpl = new CoupleT4(T1, T2, T3, T4, cons);
		for (CoupleT4 c : negativeQuery4)
			if (c.equals(cpl))
				return true;
		return false;
	}

	/****************************************************************
	 * Generalize procedure
	 * 
	 * @param Delta
	 *            : aside variables
	 * @param T_i
	 *            : type to generalise
	 * @return list of constraint Author: Zak
	 ****************************************************************/
	void afficheLattice(List<TypeVar> lat) {

		for (TypeVar var : lat) {
			// System.out.println(""+var.getName()+" general");
			parcourir(var);
		}
	}

	void parcourir(TypeVar n) {
		if (n != null) {
			n.setVisited(true);
			System.out.println("type " + n.getName());
			if (n.getGeneral() != null)
				for (TypeVar g : n.getGeneral()) {
					if (!g.getVisited())
						parcourir(g);
				}
		}
	}

	// BFS
	public void breadthFirstSearch(TypeVar v, List<TypeVar> extl) {
		Queue<TypeVar> q = new LinkedList<TypeVar>();

		v.setVisited(true);
		q.add(v);

		while (!q.isEmpty()) {
			TypeVar w = (TypeVar) q.poll();
			// Print the node.
			// System.out.print("   "+w.getName());
			extl.add(w);
			if (w.getGeneral() != null)
				for (TypeVar x : w.getGeneral()) {
					if (!x.getVisited()) {
						x.setVisited(true);
						q.add(x);
					}
				}
		}
		// System.out.println();
	}

	// DFS
	void parcourir2(TypeVar n, List<TypeVar> extl) {
		if (n != null) {
			n.setVisited(true);
			extl.add(n);
			if (n.getGeneral() != null)
				for (TypeVar g : n.getGeneral()) {
					if (!g.getVisited())
						parcourir2(g, extl);
				}
		}
	}

	void initialize(TypeVar n) {
		if (n != null) {
			n.setVisited(false);
			if (n.getGeneral() != null)
				for (TypeVar g : n.getGeneral()) {
					initialize(g);
				}
		}
	}

	boolean isSuccesseur(TypeVar ti, TypeVar tj) {
		if (ti.getGeneral() != null)
			for (TypeVar t : ti.getGeneral()) {
				if (t.getName().equals(tj.getName()))
					return true;
			}
		return false;
	}

	ArrayList<List<TypeVar>> cutExtl(List<TypeVar> extl) {
		ArrayList<List<TypeVar>> listExtl = new ArrayList<List<TypeVar>>();
		for (TypeVar t : extl) {
			t.setJump(false);
			// System.out.println(t.getName());
		}

		extl.get(1).setJump(true);

		for (int i = 0; i < extl.size() - 1; i++)
			if (!isSuccesseur(extl.get(i), extl.get(i + 1))) {
				extl.get(i + 1).setJump(true);
			}

		int i = 1;

		while (i < extl.size()) {
			List<TypeVar> extcut = new ArrayList<TypeVar>();
			if (extl.get(i).getJump()) {
				extcut.add(extl.get(i++));

				while (i < extl.size() && !extl.get(i).getJump())
					extcut.add(extl.get(i++));

			}

			listExtl.add(extcut);
			// System.out.println("Je suis là :)"+i+" "+extl.size());
		}

		return listExtl;

	}

	TypeVar getTypeVar(AcqVariable var) {
		for (TypeVar tp : lattice)
			if (tp.getVars()[0].getName().equals(var.getName()))
				return tp;
		return null;

	}

	ArrayList<TypeVar> getAllTypeVarBFS(AcqVariable var) {
		ArrayList<TypeVar> list = new ArrayList<TypeVar>();
		initialize(getTypeVar(var));
		breadthFirstSearch(getTypeVar(var), list);
		return list;
	}

	ArrayList<TypeVar> getAllTypeVarDFS(AcqVariable var) {
		ArrayList<TypeVar> list = new ArrayList<TypeVar>();
		initialize(getTypeVar(var));
		parcourir2(getTypeVar(var), list);
		return list;
	}

	// binaire
	private boolean classify(AcqConstraint cons, TypeVar t_i, TypeVar t_j) {

		System.out.println(t_i.getName() + "  " + t_j.getName());

		boolean flg;
		Type type = cons.getType();
		// System.out.println(type +" "+ t_i.getName() +" "+t_j.getName());

		if (getTypeVar(cons.getVariables()[0]).equals(t_i)
				&& getTypeVar(cons.getVariables()[1]).equals(t_j)
				|| getTypeVar(cons.getVariables()[1]).equals(t_i)
				&& getTypeVar(cons.getVariables()[0]).equals(t_j))
			return true;

		if (isNegativeQuery(t_i, t_j, cons))
			return false;


		//		 if(t_i.getName().equals(t_j.getName()) && t_i.getVars().length==1)
		//		 return false;

		NbComplexQuery++;

		if (isCouvrert(t_i, t_j, t_i, t_j)) {
			for (AcqVariable var1 : t_i.getVars())
				for (AcqVariable var2 : t_j.getVars()) {
					if (!var1.equals(var2)) {
						AcqConstraint cons1 = new AcqConstraint(type,
								var1, var2);
						System.out.println(type+" "+
								var1.getName()+" "+var2.getName());

						flg = false;
						for (AcqConstraint c : NT){

							if (c.getVariables().length==2)
								if (c.getType().equals(cons1.getType())
										&& c.getVariables()[0].getName().equals(
												cons1.getVariables()[0].getName())
												&& c.getVariables()[1].getName().equals(
														cons1.getVariables()[1].getName())
														|| c.getType().equals(cons1.getType())
														&& c.getVariables()[1].getName().equals(
																cons1.getVariables()[0].getName())
																&& c.getVariables()[0].getName().equals(
																		cons1.getVariables()[1].getName())) {
									flg = true;
									System.out.println(type+" "+
											var1.getName()+" "+var2.getName());
								}
						}

						if (!flg) {
							NbComplexQueryN++;
							return false;
						}
					}
				}
		} else
			for (AcqVariable var1 : t_i.getVars())
				for (AcqVariable var2 : t_j.getVars()) {
					AcqConstraint cons1 = new AcqConstraint(type, var1,
							var2);

					flg = false;
					for (AcqConstraint c : NT)
						if (c.getType().equals(cons1.getType())
								&& c.getVariables()[0].getName().equals(
										cons1.getVariables()[0].getName())
										&& c.getVariables()[1].getName().equals(
												cons1.getVariables()[1].getName())
												|| c.getType().equals(cons1.getType())
												&& c.getVariables()[1].getName().equals(
														cons1.getVariables()[0].getName())
														&& c.getVariables()[0].getName().equals(
																cons1.getVariables()[1].getName())) {
							flg = true;
							// System.out.println(type+" "+
							// var1.getName()+" "+var2.getName());
						}
					if (!flg) {
						NbComplexQueryN++;
						return false;
					}
				}

		NbComplexQueryP++;
		return true;
	}

	// ternaire
	private boolean classify3(AcqConstraint cons, TypeVar t_i, TypeVar t_j,
			TypeVar t_k) {

		System.out.println(t_i.getName() + "  " + t_j.getName() + "  "
				+ t_k.getName());

		boolean flg;
		Type type = cons.getType();
		// System.out.println(type +" "+ t_i.getName() +" "+t_j.getName());

		if (getTypeVar(cons.getVariables()[0]).equals(t_i)
				&& getTypeVar(cons.getVariables()[1]).equals(t_j)
				|| getTypeVar(cons.getVariables()[1]).equals(t_i)
				&& getTypeVar(cons.getVariables()[0]).equals(t_j))
			return true;

		if (isNegativeQuery3(t_i, t_j, t_k, cons))
			return false;

		NbComplexQuery++;

		if (t_i.equals(t_j) && t_i.equals(t_k)) {

			for (AcqVariable var1 : t_i.getVars())
				for (AcqVariable var2 : t_j.getVars())
					for (AcqVariable var3 : t_k.getVars()) {
						if (!var1.equals(var2) && !var1.equals(var3)) {
							AcqConstraint cons1 = new AcqConstraint(type,
									var1, var2, var3);
							// System.out.println(type+" "+
							// var1.getName()+" "+var2.getName());

							flg = false;
							for (AcqConstraint c : NT)
								if (c.equals(cons1))
									flg = true;
							// System.out.println(type+" "+
							// var1.getName()+" "+var2.getName());
							if (!flg) {
								NbComplexQueryN++;
								return false;
							}
						}
					}
		} else
			for (AcqVariable var1 : t_i.getVars())
				for (AcqVariable var2 : t_j.getVars())
					for (AcqVariable var3 : t_k.getVars()) {
						if (!var1.equals(var2) && !var1.equals(var3)) {
							AcqConstraint cons1 = new AcqConstraint(type,
									var1, var2, var3);

							flg = false;
							for (AcqConstraint c : NT)
								if (c.equals(cons1)) {
									flg = true;
									// System.out.println(type+" "+
									// var1.getName()+" "+var2.getName());
								}
							if (!flg) {
								NbComplexQueryN++;
								return false;
							}
						}
					}
		NbComplexQueryP++;
		return true;
	}

	/****************************************************************
	 * new generalization approach table based method Author: ZAK
	 ****************************************************************/

	// Cst binaire

	private List<TypeVar> generalisetable2(AcqConstraint delta) {

		ArrayList<CombType> tabofCombin = new ArrayList<CombType>();
		List<TypeVar> L = new ArrayList<TypeVar>();

		// List<TypeVar>list1=getAllTypeVarBFS(delta.getVariables()[0]);
		// List<TypeVar>list2=getAllTypeVarBFS(delta.getVariables()[1]);

		List<TypeVar> list1 = getAllTypeVarDFS(delta.getVariables()[0]);
		List<TypeVar> list2 = getAllTypeVarDFS(delta.getVariables()[1]);

		for (TypeVar ti : list1)
			for (TypeVar tj : list2) {
				TypeVar tab[] = new TypeVar[delta.getVariables().length];
				tab[0] = ti;
				tab[1] = tj;
				CombType ct = new CombType(delta.getType(), tab);
				tabofCombin.add(ct);
				// System.out.println("-->"+
				// ct.getCombTypes()[0].getName()+" "+ct.getCombTypes()[1].getName());
			}

		for (CombType ct : tabofCombin) {
			System.out.println("my tab   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName());
		}

		// + general
		for (int i = 0; i < tabofCombin.size(); i++) {

			System.out.println("my tab "
					+ tabofCombin.get(i).getCombTypes()[0].getName() + " "
					+ tabofCombin.get(i).getCombTypes()[1].getName());
			for (int j = 0; j < tabofCombin.size(); j++)
				if (isCouvrert(tabofCombin.get(i).getCombTypes()[0],
						tabofCombin.get(i).getCombTypes()[1], tabofCombin
						.get(j).getCombTypes()[0], tabofCombin.get(j)
						.getCombTypes()[1])) {
					tabofCombin.get(i).getPlusGeneral().add(tabofCombin.get(j));
					System.out.println("plus gen  "
							+ tabofCombin.get(j).getCombTypes()[0].getName()
							+ " "
							+ tabofCombin.get(j).getCombTypes()[1].getName());

				}

		}

		// - general
		for (int i = 0; i < tabofCombin.size(); i++) {
			System.out.println("my tab - "
					+ tabofCombin.get(i).getCombTypes()[0].getName() + " "
					+ tabofCombin.get(i).getCombTypes()[1].getName());

			for (int j = 0; j < tabofCombin.size(); j++)
				if (isCouvrert(tabofCombin.get(j).getCombTypes()[0],
						tabofCombin.get(j).getCombTypes()[1], tabofCombin
						.get(i).getCombTypes()[0], tabofCombin.get(i)
						.getCombTypes()[1])) {
					tabofCombin.get(i).getMoinsGeneral()
					.add(tabofCombin.get(j));
					System.out.println("moins gen  "
							+ tabofCombin.get(j).getCombTypes()[0].getName()
							+ " "
							+ tabofCombin.get(j).getCombTypes()[1].getName());

				}
		}

		for (CombType ct : tabofCombin) {

			ct.setNbConst(ct.nbDyOfConstraint(NL));
			System.out.println("av --->   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName() + "  "
					+ ct.getNbConst() + " ");
		}

		Collections.sort(tabofCombin, CombType.Comparators.MinVAR);

		for (CombType ct : tabofCombin) {
			System.out.println(" ap --->   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName() + "  "
					+ ct.getNbConst() + " ");
		}

		prinTraceGenralizationQ(null,true,true);

		for (CombType comtab : tabofCombin) {

			CoupleT C = new CoupleT(comtab.getCombTypes()[0],
					comtab.getCombTypes()[1], delta);
			if (!comtab.getQueryasked()) {
				// System.out.println("hhh LLLLL"+
				// comtab.getCombTypes()[0].getName()+" "+comtab.getCombTypes()[1].getName());
				if (isNegativeQuery(C.getT1(), C.getT2(), delta)) {
					System.out.println("hhh forb Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());
					comtab.setQueryasked(true);
					comtab.setReponse(false);
					for (CombType ctab : comtab.getPlusGeneral())
						if (!ctab.getQueryasked()) {
							ctab.setQueryasked(true);
							ctab.setReponse(false);
							CoupleT cb = new CoupleT(ctab.getCombTypes()[0],
									ctab.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
				} else if (classify(delta, comtab.getCombTypes()[0],
						comtab.getCombTypes()[1])) {
					comtab.setQueryasked(true);
					comtab.setReponse(true);
					System.out.println("hhh Class Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());

					for (CombType ctab2 : comtab.getMoinsGeneral())
						if (!ctab2.getQueryasked()) {
							ctab2.setQueryasked(true);
							ctab2.setReponse(true);
						}
					prinTraceGenralizationQ(comtab,true,false);
				} else {
					System.out.println("hhh add to forb Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());
					negativeQuery.add(C);
					comtab.setQueryasked(true);
					comtab.setReponse(false);
					for (CombType ct3 : comtab.getPlusGeneral())
						if (!ct3.getQueryasked()) {
							ct3.setQueryasked(true);
							ct3.setReponse(false);
							System.out.println("hhh add to forb Q"
									+ ct3.getCombTypes()[0].getName() + " "
									+ ct3.getCombTypes()[1].getName());
							CoupleT cb = new CoupleT(ct3.getCombTypes()[0],
									ct3.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
					prinTraceGenralizationQ(comtab,false,false);
				}
			}

			else {
				System.out.println("LLLLL" + comtab.getCombTypes()[0].getName()
						+ " " + comtab.getCombTypes()[1].getName());

				if (comtab.isReponse()) {

					for (CombType ctab2 : comtab.getMoinsGeneral())
						if (!ctab2.getQueryasked()) {
							ctab2.setQueryasked(true);
							ctab2.setReponse(true);
						}
				} else {

					for (CombType ctab3 : comtab.getPlusGeneral())
						if (!ctab3.getQueryasked()) {
							ctab3.setQueryasked(true);
							ctab3.setReponse(false);
							CoupleT cb = new CoupleT(ctab3.getCombTypes()[0],
									ctab3.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
				}

			}

		}

		for (CombType ct : tabofCombin) {
			if (ct.isReponse()) {
				// System.out.println("L --> "+
				// ct.getCombTypes()[0].getName()+" "+ct.getCombTypes()[1].getName());
				// for(int i=0;i<L.size()-1;i+=2)
				// if(isCouvrert( ct.getCombTypes()[0],
				// ct.getCombTypes()[1],L.get(i), L.get(i+1))){
				// add=true;
				// }
				// if(!add){
				// for(int i=0;i<L.size()-1;i+=2)
				// if(isCouvrert(L.get(i), L.get(i+1), ct.getCombTypes()[0],
				// ct.getCombTypes()[1])){
				// System.out.println("L --> "+
				// L.get(i).getName()+" "+L.get(i+1).getName());
				// L.remove(i); L.remove(i);
				// }
				L.add(ct.getCombTypes()[0]);
				L.add(ct.getCombTypes()[1]);
			}

			// }

		}

		List<TypeVar> gMax = new ArrayList<TypeVar>();

		for (int i = 0; i < L.size(); i += 2) {

			System.out.println("L in --> " + L.get(i).getName() + " "
					+ L.get(i + 1).getName());

		}

		for (int i = 0; i < L.size(); i += 2) {
			boolean flg = false;
			for (int j = 0; j < L.size(); j += 2)
				if (!(L.get(i).equals(L.get(j)) && L.get(i + 1).equals(
						L.get(j + 1))))
					if (isCouvrert(L.get(i), L.get(j))
							&& isCouvrert(L.get(i + 1), L.get(j + 1))) {
						System.out.println("L remov--> " + L.get(i).getName()
								+ " " + L.get(i + 1).getName());
						flg = true;
					}

			if (!flg) {
				gMax.add(L.get(i));
				gMax.add(L.get(i + 1));
			}

		}

		for (int i = 0; i < gMax.size() - 1; i += 2)
			System.out.println("L final --> " + gMax.get(i).getName() + " "
					+ gMax.get(i + 1).getName());

		return gMax;
	}

	private List<TypeVar> generalisetable2Incomplete(AcqConstraint delta) {
		int cuttoffNo=1;
		int NbReponseN = 0;
		ArrayList<CombType> tabofCombin = new ArrayList<CombType>();
		List<TypeVar> L = new ArrayList<TypeVar>();

		// List<TypeVar>list1=getAllTypeVarBFS(delta.getVariables()[0]);
		// List<TypeVar>list2=getAllTypeVarBFS(delta.getVariables()[1]);

		List<TypeVar> list1 = getAllTypeVarDFS(delta.getVariables()[0]);
		List<TypeVar> list2 = getAllTypeVarDFS(delta.getVariables()[1]);

		for (TypeVar ti : list1)
			for (TypeVar tj : list2) {
				TypeVar tab[] = new TypeVar[delta.getVariables().length];
				tab[0] = ti;
				tab[1] = tj;
				CombType ct = new CombType(delta.getType(), tab);
				tabofCombin.add(ct);
				// System.out.println("-->"+
				// ct.getCombTypes()[0].getName()+" "+ct.getCombTypes()[1].getName());
			}

		for (CombType ct : tabofCombin) {
			System.out.println("my tab   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName());
		}

		// + general
		for (int i = 0; i < tabofCombin.size(); i++) {

			System.out.println("my tab "
					+ tabofCombin.get(i).getCombTypes()[0].getName() + " "
					+ tabofCombin.get(i).getCombTypes()[1].getName());
			for (int j = 0; j < tabofCombin.size(); j++)
				if (isCouvrert(tabofCombin.get(i).getCombTypes()[0],
						tabofCombin.get(i).getCombTypes()[1], tabofCombin
						.get(j).getCombTypes()[0], tabofCombin.get(j)
						.getCombTypes()[1])) {
					tabofCombin.get(i).getPlusGeneral().add(tabofCombin.get(j));
					System.out.println("plus gen  "
							+ tabofCombin.get(j).getCombTypes()[0].getName()
							+ " "
							+ tabofCombin.get(j).getCombTypes()[1].getName());

				}

		}

		// - general
		for (int i = 0; i < tabofCombin.size(); i++) {
			System.out.println("my tab - "
					+ tabofCombin.get(i).getCombTypes()[0].getName() + " "
					+ tabofCombin.get(i).getCombTypes()[1].getName());

			for (int j = 0; j < tabofCombin.size(); j++)
				if (isCouvrert(tabofCombin.get(j).getCombTypes()[0],
						tabofCombin.get(j).getCombTypes()[1], tabofCombin
						.get(i).getCombTypes()[0], tabofCombin.get(i)
						.getCombTypes()[1])) {
					tabofCombin.get(i).getMoinsGeneral()
					.add(tabofCombin.get(j));
					System.out.println("moins gen  "
							+ tabofCombin.get(j).getCombTypes()[0].getName()
							+ " "
							+ tabofCombin.get(j).getCombTypes()[1].getName());

				}
		}

		for (CombType ct : tabofCombin) {

			ct.setNbConst(ct.nbDyOfConstraint(NL));
			System.out.println("av --->   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName() + "  "
					+ ct.getNbConst() + " ");
		}

		Collections.sort(tabofCombin, CombType.Comparators.MinVAR);

		for (CombType ct : tabofCombin) {
			System.out.println(" ap --->   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName() + "  "
					+ ct.getNbConst() + " ");
		}


		prinTraceGenralizationQ(null,false,true);

		for (CombType comtab : tabofCombin) {

			CoupleT C = new CoupleT(comtab.getCombTypes()[0],
					comtab.getCombTypes()[1], delta);
			if (!comtab.getQueryasked()) {
				// System.out.println("hhh LLLLL"+
				// comtab.getCombTypes()[0].getName()+" "+comtab.getCombTypes()[1].getName());
				if (isNegativeQuery(C.getT1(), C.getT2(), delta)) {
					System.out.println("hhh forb Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());
					comtab.setQueryasked(true);
					comtab.setReponse(false);
					for (CombType ctab : comtab.getPlusGeneral())
						if (!ctab.getQueryasked()) {
							ctab.setQueryasked(true);
							ctab.setReponse(false);
							CoupleT cb = new CoupleT(ctab.getCombTypes()[0],
									ctab.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
				} else if (classify(delta, comtab.getCombTypes()[0],
						comtab.getCombTypes()[1])) {
					NbReponseN = 0;
					comtab.setQueryasked(true);
					comtab.setReponse(true);
					System.out.println("hhh Class Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());

					for (CombType ctab2 : comtab.getMoinsGeneral())
						if (!ctab2.getQueryasked()) {
							ctab2.setQueryasked(true);
							ctab2.setReponse(true);
						}
					prinTraceGenralizationQ(comtab,true,false);
				} else {
					System.out.println("hhh add to forb Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());
					negativeQuery.add(C);
					comtab.setQueryasked(true);
					comtab.setReponse(false);
					for (CombType ct3 : comtab.getPlusGeneral())
						if (!ct3.getQueryasked()) {
							ct3.setQueryasked(true);
							ct3.setReponse(false);
							System.out.println("hhh add to forb Q"
									+ ct3.getCombTypes()[0].getName() + " "
									+ ct3.getCombTypes()[1].getName());
							CoupleT cb = new CoupleT(ct3.getCombTypes()[0],
									ct3.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
					NbReponseN++;
					prinTraceGenralizationQ(comtab,false,false);

					if (NbReponseN == cuttoffNo) {
						for (CombType ct : tabofCombin) {
							if (ct.isReponse()) {
								L.add(ct.getCombTypes()[0]);
								L.add(ct.getCombTypes()[1]);
							}
						}

						List<TypeVar> gMax = new ArrayList<TypeVar>();
						for (int i = 0; i < L.size(); i += 2) {
							boolean flg = false;
							for (int j = 0; j < L.size(); j += 2)
								if (!(L.get(i).equals(L.get(j)) && L.get(i + 1)
										.equals(L.get(j + 1))))
									if (isCouvrert(L.get(i), L.get(j))
											&& isCouvrert(L.get(i + 1),
													L.get(j + 1))) {
										flg = true;
									}
							if (!flg) {
								gMax.add(L.get(i));
								gMax.add(L.get(i + 1));
							}
						}
						for (int i = 0; i < gMax.size() - 1; i += 2)
							System.out.println(" incomp L final --> "
									+ gMax.get(i).getName() + " "
									+ gMax.get(i + 1).getName());

						return gMax;
					}

				}
			}

			else {
				System.out.println("LLLLL" + comtab.getCombTypes()[0].getName()
						+ " " + comtab.getCombTypes()[1].getName());

				if (comtab.isReponse()) {

					for (CombType ctab2 : comtab.getMoinsGeneral())
						if (!ctab2.getQueryasked()) {
							ctab2.setQueryasked(true);
							ctab2.setReponse(true);
						}
				} else {

					for (CombType ctab3 : comtab.getPlusGeneral())
						if (!ctab3.getQueryasked()) {
							ctab3.setQueryasked(true);
							ctab3.setReponse(false);
							CoupleT cb = new CoupleT(ctab3.getCombTypes()[0],
									ctab3.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
				}

			}

		}


		for (CombType ct : tabofCombin) {
			if (ct.isReponse()) {

				L.add(ct.getCombTypes()[0]);
				L.add(ct.getCombTypes()[1]);
			}

		}

		List<TypeVar> gMax = new ArrayList<TypeVar>();

		for (int i = 0; i < L.size(); i += 2) {

			System.out.println("L in --> " + L.get(i).getName() + " "
					+ L.get(i + 1).getName());

		}

		for (int i = 0; i < L.size(); i += 2) {
			boolean flg = false;
			for (int j = 0; j < L.size(); j += 2)
				if (!(L.get(i).equals(L.get(j)) && L.get(i + 1).equals(
						L.get(j + 1))))
					if (isCouvrert(L.get(i), L.get(j))
							&& isCouvrert(L.get(i + 1), L.get(j + 1))) {
						System.out.println("L remov--> " + L.get(i).getName()
								+ " " + L.get(i + 1).getName());
						flg = true;
					}

			if (!flg) {
				gMax.add(L.get(i));
				gMax.add(L.get(i + 1));
			}

		}

		for (int i = 0; i < gMax.size() - 1; i += 2)
			System.out.println("L final --> " + gMax.get(i).getName() + " "
					+ gMax.get(i + 1).getName());

		return gMax;
	}

	private List<TypeVar> generalisetable2Rand(AcqConstraint delta) {

		ArrayList<CombType> tabofCombin = new ArrayList<CombType>();
		List<TypeVar> L = new ArrayList<TypeVar>();

		// List<TypeVar>list1=getAllTypeVarBFS(delta.getVariables()[0]);
		// List<TypeVar>list2=getAllTypeVarBFS(delta.getVariables()[1]);

		List<TypeVar> list1 = getAllTypeVarDFS(delta.getVariables()[0]);
		List<TypeVar> list2 = getAllTypeVarDFS(delta.getVariables()[1]);

		for (TypeVar ti : list1)
			for (TypeVar tj : list2) {
				TypeVar tab[] = new TypeVar[delta.getVariables().length];
				tab[0] = ti;
				tab[1] = tj;
				CombType ct = new CombType(delta.getType(), tab);
				tabofCombin.add(ct);
				// System.out.println("-->"+
				// ct.getCombTypes()[0].getName()+" "+ct.getCombTypes()[1].getName());
			}

		for (CombType ct : tabofCombin) {
			System.out.println("my tab   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName());
		}

		// + general
		for (int i = 0; i < tabofCombin.size(); i++) {

			System.out.println("my tab "
					+ tabofCombin.get(i).getCombTypes()[0].getName() + " "
					+ tabofCombin.get(i).getCombTypes()[1].getName());
			for (int j = 0; j < tabofCombin.size(); j++)
				if (isCouvrert(tabofCombin.get(i).getCombTypes()[0],
						tabofCombin.get(i).getCombTypes()[1], tabofCombin
						.get(j).getCombTypes()[0], tabofCombin.get(j)
						.getCombTypes()[1])) {
					tabofCombin.get(i).getPlusGeneral().add(tabofCombin.get(j));
					System.out.println("plus gen  "
							+ tabofCombin.get(j).getCombTypes()[0].getName()
							+ " "
							+ tabofCombin.get(j).getCombTypes()[1].getName());

				}

		}

		// - general
		for (int i = 0; i < tabofCombin.size(); i++) {
			System.out.println("my tab - "
					+ tabofCombin.get(i).getCombTypes()[0].getName() + " "
					+ tabofCombin.get(i).getCombTypes()[1].getName());

			for (int j = 0; j < tabofCombin.size(); j++)
				if (isCouvrert(tabofCombin.get(j).getCombTypes()[0],
						tabofCombin.get(j).getCombTypes()[1], tabofCombin
						.get(i).getCombTypes()[0], tabofCombin.get(i)
						.getCombTypes()[1])) {
					tabofCombin.get(i).getMoinsGeneral()
					.add(tabofCombin.get(j));
					System.out.println("moins gen  "
							+ tabofCombin.get(j).getCombTypes()[0].getName()
							+ " "
							+ tabofCombin.get(j).getCombTypes()[1].getName());

				}
		}

		Collections.sort(tabofCombin);

		for (CombType ct : tabofCombin) {
			System.out.println(" ap --->   " + ct.getCombTypes()[0].getName()
					+ " " + ct.getCombTypes()[1].getName() + "  "
					+ ct.getNbConst() + " ");
		}

		while (!isEmty(tabofCombin)) {
			int seed = (int) (Math.random() * 25);
			Random r = new Random(seed);
			int i = (int) (r.nextInt(tabofCombin.size()));

			while (tabofCombin.get(i).getQueryasked()) {
				i = (int) (r.nextInt(tabofCombin.size()));
			}
			System.out.println(" ----->>> " + i);
			CombType comtab = tabofCombin.get(i);

			CoupleT C = new CoupleT(comtab.getCombTypes()[0],
					comtab.getCombTypes()[1], delta);
			if (!comtab.getQueryasked()) {
				// System.out.println("hhh LLLLL"+
				// comtab.getCombTypes()[0].getName()+" "+comtab.getCombTypes()[1].getName());
				if (isNegativeQuery(C.getT1(), C.getT2(), delta)) {
					System.out.println("hhh forb Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());
					comtab.setQueryasked(true);
					comtab.setReponse(false);
					for (CombType ctab : comtab.getPlusGeneral())
						if (!ctab.getQueryasked()) {
							ctab.setQueryasked(true);
							ctab.setReponse(false);
							CoupleT cb = new CoupleT(ctab.getCombTypes()[0],
									ctab.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
				} else if (classify(delta, comtab.getCombTypes()[0],
						comtab.getCombTypes()[1])) {
					comtab.setQueryasked(true);
					comtab.setReponse(true);
					System.out.println("hhh Class Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());

					for (CombType ctab2 : comtab.getMoinsGeneral())
						if (!ctab2.getQueryasked()) {
							ctab2.setQueryasked(true);
							ctab2.setReponse(true);
						}
				} else {
					System.out.println("hhh add to forb Q"
							+ comtab.getCombTypes()[0].getName() + " "
							+ comtab.getCombTypes()[1].getName());
					negativeQuery.add(C);
					comtab.setQueryasked(true);
					comtab.setReponse(false);
					for (CombType ct3 : comtab.getPlusGeneral())
						if (!ct3.getQueryasked()) {
							ct3.setQueryasked(true);
							ct3.setReponse(false);
							System.out.println("hhh add to forb Q"
									+ ct3.getCombTypes()[0].getName() + " "
									+ ct3.getCombTypes()[1].getName());
							CoupleT cb = new CoupleT(ct3.getCombTypes()[0],
									ct3.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
				}
			}

			else {
				System.out.println("LLLLL" + comtab.getCombTypes()[0].getName()
						+ " " + comtab.getCombTypes()[1].getName());

				if (comtab.isReponse()) {

					for (CombType ctab2 : comtab.getMoinsGeneral())
						if (!ctab2.getQueryasked()) {
							ctab2.setQueryasked(true);
							ctab2.setReponse(true);
						}
				} else {

					for (CombType ctab3 : comtab.getPlusGeneral())
						if (!ctab3.getQueryasked()) {
							ctab3.setQueryasked(true);
							ctab3.setReponse(false);
							CoupleT cb = new CoupleT(ctab3.getCombTypes()[0],
									ctab3.getCombTypes()[1], delta);
							if (!isNegativeQuery(cb.getT1(), cb.getT2(), delta))
								negativeQuery.add(cb);
						}
				}

			}
		}

		for (CombType ct : tabofCombin) {
			if (ct.isReponse()) {
				L.add(ct.getCombTypes()[0]);
				L.add(ct.getCombTypes()[1]);
			}
		}

		List<TypeVar> gMax = new ArrayList<TypeVar>();

		for (int i = 0; i < L.size(); i += 2) {

			System.out.println("L in --> " + L.get(i).getName() + " "
					+ L.get(i + 1).getName());

		}

		for (int i = 0; i < L.size(); i += 2) {
			boolean flg = false;
			for (int j = 0; j < L.size(); j += 2)
				if (!(L.get(i).equals(L.get(j)) && L.get(i + 1).equals(
						L.get(j + 1))))
					if (isCouvrert(L.get(i), L.get(j))
							&& isCouvrert(L.get(i + 1), L.get(j + 1))) {
						System.out.println("L remov--> " + L.get(i).getName()
								+ " " + L.get(i + 1).getName());
						flg = true;
					}

			if (!flg) {
				gMax.add(L.get(i));
				gMax.add(L.get(i + 1));
			}

		}

		for (int i = 0; i < gMax.size() - 1; i += 2)
			System.out.println("L final --> " + gMax.get(i).getName() + " "
					+ gMax.get(i + 1).getName());

		return gMax;
	}

	private boolean isEmty(ArrayList<CombType> tabofCombin) {

		for (CombType t : tabofCombin) {
			if (!t.getQueryasked())
				return false;
		}
		return true;
	}

	public boolean isCouvrert(TypeVar tv1, TypeVar tv2) {

		for (AcqVariable v : tv1.getVars()) {
			boolean flag = false;
			for (AcqVariable v2 : tv2.getVars())
				if (v.equals(v2))
					flag = true;

			if (!flag)
				return false;

		}

		return true;

	}

	public boolean isCouvrert(TypeVar tv1, TypeVar tv2, TypeVar tv3, TypeVar tv4) {

		boolean flag1 = true;
		boolean flag2 = true;

		for (AcqVariable v : tv1.getVars()) {
			boolean flag = false;
			for (AcqVariable v2 : tv3.getVars())
				if (v.equals(v2))
					flag = true;

			if (!flag) {
				flag1 = false;
				break;
			}
		}

		for (AcqVariable v : tv2.getVars()) {
			boolean flag = false;
			for (AcqVariable v2 : tv4.getVars())
				if (v.equals(v2))
					flag = true;
			if (!flag) {
				flag2 = false;
				break;
			}
		}

		if (flag1 && flag2)
			return true;
		return false;
	}

	// Cst ternaire

	private List<TypeVar> generalisetable3(AcqConstraint delta) {

		ArrayList<CombType> tabofCombin = new ArrayList<CombType>();
		List<TypeVar> L = new ArrayList<TypeVar>();

		List<TypeVar> list1 = getAllTypeVarBFS(delta.getVariables()[0]);
		List<TypeVar> list2 = getAllTypeVarBFS(delta.getVariables()[1]);
		List<TypeVar> list3 = getAllTypeVarBFS(delta.getVariables()[2]);

		for (TypeVar ti : list1)
			for (TypeVar tj : list2)
				for (TypeVar tk : list3) {
					TypeVar tab[] = new TypeVar[delta.getVariables().length];
					tab[0] = ti;
					tab[1] = tj;
					tab[2] = tk;
					CombType ct = new CombType(delta.getType(), tab);
					tabofCombin.add(ct);
					// System.out.println("-->"+
					// ct.getCombTypes()[0].getName()+" "+ct.getCombTypes()[1].getName());
				}
		for (CombType ct : tabofCombin) {
			System.out.println("L --> " + ct.getCombTypes()[0].getName() + " "
					+ ct.getCombTypes()[1].getName() + " "
					+ ct.getCombTypes()[2].getName());
		}

		// + general
		for (int i = 0; i < tabofCombin.size() - 1; i++) {
			for (int j = i + 1; j < tabofCombin.size(); j++)
				tabofCombin.get(i).getPlusGeneral().add(tabofCombin.get(j));
		}

		// - general
		for (int i = 0; i < tabofCombin.size() - 1; i++) {
			for (int j = i - 1; j >= 0; j--)
				tabofCombin.get(i).getMoinsGeneral().add(tabofCombin.get(j));
		}

		CombType comtab = tabofCombin.get(0);

		for (CombType ct : tabofCombin) {
			System.out.println("L --> " + ct.getCombTypes()[0].getName() + " "
					+ ct.getCombTypes()[1].getName());
			L.add(ct.getCombTypes()[0]);
			L.add(ct.getCombTypes()[1]);
		}

		return L;
	}

	// void h_tri_table(ArrayList<CombType> table){
	// //ArrayList<CombType> tabofCombin=new ArrayList<CombType>();
	// for(int i=0;i<table.size()-1;i++){
	// for(int j=i+1;j<table.size();j++)
	// if(table.get(i).getNbVariables()>table.get(j).getNbVariables()){
	// CombType temp1= new CombType(table.get(i));
	// table.remove(i);
	// CombType temp2= new CombType(table.get(j));
	// table.add(i, temp2);
	// table.add(j, temp1);
	// table.remove(j);
	//
	// }
	//
	// }
	//
	// }



	public void prinTraceGenralizationQ(CombType query,boolean response, boolean newcstQa) {
		File file = new File(Config.getExpDir() + "AskedGenzQ.log");
		try {

			// check whether the file is existed or not
			if (!file.exists()) {

				// create a new file if the file is not existed
				file.createNewFile();
			}

			// new a writer and point the writer to the file
			BufferedWriter writer = new BufferedWriter(new FileWriter(file,
					true));
			if(!newcstQa)
				if (response)
					writer.append(" Ask(DifferentXY,"
							+ query.getCombTypes()[0].getName() + ","
							+ query.getCombTypes()[1].getName() + ")=yes\n");
				else
					writer.append(" Ask(DifferentXY,"
							+ query.getCombTypes()[0].getName() + ","
							+ query.getCombTypes()[1].getName() + ")=no\n");
			else
				writer.append(" This is a new constraint acquired by Quacq\n");

			// writer the content to the file

			// always remember to close the writer
			writer.close();
			writer = null;
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public  static int getNbGQuery() {
		// TODO Auto-generated method stub
		return NbComplexQuery;
	}

	public static  int getNbGQueryN() {
		// TODO Auto-generated method stub
		return NbComplexQueryN;
	}
	public  static int getNbGQueryP() {
		// TODO Auto-generated method stub
		return NbComplexQueryP;
	}
<<<<<<< .mine
	/**********************************************
	 * BCLEAN: Acquisition baseline by asking a recommendation query on all B 
	 * 
	 * 
	 * @date: 25-03-16 (Nancy-MPL)
	 * @author LAZAAR
	 **********************************************/
	public void bclean() {

		int RecQueries=0;
		int Redundant=0;
		Collections.shuffle(B);

		Iterator<AcqConstraint> iterator1 = B.iterator();
		List<AcqConstraint> nTUnotC = new ArrayList<AcqConstraint>();
		System.err.println("Network: "+NT.size());

		while( iterator1.hasNext()) {

			AcqConstraint c= (AcqConstraint) iterator1.next();

		
				if(NT.contains(c))
				{

					myLog("B size init: "+B.size(),"recLog");
					NL.add(c);
					nTUnotC.add(c);
					if(NL.size()-nTUnotC.size()!=0)
					System.out.println ("ERROR");
					B.remove(c.negation());
					myLog("learned: "+ c,"recLog");
					myLog("removed: "+ c.negation(),"recLog");
				}
				else{

					AcqConstraint oppositeC = c.negation();	
					nTUnotC.add(oppositeC); 								

					if ( !solver.isSoluble(nTUnotC)) {
						Redundant++;
						System.err.println(Redundant);
				
					}
					else
						RecQueries++;
					
					nTUnotC.remove(oppositeC); 								//end(temp) TEMP on le retire 

				}
			

			B.remove(c);
			myLog("B size end: "+B.size(),"recLog");
			myLog("rec queries: "+RecQueries,"recLog");
			myLog("red queries: "+Redundant,"recLog");
			myLog("------------------------"+B.size(),"recLog");

			iterator1 = B.iterator();
		}
		
		myLog(RecQueries,"RecQueries");

	}
=======
>>>>>>> .r200

}
